﻿package wbEHPTlib "Electric and Hybrid Power train library Rev March 2017"
  //Symbol to force Dymola to use UTF: €
  //package Propulsion
  extends Modelica.Icons.Package;
  //end Propulsion;

  package MapBased "Contains map-based models of Internal combustion engines and electric drives"
    extends Modelica.Icons.Package;

    class Information
      extends Modelica.Icons.Information;
      annotation (
        Documentation(info="<html>
<p>The map-based folder contains simple model whose only dynamics is due to their mechanical inertia.</p>
<p>The ice model, since implements an Internal Combustion Engine, can just deliver (never absorb) power, while the other two (&QUOT;oneFlange&QUOT; and &QUOT;twoFlange&QUOT;) simulate electric drive trains, i.e. the assembly of an electric machine and the corresponding AC/DC converter, and therefore can absorb or deliver power.</p>
<p>The input torque of the ice model is in Newton-metres, while in the other cases it is normalised: it is between -1 and +1, where -1 means maximum available torque to be absorbed, +1 to be delivered.</p>
<p>Some of the models have a special &QUOT;Conn&QUOT; version that allows interfacing with the exterior by means of an expandable connector.</p>
<p>Note that usage of expandable connectors requires to give special names to the connector&apos;s variables, and therefore the models are more specific than their Modelica.Blocks.Connectors counterparts. Therefore here the models have receives specifi names such as &QUOT;PsdGenConn&QUOT; as a replacement of OneFlange: this is aone flance component to which we added a connector and spfcific signal names.</p>
<p><br><u>Names and meaning </u>of the pre-defined quantities circulating through the connection bus in the model versions having &QUOT;Conn&QUOT; in their names.</p>
<p>The detailed meaning of different quantities are to be read in the corresponding Table in the accompanying document &QUOT;webBookCeraolo&QUOT; in its section PSD-HEV.</p>
</html>"),
        uses(Modelica(version = "3.2.1")));
    end Information;

    model IceT "Simple  map-based ice model with connector"
      import Modelica.Constants.*;
      extends Partial.PartialIce;
      parameter Modelica.SIunits.AngularVelocity wIceStart = 167;
      // rad/s
      Modelica.Blocks.Interfaces.RealInput tauRef "torque request (positive when motor)" annotation (
        Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 90, origin = {-60, -100}), iconTransformation(extent = {{-20, -20}, {20, 20}}, rotation = 90, origin = {-60, -100})));
      Modelica.Blocks.Interfaces.RealOutput fuelCons "Fuel consumption (g/h)" annotation (
        Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = -90, origin = {60, -90})));
      Modelica.Blocks.Nonlinear.Limiter limiter(uMin = 0, uMax = 1e99) annotation (
        Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-60, -16})));
      Modelica.Blocks.Math.Min min annotation (
        Placement(transformation(extent = {{-48, 50}, {-28, 70}})));
    equation
      connect(toG_perHour.y, fuelCons) annotation (
        Line(points = {{30, -61}, {30, -61}, {26, -61}, {60, -61}, {60, -90}}, color = {0, 0, 127}, smooth = Smooth.None));
      connect(limiter.u, tauRef) annotation (
        Line(points = {{-60, -28}, {-60, -100}}, color = {0, 0, 127}, smooth = Smooth.None));
      connect(min.u1, toLimTau.y[1]) annotation (
        Line(points = {{-50, 66}, {-58, 66}, {-61, 66}}, color = {0, 0, 127}));
      connect(min.u2, limiter.y) annotation (
        Line(points = {{-50, 54}, {-60, 54}, {-60, -5}}, color = {0, 0, 127}));
      connect(min.y, Tice.tau) annotation (
        Line(points = {{-27, 60}, {-21.5, 60}, {-14, 60}}, color = {0, 0, 127}));
      annotation (
        Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -80}, {100, 80}})),
        experiment(StopTime = 200, __Dymola_NumberOfIntervals = 1000, __Dymola_Algorithm = "Lsodar"),
        __Dymola_experimentSetupOutput,
        Documentation(info = "<html>
<p>This model belongs to the map-based models of power train components.</p>
<p>It models an Internal Combustion Engine, neglecting any dynamics except that related with its rotor inertia.</p>
<p>The input signal is the torque request (Nm). </p>
<p>The generated torque is the minimum between this signal (negative values are transformed to 0) and the maximum deliverable torque at the actual engine speed, defined by means of a table.</p>
<p>From the generated torque and speed the fuel consumption is computed.</p>
<p>Compare ICE input tau and internal Tice.tau.</p>
</html>"),
        Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = false, initialScale = 0.1, grid = {2, 2}), graphics={  Text(extent = {{-100, -44}, {-22, -72}}, lineColor = {0, 0, 127}, textString = "Nm")}));
    end IceT;

    model IceT01 "Simple  map-based ice model with connector"
      import Modelica.Constants.*;
      extends Partial.PartialIce(toLimTau(table = maxIceTau, tableOnFile = tablesOnFile, tableName = "maxIceTau", fileName = mapsFileName), toSpecCons(tableOnFile = tablesOnFile, fileName = mapsFileName, tableName = "specificCons"));
      parameter Modelica.SIunits.AngularVelocity wIceStart = 167;
      Modelica.Blocks.Interfaces.RealInput nTauRef "normalized torque request" annotation (
        Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 90, origin = {-60, -100}), iconTransformation(extent = {{-20, -20}, {20, 20}}, rotation = 90, origin = {-60, -100})));
      Modelica.Blocks.Interfaces.RealOutput fuelCons "Fuel consumption (g/h)" annotation (
        Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = -90, origin = {60, -90})));
      Modelica.Blocks.Nonlinear.Limiter limiter(uMin = 0, uMax = 1) annotation (
        Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-60, -16})));
      Modelica.Blocks.Math.Product product annotation (
        Placement(transformation(extent = {{-48, 50}, {-28, 70}})));
    equation
      connect(toG_perHour.y, fuelCons) annotation (
        Line(points = {{30, -61}, {30, -62}, {30, -62}, {30, -62}, {30, -70}, {60, -70}, {60, -90}}, color = {0, 0, 127}, smooth = Smooth.None));
      connect(limiter.u, nTauRef) annotation (
        Line(points = {{-60, -28}, {-60, -100}}, color = {0, 0, 127}, smooth = Smooth.None));
      connect(Tice.tau, product.y) annotation (
        Line(points = {{-14, 60}, {-27, 60}}, color = {0, 0, 127}));
      connect(product.u2, limiter.y) annotation (
        Line(points = {{-50, 54}, {-60, 54}, {-60, 52}, {-60, -5}}, color = {0, 0, 127}));
      connect(product.u1, toLimTau.y[1]) annotation (
        Line(points = {{-50, 66}, {-58, 66}, {-58, 66}, {-61, 66}}, color = {0, 0, 127}));
      annotation (
        Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -80}, {100, 80}})),
        experiment(StopTime = 200, __Dymola_NumberOfIntervals = 1000, __Dymola_Algorithm = "Lsodar"),
        __Dymola_experimentSetupOutput,
        Documentation(info="<html>
<p>This model belongs to the map-based models of power train components.</p>
<p>It models an Internal Combustion Engine, neglecting any dynamics except that related with its rotor inertia.</p>
<p>The input signal is a normalised request (0..1). </p>
<p>The generated torque is the product of the maximum deliverable torque at the actual engine speed, defined by means of a table, and the normalised input signal</p>
<p>From the generated torque and speed the fuel consumption is computed.</p>
</html>"),
        Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = false, initialScale = 0.1, grid = {2, 2}), graphics={  Text(extent = {{-100, -40}, {-20, -70}}, lineColor = {0, 0, 127}, textString = "0..1")}));
    end IceT01;

    model OneFlange "Simple map-based model of an electric drive"
      extends Partial.PartialOneFlange;
      Modelica.Blocks.Interfaces.RealInput tauRef "(positive when motor peration)" annotation (
        Placement(visible = true, transformation(origin = {-114, 0}, extent = {{-18, -18}, {18, 18}}, rotation = 0), iconTransformation(origin = {-114, 0}, extent = {{-20, -20}, {20, 20}}, rotation = 0)));
    equation
      connect(tauRef, limTau.tau) annotation (
        Line(points = {{-114, 0}, {-86, 0}, {-86, -66}, {20, -66}, {20, -45.2}, {8.2, -45.2}}, color = {0, 0, 127}));
      annotation (
        Documentation(info="<html>
<p>This is a model that models an electric drive: electornic converter + electric machine.</p>
<p>The only model dynamics is its inertia. </p>
<p>The input signal is a torque request (Nm). The requested torque is applied to a mechanical inertia. </p>
<p>The maximum available torque is internally computed considering a direct torque maximum (tauMax) and a power maximum (powMax) </p>
<p>The model then computes the inner losses and absorbs the total power from the DC input.</p>
<p>Note that to evaluate the inner losses the model uses an efficiency map (i.e. a table), in which torques are ratios of actual torques to tauMax and speeds are ratios of w to wMax. Because of this wMax must be supplied as a parameter.</p>
</html>"),
        Diagram(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = false, initialScale = 0.1)),
        Icon(coordinateSystem(extent = {{-100, -80}, {100, 100}}, preserveAspectRatio = false, initialScale = 0.1)));
    end OneFlange;

    model OneFlange2 "Simple map-based model of an electric drive"
      extends Partial.PartialOneFlange2;
      Modelica.Blocks.Interfaces.RealInput tauRef "(positive when motor peration)" annotation (
        Placement(visible = true, transformation(origin = {0, -112}, extent = {{-18, -18}, {18, 18}}, rotation = 90), iconTransformation(origin = {0, -94}, extent = {{-20, -20}, {20, 20}}, rotation = 90)));
    equation
      connect(tauRef, limTau.tau) annotation (
        Line(points = {{0, -112}, {0, -112}, {0, -64}, {20, -64}, {20, -27.6}, {8, -27.6}}, color = {0, 0, 127}));
      annotation (
        Documentation(info = "<html>
<p>See MBOneFlange for description.</p>
<p>This model differs from that only in the graphical disposition of the interfaces.</p>
</html>"),
        Diagram(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = false, initialScale = 0.1)),
        Icon(coordinateSystem(extent = {{-100, -80}, {100, 100}}, preserveAspectRatio = false, initialScale = 0.1)));
    end OneFlange2;

    model TwoFlange "Simple map-based two-flange electric drive model"
      extends Partial.PartialTwoFlange;
      Modelica.Blocks.Interfaces.RealInput tauRef annotation (
        Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 90, origin = {0, -114}), iconTransformation(extent = {{-20, -20}, {20, 20}}, rotation = 90, origin = {0, -92})));
    equation
      connect(tauRef, limTau.tau) annotation (
        Line(points = {{0, -114}, {0, -114}, {0, -60}, {0, -58}, {-60, -58}, {-60, -9.6}, {-54.2, -9.6}}, color = {0, 0, 127}));
      annotation (
        Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}})),
        Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = false, initialScale = 0.1, grid = {2, 2}), graphics={  Rectangle(fillColor = {192, 192, 192},
                fillPattern =                                                                                                                                                                        FillPattern.HorizontalCylinder, extent = {{-100, 10}, {-66, -10}}), Rectangle(fillColor = {192, 192, 192},
                fillPattern =                                                                                                                                                                                                        FillPattern.HorizontalCylinder, extent = {{66, 8}, {100, -12}}), Rectangle(origin = {-25, 2}, extent = {{-75, 74}, {125, -74}}), Line(origin = {20, -2}, points = {{-60, 94}, {-60, 76}}, color = {0, 0, 255}), Line(origin = {-20, -2}, points = {{60, 94}, {60, 76}}, color = {0, 0, 255})}),
        Documentation(info="<html>
<p>This is a model that models an electric drive: electornic converter + electric machine.</p>
<p>The only model dynamics is its inertia. </p>
<p>The input signal is a torque request (Nm). The requested torque is applied to a mechanical inertia. </p>
<p><span style=\"font-family: MS Shell Dlg 2;\">The model receives from the input connector the torque request and tries to &QUOT;deliver&QUOT; it. Delivering means adding a torque to the system so that the torque exiting from flange B equals the one entering from flange A plus the one requested from the input connector.</span></p>
<p><span style=\"font-family: MS Shell Dlg 2;\">This simulates a two-flange electrical machine, where the added torque is the torque produced by the machine magnetic field stator-rotor interaction.</span></p>
<p><span style=\"font-family: MS Shell Dlg 2;\">However, before delivering the requested torque, the model limits it considering the maximum deliverable torque and power. In addition, it computes and considers inner losses as determined by means of a map. </span></p>
<p><span style=\"font-family: MS Shell Dlg 2;\">The maximum available torque is internally computed considering a direct torque maximum (tauMax) and a power maximum (powMax) </span></p>
<p><span style=\"font-family: MS Shell Dlg 2;\">The requested torque is applied to a mechancal inertia. The inertia is interfaced by means of two flanges with the exterior.</span></p>
<p><span style=\"font-family: MS Shell Dlg 2;\">The model then computes the inner losses and absorbs the total power from the DC input.</span></p>
</html>"));
    end TwoFlange;

    model Genset "GenSet GMS+GEN+SEngine"
      import Modelica.Constants.inf;
      import Modelica.Constants.pi;
      parameter Modelica.SIunits.Time OptiTime "Time parameter of the PI OptiSpeed controller";
      parameter String mapsFileName = "maps.txt" "Name of the file containing data maps (names: maxIceTau, specificCons, optiSpeed)";
      parameter Modelica.SIunits.AngularVelocity wIceStart = 167;
      parameter Modelica.SIunits.AngularVelocity wGenMax;
      Modelica.Mechanics.Rotational.Sensors.SpeedSensor speedSensor annotation (
        Placement(transformation(extent = {{-8, -8}, {8, 8}}, rotation = 180, origin = {-24, -20})));
      Modelica.Mechanics.Rotational.Sensors.PowerSensor IcePow annotation (
        Placement(transformation(extent = {{24, -2}, {42, 16}})));
      Modelica.Blocks.Interfaces.RealInput powRef(unit = "W") "Reference genset power" annotation (
        Placement(transformation(extent = {{15, -15}, {-15, 15}}, rotation = 90, origin = {61, 115})));
      Modelica.Electrical.Analog.Interfaces.PositivePin pin_p annotation (
        Placement(transformation(extent = {{90, 50}, {110, 70}})));
      Modelica.Electrical.Analog.Interfaces.NegativePin pin_n annotation (
        Placement(transformation(extent = {{92, -70}, {112, -50}})));
      Modelica.Blocks.Nonlinear.Limiter limiter(uMax = inf, uMin = 0) annotation (
        Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 90, origin = {-80, 54})));
      ECUs.GMS myGMS(mapsFileName = mapsFileName) annotation (
        Placement(transformation(extent = {{-70, 10}, {-50, 30}})));
      OneFlange gen(wMax = wGenMax, powMax = 1e6, tauMax = 1e4, mapsFileName = mapsFileName, effMapName = "genEffTable") annotation (
        Placement(transformation(extent = {{68, 16}, {48, -4}})));
      IceT01 mBiceT(tablesOnFile = true, mapsFileName = mapsFileName, wIceStart = wIceStart) annotation (
        Placement(transformation(extent = {{-34, -2}, {-14, 18}})));
      Modelica.Blocks.Math.Gain gain(k = -1) annotation (
        Placement(transformation(extent = {{-14, 30}, {6, 50}})));
      Modelica.Blocks.Math.Gain gain1(k = 1) annotation (
        Placement(visible = true, transformation(origin = {-60, -8}, extent = {{-6, -6}, {6, 6}}, rotation = 90)));


      Modelica.Blocks.Continuous.Integrator toGrams(k = 1 / 3600) annotation (
        Placement(transformation(extent = {{18, -42}, {38, -22}})));
    equation
      connect(gain1.u, speedSensor.w) annotation (
        Line(points = {{-60, -15.2}, {-60, -20}, {-32.8, -20}}, color = {0, 0, 127}));
      connect(myGMS.Wmecc, gain1.y) annotation (
        Line(points = {{-59.9, 8.5}, {-60, 8.5}, {-60, -1.4}}, color = {0, 0, 127}));
      connect(limiter.u, powRef) annotation (
        Line(points = {{-80, 66}, {-80, 80}, {61, 80}, {61, 115}}, color = {0, 0, 127}, smooth = Smooth.None));
      connect(limiter.y, myGMS.pRef) annotation (
        Line(points = {{-80, 43}, {-80, 20}, {-72, 20}}, color = {0, 0, 127}, smooth = Smooth.None));
      connect(IcePow.flange_b, gen.flange_a) annotation (
        Line(points = {{42, 7}, {46, 7}, {46, 7.11111}, {48, 7.11111}}, color = {0, 0, 0}));
      connect(gen.pin_n, pin_p) annotation (
        Line(points = {{63.8, -4}, {80, -4}, {80, 60}, {100, 60}}, color = {0, 0, 255}));
      connect(gen.pin_p, pin_n) annotation (
        Line(points = {{54, -4}, {78, -4}, {78, -60}, {102, -60}}, color = {0, 0, 255}));
      connect(mBiceT.nTauRef, myGMS.throttle) annotation (
        Line(points = {{-30, -2}, {-30, -6}, {-49, -6}, {-49, 14}}, color = {0, 0, 127}));
      connect(IcePow.flange_a, mBiceT.flange_a) annotation (
        Line(points = {{24, 7}, {6, 7}, {6, 10}, {-14, 10}}, color = {0, 0, 0}));
      connect(speedSensor.flange, mBiceT.flange_a) annotation (
        Line(points = {{-16, -20}, {-6, -20}, {-6, 10}, {-14, 10}}, color = {0, 0, 0}));
      connect(gain.u, myGMS.tRef) annotation (
        Line(points = {{-16, 40}, {-40, 40}, {-40, 26}, {-49, 26}}, color = {0, 0, 127}));
      connect(gain.y, gen.tauRef) annotation (
        Line(points = {{7, 40}, {69.4, 40}, {69.4, 7.11111}}, color = {0, 0, 127}));
      connect(toGrams.u, mBiceT.fuelCons) annotation (
        Line(points = {{16, -32}, {12, -32}, {8, -32}, {8, -6}, {-18, -6}, {-18, -1}}, color = {0, 0, 127}));
      annotation (
        Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -60}, {100, 100}})),
        experiment(StopTime = 20, Interval = 0.01),
        experimentSetupOutput,
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics={  Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255},
                fillPattern =                                                                                                                                                                                                FillPattern.Solid), Text(extent = {{-98, 94}, {78, 68}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255},
                fillPattern =                                                                                                                                                                                                        FillPattern.Solid, textString = "%name"), Rectangle(fillColor = {192, 192, 192},
                fillPattern =                                                                                                                                                                                                        FillPattern.HorizontalCylinder, extent = {{-20, 0}, {26, -14}}), Rectangle(fillColor = {192, 192, 192},
                fillPattern =                                                                                                                                                                                                        FillPattern.HorizontalCylinder, extent = {{-44, 30}, {-14, -44}}), Line(points = {{-72, 30}, {-72, 6}}), Polygon(points = {{-72, -2}, {-78, 8}, {-66, 8}, {-72, -2}}), Rectangle(extent = {{-96, 38}, {-50, -48}}), Rectangle(fillColor = {95, 95, 95},
                fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-96, -6}, {-50, -24}}), Rectangle(fillColor = {135, 135, 135},
                fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-78, -24}, {-68, -44}}), Polygon(points = {{-72, 34}, {-78, 24}, {-66, 24}, {-72, 34}}), Rectangle(fillColor = {192, 192, 192},
                fillPattern =                                                                                                                                                                                                        FillPattern.HorizontalCylinder, extent = {{6, 30}, {62, -44}}), Line(points = {{94, 60}, {74, 60}, {74, 18}, {62, 18}}, color = {0, 0, 255}), Line(points = {{100, -60}, {74, -60}, {74, -28}, {62, -28}}, color = {0, 0, 255})}),
        Documentation(info = "<html>
<p>Generator set containing Internal Combustion Engine, Electric generator (with DC output), and the related control.</p>
<p>The control logic tends to deliver at the DC port the input power, using the optimal generator speed.</p>
</html>"));
    end Genset;

    model GensetOO "GenSet GMS+GEN+SEngine with On/Off"
      import Modelica.Constants.inf;
      import Modelica.Constants.pi;
      parameter Real gsRatio = 2 "IdealGear speed reduction factor";
      parameter String mapsFileName = "maps.txt" "Name of the file containing data maps (names: maxIceTau, specificCons, optiSpeed)";
      parameter Modelica.SIunits.Torque tauMax = 1e4 "Maximum deliverable torque";
      parameter Modelica.SIunits.AngularVelocity wIceStart = 300;
      Modelica.Mechanics.Rotational.Sensors.SpeedSensor speedSensor annotation (
        Placement(visible = true, transformation(origin = {-26, -40}, extent = {{-8, -8}, {8, 8}}, rotation = 180)));
      Modelica.Mechanics.Rotational.Components.IdealGear idealGear(ratio = gsRatio) annotation (
        Placement(visible = true, transformation(extent = {{0, -18}, {18, 0}}, rotation = 0)));
      Modelica.Blocks.Interfaces.BooleanInput ON "when true engine is ON" annotation (
        Placement(visible = true, transformation(origin = {-55, 69}, extent = {{15, -15}, {-15, 15}}, rotation = 90), iconTransformation(origin = {-60, 116}, extent = {{15, -15}, {-15, 15}}, rotation = 90)));
      Modelica.Mechanics.Rotational.Sensors.PowerSensor IcePow annotation (
        Placement(visible = true, transformation(extent = {{22, -18}, {40, 0}}, rotation = 0)));
      Modelica.Blocks.Interfaces.RealInput powRef(unit = "W") "Reference genset power" annotation (
        Placement(visible = true, transformation(origin = {59, 71}, extent = {{15, -15}, {-15, 15}}, rotation = 90), iconTransformation(extent = {{15, -15}, {-15, 15}}, rotation = 90, origin = {60, 116})));
      Modelica.Electrical.Analog.Interfaces.PositivePin pin_p annotation (
        Placement(visible = true, transformation(extent = {{88, 30}, {108, 50}}, rotation = 0), iconTransformation(extent = {{90, 50}, {110, 70}}, rotation = 0)));
      Modelica.Electrical.Analog.Interfaces.NegativePin pin_n annotation (
        Placement(visible = true, transformation(extent = {{88, -50}, {108, -30}}, rotation = 0), iconTransformation(extent = {{92, -70}, {112, -50}}, rotation = 0)));
      Modelica.Blocks.Nonlinear.Limiter limiter(uMax = inf, uMin = 0) annotation (
        Placement(visible = true, transformation(origin = {-82, 36}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
      ECUs.GMSoo gms(tauMax = tauMax, mapsFileName = mapsFileName, throttlePerWerr = 0.1, tablesOnFile = true) annotation (
        Placement(visible = true, transformation(extent = {{-72, -10}, {-52, 10}}, rotation = 0)));
      wbEHPTlib.MapBased.OneFlange2 gen(powMax = 1e6, tauMax = 1e4, mapsFileName = mapsFileName, effMapName = "genEffTable") annotation (
        Placement(visible = true, transformation(extent = {{68, 2}, {48, -18}}, rotation = 0)));
      IceT01 mbIce(wIceStart = wIceStart, mapsFileName = mapsFileName, iceJ = 10, tablesOnFile = true) annotation (
        Placement(visible = true, transformation(extent = {{-36, -22}, {-16, -2}}, rotation = 0)));
      Modelica.Blocks.Math.Gain revGain(k = -0.9 * gsRatio) annotation (
        Placement(visible = true, transformation(extent = {{-10, 10}, {10, 30}}, rotation = 0)));
      Modelica.Blocks.Continuous.Integrator toGrams(k = 1 / 3600) annotation (
        Placement(transformation(extent = {{18, -48}, {38, -28}})));
    equation
      connect(revGain.y, gen.tauRef) annotation (
        Line(points = {{11, 20}, {58, 20}, {58, 3.55556}}, color = {0, 0, 127}));
      connect(gen.pin_p, pin_n) annotation (
        Line(points={{68,-13.5556},{76,-13.5556},{76,-40},{98,-40}},          color = {0, 0, 255}));
      connect(gen.pin_n, pin_p) annotation (
        Line(points = {{68, -0.444444}, {78, -0.444444}, {78, 40}, {98, 40}}, color = {0, 0, 255}));
      connect(IcePow.flange_b, gen.flange_a) annotation (
        Line(points = {{40, -9}, {44, -9}, {44, -8.88889}, {45.875, -8.88889}, {45.875, -6.88889}, {48, -6.88889}}));
      connect(IcePow.flange_a, idealGear.flange_b) annotation (
        Line(points = {{22, -9}, {18, -9}}));
      connect(revGain.u, gms.tRef) annotation (
        Line(points = {{-12, 20}, {-38, 20}, {-38, 6}, {-51, 6}}, color = {0, 0, 127}));
      connect(mbIce.flange_a, idealGear.flange_a) annotation (
        Line(points = {{-16, -10}, {-12, -10}, {-6, -10}, {-6, -9}, {0, -9}}));
      connect(mbIce.nTauRef, gms.throttle) annotation (
        Line(points = {{-32, -22}, {-32, -26}, {-51, -26}, {-51, -6}}, color = {0, 0, 127}));
      connect(ON, gms.on) annotation (
        Line(points = {{-55, 69}, {-55, 18}, {-73.8, 18}, {-73.8, 6}}, color = {255, 0, 255}));
      connect(limiter.y, gms.pRef) annotation (
        Line(points = {{-82, 25}, {-82, 0}, {-74, 0}}, color = {0, 0, 127}));
      connect(speedSensor.w, gms.Wmecc) annotation (
        Line(points = {{-34.8, -40}, {-61.9, -40}, {-61.9, -11.5}}, color = {0, 0, 127}));
      connect(limiter.u, powRef) annotation (
        Line(points = {{-82, 48}, {-82, 52}, {59, 52}, {59, 71}}, color = {0, 0, 127}));
      connect(speedSensor.flange, idealGear.flange_a) annotation (
        Line(points = {{-18, -40}, {-6, -40}, {-6, -9}, {0, -9}}));
      connect(toGrams.u, mbIce.fuelCons) annotation (
        Line(points = {{16, -38}, {2, -38}, {2, -30}, {-20, -30}, {-20, -21}}, color = {0, 0, 127}));
      annotation (
        Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -60}, {100, 60}})),
        experiment(StopTime = 20, Interval = 0.01),
        experimentSetupOutput,
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics={  Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255},
                fillPattern =                                                                                                                                                                                                FillPattern.Solid), Text(extent = {{-98, 94}, {78, 68}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255},
                fillPattern =                                                                                                                                                                                                        FillPattern.Solid, textString = "%name"), Rectangle(fillColor = {192, 192, 192},
                fillPattern =                                                                                                                                                                                                        FillPattern.HorizontalCylinder, extent = {{-20, 0}, {26, -14}}), Rectangle(fillColor = {192, 192, 192},
                fillPattern =                                                                                                                                                                                                        FillPattern.HorizontalCylinder, extent = {{-44, 30}, {-14, -44}}), Line(points = {{-72, 30}, {-72, 6}}), Polygon(points = {{-72, -2}, {-78, 8}, {-66, 8}, {-72, -2}}), Rectangle(extent = {{-96, 38}, {-50, -48}}), Rectangle(fillColor = {95, 95, 95},
                fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-96, -6}, {-50, -24}}), Rectangle(fillColor = {135, 135, 135},
                fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-78, -24}, {-68, -44}}), Polygon(points = {{-72, 34}, {-78, 24}, {-66, 24}, {-72, 34}}), Rectangle(fillColor = {192, 192, 192},
                fillPattern =                                                                                                                                                                                                        FillPattern.HorizontalCylinder, extent = {{6, 30}, {62, -44}}), Line(points = {{94, 60}, {74, 60}, {74, 18}, {62, 18}}, color = {0, 0, 255}), Line(points = {{100, -60}, {74, -60}, {74, -28}, {62, -28}}, color = {0, 0, 255})}),
        Documentation(info = "<html>
<p>Generator set containing Internal Combustion Engine, Electric generator (with DC output), and the related control.</p>
<p>The control logic tends to deliver at the DC port the input power, using the optimal generator speed.</p>
<p>In addition, it switches ON or OFF depending on the input boolean control signal.</p>
</html>"),
        __OpenModelica_commandLineOptions = "");
    end GensetOO;

    model IceConnP "Simple map-based ice model with connector; follows power request"
      extends Partial.PartialIceP(toGramsPerkWh(fileName = mapsFileName));
      import Modelica.Constants.*;
      parameter String mapsFileName = "maps.txt" "Name of the file containing data maps (name:iceSpecificCons)";
      SupportModels.ConnectorRelated.Conn conn annotation (
        Placement(visible = true, transformation(extent = {{-20, -78}, {20, -118}}, rotation = 0), iconTransformation(extent = {{-20, -78}, {20, -118}}, rotation = 0)));
      Modelica.Blocks.Continuous.Integrator tokgFuel(k = 1 / 3.6e6) annotation (
        Placement(visible = true, transformation(origin = {28, -70}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
      Modelica.Blocks.Math.Product product1 annotation (
        Placement(visible = true, transformation(origin = {28, -40}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
    equation
      connect(tokW.y, product1.u2) annotation (
        Line(points = {{-18, -29}, {-18, -29}, {-18, -36}, {10, -36}, {10, -20}, {22, -20}, {22, -28}, {22, -28}}, color = {0, 0, 127}));
      connect(feedback.u1, conn.icePowRef) annotation (
        Line(points = {{-88, 52}, {-92, 52}, {-92, -98}, {0, -98}}, color = {0, 0, 127}),
        Text(string = "%second", index = 1, extent = {{6, 3}, {6, 3}}));
      connect(Pice.power, conn.icePowDel) annotation (
        Line(points = {{68, 63}, {68, 63}, {68, 6}, {78, 6}, {78, -98}, {0, -98}}, color = {0, 0, 127}),
        Text(string = "%second", index = 1, extent = {{6, 3}, {6, 3}}));
      connect(w.w, conn.iceW) annotation (
        Line(points = {{58, 25}, {58, 25}, {58, 6}, {58, -98}, {0, -98}}, color = {0, 0, 127}),
        Text(string = "%second", index = 1, extent = {{6, 3}, {6, 3}}));
      connect(product1.u1, toGramsPerkWh.y) annotation (
        Line(points = {{34, -28}, {34, -20}, {42, -20}, {42, -13}}, color = {0, 0, 127}));
      connect(tokgFuel.u, product1.y) annotation (
        Line(points = {{28, -58}, {28, -51}, {28, -51}}, color = {0, 0, 127}));
      annotation (
        Diagram(coordinateSystem(preserveAspectRatio = false, initialScale = 0.1, extent = {{-100, -100}, {100, 100}})),
        experiment(StopTime = 200, __Dymola_NumberOfIntervals = 1000, __Dymola_Algorithm = "Lsodar"),
        __Dymola_experimentSetupOutput,
        Documentation(info = "<html>
<p><b><span style=\"font-family: MS Shell Dlg 2;\">Simple map-based ICE model for power- with connector</span></b> </p>
<p><span style=\"font-family: MS Shell Dlg 2;\">This is a version of Ice model that has the following differences from IceT:</span></p>
<p><span style=\"font-family: MS Shell Dlg 2;\">- it follows an input power instead of an input torque</span></p>
<p><span style=\"font-family: MS Shell Dlg 2;\">- signals are exchanged through an expandable connector, instead of input/output connectors</span></p>
<p><span style=\"font-family: MS Shell Dlg 2;\">- it does not make any check about the feasibility of the computed torque.</span></p>
<p>It has an inner feedback control loop aimed at making the ice deliver the power requested ad input. </p>
<p>From the generated torque and speed the fuel consumption is computed.</p>
<p><br><span style=\"font-family: MS Shell Dlg 2;\">Signals connected to the connector:</span></p>
<p><span style=\"font-family: MS Shell Dlg 2;\">- icePowRef (input) is the power request (W). Negative values are internally converted to zero</span></p>
<p><span style=\"font-family: MS Shell Dlg 2;\">- iceW (output) is the measured ICE speed (rad/s)</span></p>
<p><span style=\"font-family: MS Shell Dlg 2;\">- icePowDel (output) delivered power (W)</span></p>
</html>"),
        Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = false, initialScale = 0.1, grid = {2, 2})));
    end IceConnP;

    model IceConnPOO "Simple map-based ice model with connector; follows power request with ON-OFF"
      extends Partial.PartialIceP(toGramsPerkWh(fileName = mapsFileName));
      import Modelica.Constants.*;
      // rad/s
      parameter String mapsFileName = "maps.txt" "Name of the file containing data maps (names: maxIceTau, specificCons, optiSpeed)";
      SupportModels.ConnectorRelated.Conn conn annotation (
        Placement(visible = true, transformation(extent = {{-20, -78}, {20, -118}}, rotation = 0), iconTransformation(extent = {{-20, -78}, {20, -118}}, rotation = 0)));
      Modelica.Blocks.Continuous.Integrator tokgFuel(k = 1 / 3.6e6) annotation (
        Placement(visible = true, transformation(origin = {38, -76}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
      Modelica.Blocks.Logical.Switch switch1 annotation (
        Placement(visible = true, transformation(origin = {2, -46}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Blocks.Sources.Constant zero(k = 0) annotation (
        Placement(visible = true, transformation(extent = {{-46, -74}, {-26, -54}}, rotation = 0)));
      Modelica.Blocks.Math.Product toG_perHour annotation (
        Placement(visible = true, transformation(origin = {38, -42}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
    equation
      connect(toG_perHour.u1, toGramsPerkWh.y) annotation (
        Line(points = {{44, -30}, {42, -30}, {42, -13}, {42, -13}}, color = {0, 0, 127}));
      connect(switch1.y, toG_perHour.u2) annotation (
        Line(points = {{13, -46}, {20, -46}, {20, -22}, {32, -22}, {32, -30}, {32, -30}}, color = {0, 0, 127}));
      connect(toG_perHour.y, tokgFuel.u) annotation (
        Line(points = {{38, -53}, {38, -53}, {38, -64}, {38, -64}}, color = {0, 0, 127}));
      connect(tokW.y, switch1.u1) annotation (
        Line(points = {{-18, -29}, {-18, -29}, {-18, -38}, {-10, -38}, {-10, -38}}, color = {0, 0, 127}));
      connect(switch1.u3, zero.y) annotation (
        Line(points = {{-10, -54}, {-18.5, -54}, {-18.5, -64}, {-25, -64}}, color = {0, 0, 127}));
      connect(switch1.u2, conn.iceON) annotation (
        Line(points = {{-10, -46}, {-60, -46}, {-60, -82}, {0, -82}, {0, -98}}, color = {255, 0, 255}));
      connect(feedback.u1, conn.icePowRef) annotation (
        Line(points = {{-88, 52}, {-88, 52}, {-88, -98}, {0, -98}}, color = {0, 0, 127}),
        Text(string = "%second", index = 1, extent = {{6, 3}, {6, 3}}));
      connect(Pice.power, conn.icePowDel) annotation (
        Line(points = {{68, 63}, {68, 63}, {68, 6}, {78, 6}, {78, -98}, {0, -98}}, color = {0, 0, 127}),
        Text(string = "%second", index = 1, extent = {{6, 3}, {6, 3}}));
      connect(w.w, conn.iceW) annotation (
        Line(points = {{58, 25}, {58, 25}, {58, 6}, {58, -98}, {0, -98}}, color = {0, 0, 127}),
        Text(string = "%second", index = 1, extent = {{6, 3}, {6, 3}}));
      annotation (
        Diagram(coordinateSystem(preserveAspectRatio = false, initialScale = 0.1, extent = {{-100, -100}, {100, 100}})),
        experiment(StopTime = 200, __Dymola_NumberOfIntervals = 1000, __Dymola_Algorithm = "Lsodar"),
        __Dymola_experimentSetupOutput,
        Documentation(info = "<html>
<p><b><span style=\"font-family: MS Shell Dlg 2;\">Simple map-based ICE model for power-split power trains - with connector</span></b> </p>
<p><span style=\"font-family: MS Shell Dlg 2;\">This is an evolution of IceConnP: ON/OFF control is added though an hysteresis block. </span></p>
<p><span style=\"font-family: MS Shell Dlg 2;\">For its general operation see the description of IceConnP.</span></p>
</html>"),
        Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = false, initialScale = 0.1, grid = {2, 2}), graphics={  Text(origin = {34, -1}, lineColor = {255, 255, 255}, extent = {{32, -19}, {-48, 29}}, textString = "OO")}));
    end IceConnPOO;

    model PsdGenConn "Simple map-based one-flange electric drive "
      extends Partial.PartialOneFlange;
      SupportModels.ConnectorRelated.Conn conn annotation (
        Placement(visible = true, transformation(extent = {{-18, -62}, {22, -102}}, rotation = 0), iconTransformation(extent = {{80, -58}, {120, -98}}, rotation = 0)));
    equation
      connect(limTau.y, conn.genTauLim) annotation (
        Line(points = {{-17.1, -38}, {-26, -38}, {-26, -72}, {2, -72}, {2, -82}}, color = {0, 0, 127}, smooth = Smooth.None),
        Text(string = "%second", index = 1, extent = {{6, 3}, {6, 3}}));
      connect(wSensor.w, conn.genW) annotation (
        Line(points = {{78, -4.8}, {78, -72}, {2, -72}, {2, -82}}, color = {0, 0, 127}, smooth = Smooth.None),
        Text(string = "%second", index = 1, extent = {{6, 3}, {6, 3}}));
      connect(limTau.tau, conn.genTauRef) annotation (
        Line(points = {{8.2, -45.2}, {18, -45.2}, {18, -82}, {2, -82}}, color = {0, 0, 127}),
        Text(string = "%second", index = 1, extent = {{6, 3}, {6, 3}}));
      connect(powSensor.power, conn.genPowDel) annotation (
        Line(points = {{60, 21}, {60, -62}, {2, -62}, {2, -82}}, color = {0, 0, 127}),
        Text(string = "%second", index = 1, extent = {{6, 3}, {6, 3}}));
      annotation (
        Diagram(coordinateSystem(preserveAspectRatio = false, initialScale = 0.1, grid = {2, 2})),
        Icon(coordinateSystem(preserveAspectRatio = false, initialScale = 0.1, grid = {2, 2}), graphics={  Line(points = {{62, -7}, {82, -7}})}),
        Documentation(info="<html>
<p>The input signal is interpreted as a <u>normalised</u> torque request (0 means null torque, 1 maximum availabile torque).</p>
<p>The maximum available torque is internally computed considering a direct torque maximum (tauMax) and a power maximum (powMax) </p>
<p>The requested torque is applied to a mechancal inertia. The inertia is interfaced by means ot two flanges with the exterior.</p>
<p>The model then computes the inner losses and absorbs the total power from the DC input.</p>
<p><br><u>Signals connected to the bus connecto</u>r (the names are chosen from the examples FullVehicles.PSecu1 and PSecu2 where the one-flange machine is called &QUOT;gen&QUOT;):</p>
<p>- genTauRef (input) is the torque request (Nm)</p>
<p>- genPowDel (output) is the delivered mechanical power (W)</p>
<p>- genTauLim (output) maximum available torque at the given machine rotational speed (Nm)</p>
</html>"));
    end PsdGenConn;

    model PsdMotConn "Simple map-based two-flange electric drive model"
      extends Partial.PartialTwoFlange;
      SupportModels.ConnectorRelated.Conn conn annotation (
        Placement(visible = true, transformation(extent = {{-112, -58}, {-72, -98}}, rotation = 0), iconTransformation(extent = {{-112, -58}, {-72, -98}}, rotation = 0)));
    equation
      connect(outBPow_.power, conn.motPowDelB) annotation (
        Line(points = {{64, 39}, {64, -78}, {-92, -78}}, color = {0, 0, 127}, smooth = Smooth.None),
        Text(string = "%second", index = 1, extent = {{6, 3}, {6, 3}}));
      connect(speedRing.w, conn.motW) annotation (
        Line(points = {{-80, 29}, {-86, 29}, {-86, 28}, {-92, 28}, {-92, -78}}, color = {0, 0, 127}, smooth = Smooth.None),
        Text(string = "%second", index = 1, extent = {{6, 3}, {6, 3}}));
      connect(add.y, conn.motPowDelAB) annotation (
        Line(points = {{28, -1}, {28, -16}, {-10, -16}, {-10, -78}, {-92, -78}}, color = {0, 0, 127}, smooth = Smooth.None),
        Text(string = "%second", index = 1, extent = {{6, 3}, {6, 3}}));
      connect(limTau.tau, conn.motTauRef) annotation (
        Line(points = {{-54.2, -9.6}, {-54.2, -78}, {-92, -78}}, color = {0, 0, 127}),
        Text(string = "%second", index = 1, extent = {{6, 3}, {6, 3}}));
      connect(tqSens.tau, conn.tauProp) annotation (
        Line(points = {{28, 39}, {28, 39}, {28, 34}, {28, 32}, {82, 32}, {82, -78}, {-92, -78}}, color = {0, 0, 127}),
        Text(string = "%second", index = 1, extent = {{6, 3}, {6, 3}}));
      annotation (
        Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}})),
        Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = false, initialScale = 0.1, grid = {2, 2}), graphics={  Rectangle(fillColor = {192, 192, 192},
                fillPattern =                                                                                                                                                                        FillPattern.HorizontalCylinder, extent = {{-100, 10}, {-66, -10}}), Rectangle(fillColor = {192, 192, 192},
                fillPattern =                                                                                                                                                                                                        FillPattern.HorizontalCylinder, extent = {{66, 8}, {100, -12}}), Rectangle(origin = {-25, 2}, extent = {{-75, 74}, {125, -74}}), Line(origin = {20, -2}, points = {{-60, 94}, {-60, 76}}, color = {0, 0, 255}), Line(origin = {-20, -2}, points = {{60, 94}, {60, 76}}, color = {0, 0, 255})}),
        Documentation(info = "<html>
<p><b><span style=\"font-family: MS Shell Dlg 2;\">Simple map-based ICE model for power-split power trains - with connector</b> </span></p>
<p><span style=\"font-family: MS Shell Dlg 2;\">This is a &QUOT;connector&QUOT; version of MBice.</span></p>
<p><span style=\"font-family: MS Shell Dlg 2;\">For a general descritiption see the info of MBice.</span></p>
<p><span style=\"font-family: MS Shell Dlg 2;\">Signals connected to the connector:</span></p>
<p><span style=\"font-family: MS Shell Dlg 2;\">- icePowRef (input) is the power request (W). Negative values are internally converted to zero</span></p>
<p><span style=\"font-family: MS Shell Dlg 2;\">- iceW (output) is the measured ICE speed (rad/s)</span></p>
<p><span style=\"font-family: MS Shell Dlg 2;\">- icePowDel (output) delivered power (W)</span></p>
</html>"));
    end PsdMotConn;

    package Partial
      partial model PartialOneFlange "Partial map-based one-Flange electric drive model"
        parameter Modelica.SIunits.Power powMax = 22000 "Maximum  power ";
        parameter Modelica.SIunits.Torque tauMax = 80 "Maximum  torque";
        parameter Modelica.SIunits.AngularVelocity wMax(min = powMax / tauMax) = 3000 "Maximum drive speed";
        parameter Modelica.SIunits.MomentOfInertia J = 0.25 "Rotor's moment of inertia";
        parameter String mapsFileName "Name of the txt file where models' maps are stored";
        parameter String effMapName "Name of the efficiency map in mapsFileName";
        //the name is passed because a file ccan contain efficiency tables for
        //different submodels, e.g. genEfficiency for generator and motEfficiency for motor.
        Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_a "Left flange of shaft" annotation (
          Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0), iconTransformation(extent = {{90, -10}, {110, 10}})));
        Modelica.Mechanics.Rotational.Sensors.SpeedSensor wSensor annotation (
          Placement(transformation(extent = {{8, -8}, {-8, 8}}, rotation = 90, origin = {78, 4})));
        Modelica.Blocks.Math.Abs abs1 annotation (
          Placement(transformation(extent = {{52, -40}, {32, -20}})));
        SupportModels.MapBasedRelated.TauLim limTau(tauMax = tauMax, powMax = powMax) annotation (
          Placement(transformation(extent = {{6, -50}, {-16, -26}})));
        SupportModels.MapBasedRelated.EfficiencyT toElePow(tauMax = tauMax, wMax = wMax, powMax = powMax, mapsFileName = mapsFileName, effMapName = effMapName) annotation (
          Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = -90, origin = {-60, -8})));
        Modelica.Electrical.Analog.Interfaces.PositivePin pin_p annotation (
          Placement(transformation(extent = {{30, 90}, {50, 110}}), iconTransformation(extent = {{30, 90}, {50, 110}})));
        Modelica.Electrical.Analog.Interfaces.NegativePin pin_n annotation (
          Placement(transformation(extent = {{-68, 90}, {-48, 110}}), iconTransformation(extent = {{-68, 90}, {-48, 110}})));
        Modelica.Mechanics.Rotational.Components.Inertia inertia(J = J) annotation (
          Placement(transformation(extent = {{22, 22}, {42, 42}})));
        Modelica.Mechanics.Rotational.Sources.Torque torque annotation (
          Placement(transformation(extent = {{-16, 22}, {4, 42}})));
        Modelica.Blocks.Math.Gain gain(k = 1) annotation (
          Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = -90, origin = {-60, 38})));
        Modelica.Mechanics.Rotational.Sensors.PowerSensor powSensor annotation (
          Placement(transformation(extent = {{58, 22}, {78, 42}})));
        SupportModels.Greater tauLimited annotation (
          Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = 90, origin = {6, -2})));
        SupportModels.MapBasedRelated.ConstPg constPg annotation (
          Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = -90, origin = {0, 88})));
      equation
        assert(wMax >= powMax / tauMax, "\n****\n" + "PARAMETER VERIFICATION ERROR:\nwMax must be not lower than powMax/tauMax" + "\n***\n");
        connect(abs1.u, wSensor.w) annotation (
          Line(points = {{54, -30}, {78, -30}, {78, -4.8}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(toElePow.w, wSensor.w) annotation (
          Line(points = {{-64, -20}, {-64, -72}, {78, -72}, {78, -4.8}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(limTau.w, abs1.y) annotation (
          Line(points = {{8.2, -30.8}, {26, -30.8}, {26, -30}, {31, -30}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(powSensor.flange_b, flange_a) annotation (
          Line(points = {{78, 32}, {94, 32}, {94, 0}, {100, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
        connect(wSensor.flange, flange_a) annotation (
          Line(points = {{78, 12}, {78, 14}, {86, 14}, {86, 14}, {94, 14}, {94, 0}, {100, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
        connect(toElePow.elePow, gain.u) annotation (
          Line(points = {{-60, 2.6}, {-60, 26}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(inertia.flange_a, torque.flange) annotation (
          Line(points = {{22, 32}, {4, 32}}, color = {0, 0, 0}, smooth = Smooth.None));
        connect(inertia.flange_b, powSensor.flange_a) annotation (
          Line(points = {{42, 32}, {58, 32}}, color = {0, 0, 0}, smooth = Smooth.None));
        connect(limTau.y, torque.tau) annotation (
          Line(points = {{-17.1, -38}, {-34, -38}, {-34, 32}, {-18, 32}}, color = {0, 0, 127}));
        connect(toElePow.tau, torque.tau) annotation (
          Line(points = {{-56, -20}, {-56, -20}, {-34, -20}, {-34, 32}, {-18, 32}}, color = {0, 0, 127}));
        connect(tauLimited.u2, torque.tau) annotation (
          Line(points = {{-2, -14}, {-26, -14}, {-26, -38}, {-34, -38}, {-34, 32}, {-18, 32}}, color = {0, 0, 127}));
        connect(tauLimited.u1, limTau.tau) annotation (
          Line(points = {{6, -14}, {18, -14}, {18, -45.2}, {8.2, -45.2}}, color = {0, 0, 127}));
        connect(constPg.pin_p, pin_p) annotation (
          Line(points = {{10, 88}, {24, 88}, {40, 88}, {40, 100}}, color = {0, 0, 255}));
        connect(constPg.pin_n, pin_n) annotation (
          Line(points = {{-9.8, 88}, {-58, 88}, {-58, 100}}, color = {0, 0, 255}));
        connect(gain.y, constPg.Pref) annotation (
          Line(points = {{-60, 49}, {-60, 49}, {-60, 68}, {-1.55431e-015, 68}, {-1.55431e-015, 79.8}}, color = {0, 0, 127}));
        annotation (
          Diagram(coordinateSystem(preserveAspectRatio = false, initialScale = 0.1)),
          Icon(coordinateSystem(preserveAspectRatio = false, initialScale = 0.1), graphics={  Rectangle(fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                    FillPattern.Solid, extent = {{-100, 80}, {100, -80}}), Rectangle(fillColor = {192, 192, 192},
                  fillPattern =                                                                                                                                                                                                        FillPattern.HorizontalCylinder, extent = {{-72, 50}, {58, -46}}), Line(points = {{62, -5}, {82, -5}}), Rectangle(fillColor = {192, 192, 192},
                  fillPattern =                                                                                                                                                                                                        FillPattern.HorizontalCylinder, extent = {{56, 10}, {100, -10}}), Line(origin = {-58, 62}, rotation = 90, points = {{-14, 0}, {32, -3.89666e-17}}, color = {0, 0, 255}), Text(origin = {-20.9415, -182}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-79.0585, 98}, {112.936, 60}}, textString = "%name"), Line(origin = {40, 60}, rotation = 90, points = {{-14, 0}, {32, -3.89666e-17}}, color = {0, 0, 255}), Rectangle(fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-66, 16}, {50, -16}}), Text(origin = {-9.8571, 51.1}, extent = {{-54.1429, -39.1}, {61.8571, -62.1}}, textString = "J=%J")}),
          Documentation(info="<html>
<p>Partial one-flange electric drive.</p>
<p>The input signal is the requested normalised torque (1 means nominal torque)</p>
</html>"));
      end PartialOneFlange;

      partial model PartialTwoFlange "Simple map-based two-flange electric drive model"
        parameter Modelica.SIunits.Power powMax = 50000 "Maximum Mechanical drive power";
        parameter Modelica.SIunits.Torque tauMax = 400 "Maximum drive Torque";
        parameter Modelica.SIunits.AngularVelocity wMax = 650 "Maximum drive speed";
        parameter Modelica.SIunits.MomentOfInertia J = 0.59 "Moment of Inertia";
        parameter String mapsFileName = "mapsFileName.txt" "Name of the txt file where models' maps are stored";
        parameter String effMapName = "effMapName.txt" "Name of the efficiency map in mapsFileName";
        //the name is passed because a file can contain efficiency tables for
        //different submodels, e.g. genEfficiency for generator and motEfficiency for motor.
        SupportModels.MapBasedRelated.TauLim limTau(tauMax = tauMax, powMax = powMax) annotation (
          Placement(transformation(extent = {{-52, -14}, {-30, 8}})));
        SupportModels.MapBasedRelated.InertiaTq inertia(w(displayUnit = "rad/s", start = 0), J = J) annotation (
          Placement(transformation(extent = {{-6, 40}, {14, 60}})));
        Modelica.Mechanics.Rotational.Sensors.SpeedSensor speedRing annotation (
          Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-80, 40})));
        Modelica.Blocks.Math.Abs abs1 annotation (
          Placement(transformation(extent = {{-76, -4}, {-62, 10}})));
        SupportModels.MapBasedRelated.EfficiencyT effMap(tauMax = tauMax, wMax = wMax, powMax = powMax, mapsFileName = mapsFileName, effMapName = effMapName) annotation (
          Placement(transformation(extent = {{20, -46}, {40, -26}})));
        Modelica.Mechanics.Rotational.Sensors.PowerSensor outBPow_ annotation (
          Placement(transformation(extent = {{62, 40}, {82, 60}})));
        Modelica.Mechanics.Rotational.Sensors.PowerSensor outAPow_ annotation (
          Placement(transformation(extent = {{-18, 40}, {-38, 60}})));
        Modelica.Blocks.Math.Add add annotation (
          Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = -90, origin = {28, 10})));
        Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_b "Right flange of shaft" annotation (
          Placement(visible = true, transformation(extent = {{90, 40}, {110, 60}}, rotation = 0), iconTransformation(extent = {{90, -12}, {110, 8}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_a "Left flange of shaft" annotation (
          Placement(visible = true, transformation(extent = {{-110, 40}, {-90, 60}}, rotation = 0), iconTransformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
        Modelica.Electrical.Analog.Interfaces.PositivePin pin_p annotation (
          Placement(visible = true, transformation(extent = {{-70, 90}, {-50, 110}}, rotation = 0), iconTransformation(extent = {{-50, 88}, {-30, 108}}, rotation = 0)));
        Modelica.Electrical.Analog.Interfaces.NegativePin pin_n annotation (
          Placement(visible = true, transformation(extent = {{30, 90}, {50, 110}}, rotation = 0), iconTransformation(extent = {{30, 90}, {50, 110}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Sensors.TorqueSensor tqSens annotation (
          Placement(transformation(extent = {{26, 40}, {46, 60}})));
        SupportModels.MapBasedRelated.ConstPg constPg annotation (
          Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = -90, origin = {0, 94})));
      equation
        connect(flange_a, speedRing.flange) annotation (
          Line(points = {{-100, 50}, {-80, 50}}, color = {0, 0, 0}, smooth = Smooth.None));
        connect(abs1.u, speedRing.w) annotation (
          Line(points = {{-77.40000000000001, 3}, {-80, 3}, {-80, 29}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(limTau.w, abs1.y) annotation (
          Line(points = {{-54.2, 3.6}, {-54.2, 3}, {-61.3, 3}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(effMap.w, speedRing.w) annotation (
          Line(points = {{18, -40}, {-80, -40}, {-80, 29}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(flange_b, outBPow_.flange_b) annotation (
          Line(points = {{100, 50}, {82, 50}}, color = {0, 0, 0}, smooth = Smooth.None));
        connect(inertia.flange_a, outAPow_.flange_a) annotation (
          Line(points = {{-6, 50}, {-18, 50}}, color = {0, 0, 0}, smooth = Smooth.None));
        connect(outAPow_.flange_b, speedRing.flange) annotation (
          Line(points = {{-38, 50}, {-80, 50}}, color = {0, 0, 0}, smooth = Smooth.None));
        connect(add.u1, outBPow_.power) annotation (
          Line(points = {{34, 22}, {34, 28}, {64, 28}, {64, 39}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(add.u2, outAPow_.power) annotation (
          Line(points = {{22, 22}, {22, 28}, {-20, 28}, {-20, 39}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(effMap.tau, inertia.tau) annotation (
          Line(points = {{18, -32}, {10, -32}, {10, -4}, {-1.45, -4}, {-1.45, 40}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(limTau.y, inertia.tau) annotation (
          Line(points = {{-28.9, -3}, {-2, -3}, {-2, 38}, {-2, 38}, {-2, 40}, {-1.45, 40}}, color = {0, 0, 127}));
        connect(tqSens.flange_a, inertia.flange_b) annotation (
          Line(points = {{26, 50}, {26, 50}, {14, 50}}, color = {0, 0, 0}));
        connect(tqSens.flange_b, outBPow_.flange_a) annotation (
          Line(points = {{46, 50}, {46, 50}, {62, 50}}, color = {0, 0, 0}));
        connect(constPg.pin_n, pin_n) annotation (
          Line(points = {{9.8, 94}, {24, 94}, {40, 94}, {40, 100}}, color = {0, 0, 255}));
        connect(constPg.pin_p, pin_p) annotation (
          Line(points = {{-10, 94}, {-36, 94}, {-60, 94}, {-60, 100}}, color = {0, 0, 255}));
        connect(constPg.Pref, effMap.elePow) annotation (
          Line(points = {{0, 85.8}, {0, 70}, {50, 70}, {50, -36}, {40.6, -36}}, color = {0, 0, 127}));
        annotation (
          Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}})),
          Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = false, initialScale = 0.1, grid = {2, 2}), graphics={  Rectangle(origin = {-25, 2}, extent = {{-75, 74}, {125, -74}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Text(origin = {4, -6}, lineColor = {0, 0, 255}, extent = {{-110, 84}, {100, 44}}, textString = "%name"), Rectangle(fillColor = {192, 192, 192},
                  fillPattern =                                                                                                                                                                                                        FillPattern.HorizontalCylinder, extent = {{-64, 38}, {64, -44}}), Rectangle(fillColor = {192, 192, 192},
                  fillPattern =                                                                                                                                                                                                        FillPattern.HorizontalCylinder, extent = {{-100, 10}, {-64, -10}}), Rectangle(fillColor = {192, 192, 192},
                  fillPattern =                                                                                                                                                                                                        FillPattern.HorizontalCylinder, extent = {{64, 8}, {100, -12}}), Line(origin = {20, 0}, points = {{-60, 94}, {-60, 76}}, color = {0, 0, 255}), Line(origin = {-20, 0}, points = {{60, 94}, {60, 76}}, color = {0, 0, 255}), Rectangle(fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-58, 14}, {58, -18}}), Text(origin = {2.733, 55.1578}, extent = {{-100.733, -41.1578}, {93.2672, -75.1578}}, textString = "J=%J")}),
          Documentation(info="<html>
<p>Partial two-flange electric drive.</p>
<p>This model receives from the connector the torque request (variable tauRef or motTauRef respectively for the inherited model interfaced through a Real input or an Expandable Connector connector) and tries to deliver it.</p>
<p>However, before delivering the requested torque, the model limits it considering the maximum deliverable torque and power. In addition it computes and considers inner losses as determined by means of a map. </p>
</html>"));
      end PartialTwoFlange;

      partial model PartialIce "Simple  map-based Internal Combustion Engine model"
        import Modelica.Constants.*;
        parameter Modelica.SIunits.AngularVelocity wIceStart = 167;
        // rad/s
        parameter Modelica.SIunits.MomentOfInertia iceJ = 0.5 "ICE moment of inertia";
        parameter Boolean tablesOnFile = false "= true, if tables are got from a file";
        parameter String mapsFileName = "NoName" "File where matrix is stored" annotation (
          Dialog(enable = tablesOnFile, loadSelector(filter = "Text files (*.txt)", caption = "Open file in which required tables are")));
        parameter Real maxIceTau[:, :] = [0, 80; 100, 80; 350, 95; 500, 95] "First column: speed (rad/s); first column: maximum ICE torque (Nm)" annotation (
          Dialog(enable = not tablesOnFile));
        Modelica.Mechanics.Rotational.Sensors.SpeedSensor w annotation (
          Placement(visible = true, transformation(origin = {52, 44}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
        Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_a annotation (
          Placement(visible = true, transformation(extent = {{90, 10}, {110, 30}}, rotation = 0), iconTransformation(extent = {{90, 10}, {110, 30}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Sensors.PowerSensor icePow annotation (
          Placement(visible = true, transformation(extent = {{66, 50}, {86, 70}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Sources.Torque Tice annotation (
          Placement(visible = true, transformation(extent = {{-12, 50}, {8, 70}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia ICE(w(fixed = true, start = wIceStart, displayUnit = "rpm"), J = iceJ) annotation (
          Placement(visible = true, transformation(extent = {{16, 50}, {36, 70}}, rotation = 0)));
        Modelica.Blocks.Math.Product toPowW annotation (
          Placement(visible = true, transformation(origin = {0, 12}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
        Modelica.Blocks.Math.Product toG_perHour annotation (
          Placement(visible = true, transformation(origin = {30, -50}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
        //  Modelica.Blocks.Continuous.Integrator toGrams(k = 1 / 3600000.0)
        // annotation(Placement(visible = true, transformation(origin = {26, -44},
        //extent = {{-10, -10}, {10, 10}}, rotation = 270)));
        Modelica.Blocks.Tables.CombiTable1D toLimTau(table = maxIceTau) annotation (
          Placement(visible = true, transformation(origin = {-72, 66}, extent = {{10, -10}, {-10, 10}}, rotation = 180)));
        Modelica.Blocks.Sources.RealExpression rotorW(y = w.w) annotation (
          Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-88, 36})));
        Modelica.Blocks.Math.Gain tokW(k = 1e-3) annotation (
          Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = -90, origin = {0, -18})));
        Modelica.Blocks.Tables.CombiTable2D toSpecCons(tableOnFile = true, fileName = "PSDmaps.txt", tableName = "specificCons") annotation (
          Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = -90, origin = {40, 0})));
      equation
        connect(toPowW.u1, w.w) annotation (
          Line(points = {{6, 24}, {6, 33}, {52, 33}}, color = {0, 0, 127}));
        connect(w.flange, ICE.flange_b) annotation (
          Line(points = {{52, 54}, {52, 60}, {36, 60}}));
        connect(icePow.flange_a, ICE.flange_b) annotation (
          Line(points = {{66, 60}, {36, 60}}));
        connect(Tice.flange, ICE.flange_a) annotation (
          Line(points = {{8, 60}, {16, 60}}));
        connect(icePow.flange_b, flange_a) annotation (
          Line(points = {{86, 60}, {94, 60}, {94, 20}, {100, 20}}));
        connect(toLimTau.u[1], rotorW.y) annotation (
          Line(points = {{-84, 66}, {-88, 66}, {-88, 47}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(toPowW.y, tokW.u) annotation (
          Line(points = {{-2.22045e-015, 1}, {-2.22045e-015, -2}, {2.22045e-015, -2}, {2.22045e-015, -6}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(toSpecCons.y, toG_perHour.u1) annotation (
          Line(points = {{40, -11}, {40, -24}, {36, -24}, {36, -38}}, color = {0, 0, 127}));
        connect(toG_perHour.u2, tokW.y) annotation (
          Line(points = {{24, -38}, {24, -32}, {0, -32}, {0, -29}}, color = {0, 0, 127}));
        connect(toSpecCons.u2, w.w) annotation (
          Line(points = {{46, 12}, {46, 28}, {52, 28}, {52, 33}}, color = {0, 0, 127}));
        connect(toSpecCons.u1, Tice.tau) annotation (
          Line(points = {{34, 12}, {34, 12}, {34, 38}, {34, 42}, {-22, 42}, {-22, 60}, {-14, 60}}, color = {0, 0, 127}));
        connect(toPowW.u2, Tice.tau) annotation (
          Line(points = {{-6, 24}, {-6, 42}, {-22, 42}, {-22, 60}, {-14, 60}}, color = {0, 0, 127}));
        annotation (
          Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -80}, {100, 80}})),
          experiment(StopTime = 200, __Dymola_NumberOfIntervals = 1000, __Dymola_Algorithm = "Lsodar"),
          __Dymola_experimentSetupOutput,
          Documentation(info="<html>
<h4>Basic map-based ICE model.</h4>
<p>It receives as input the reference torque as a fracton of the maximum deliverable torque at a given speed. It can be approximately thought as a signal proportional to the vehicle&apos;s accelerator pedal position.</p>
<p>The generated torque is the minimum between this signal and the maximum deliverable torque at the actual engine speed (defined by means of a table).</p>
<p>From the generated torque and speed the fuel consumption is computed.</p>
<p>The used maxTorque (toLimTau) and specific fuel consumption (toSpecCons) maps are inspired to public data related to the Toyota Prius&apos; engine </p>
</html>"),Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = false, initialScale = 0.1, grid = {2, 2}), graphics={  Rectangle(extent = {{-100, 80}, {100, -80}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Rectangle(fillColor = {192, 192, 192},
                  fillPattern =                                                                                                                                                                                                        FillPattern.HorizontalCylinder, extent = {{-24, 68}, {76, -24}}), Rectangle(fillColor = {192, 192, 192},
                  fillPattern =                                                                                                                                                                                                        FillPattern.HorizontalCylinder, extent = {{76, 30}, {100, 10}}), Text(origin = {0, 30}, lineColor = {0, 0, 255}, extent = {{-140, 100}, {140, 60}}, textString = "%name"), Rectangle(extent = {{-90, 68}, {-32, -26}}), Rectangle(fillColor = {95, 95, 95},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-90, 22}, {-32, 0}}), Line(points = {{-60, 56}, {-60, 32}}), Polygon(points = {{-60, 66}, {-66, 56}, {-54, 56}, {-60, 66}}), Polygon(points = {{-60, 24}, {-66, 34}, {-54, 34}, {-60, 24}}), Rectangle(fillColor = {135, 135, 135},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-64, 0}, {-54, -20}})}));
      end PartialIce;

      partial model PartialOneFlange2 "Partial map-based one-Flange electric drive model"
        parameter Real powMax = 22000 "Maximum drive power  (W)";
        parameter Real tauMax = 80 "Maximum drive torque (Nm)";
        parameter Real wMax(min = powMax / tauMax) = 3000 "Maximum drive speed (rad/s)";
        parameter Real J = 0.25 "Rotor's moment of inertia (kg.m^2)";
        parameter String mapsFileName "maps.txtName of the txt file where models' maps are stored";
        parameter String effMapName "Name of the efficiency map in mapsFileName";
        //the name is passed because a file can contain efficiency tables for
        //different submodels, e.g. genEfficiency for generator and motEfficiency for motor.
        Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_a "Left flange of shaft" annotation (
          Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0), iconTransformation(extent = {{90, -10}, {110, 10}})));
        Modelica.Mechanics.Rotational.Sensors.SpeedSensor wSensor annotation (
          Placement(visible = true, transformation(origin = {80, -2}, extent = {{8, -8}, {-8, 8}}, rotation = 90)));
        Modelica.Blocks.Math.Abs abs1 annotation (
          Placement(transformation(extent = {{60, -30}, {40, -10}})));
        SupportModels.MapBasedRelated.TauLim limTau(tauMax = tauMax, powMax = powMax) annotation (
          Placement(transformation(extent = {{6, -32}, {-14, -10}})));
        SupportModels.MapBasedRelated.EfficiencyT toElePow(tauMax = tauMax, wMax = wMax, powMax = powMax, mapsFileName = mapsFileName, effMapName = effMapName) annotation (
          Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 0, origin = {-40, -54})));
        Modelica.Electrical.Analog.Interfaces.PositivePin pin_p annotation (
          Placement(visible = true, transformation(extent = {{-110, 50}, {-90, 70}}, rotation = 0), iconTransformation(extent = {{-110, 50}, {-90, 70}}, rotation = 0)));
        Modelica.Electrical.Analog.Interfaces.NegativePin pin_n annotation (
          Placement(visible = true, transformation(origin = {-100, -58}, extent = {{10, -10}, {-10, 10}}, rotation = 0), iconTransformation(origin = {-100, -58}, extent = {{10, -10}, {-10, 10}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia inertia(J = J) annotation (
          Placement(transformation(extent = {{22, 10}, {42, 30}})));
        Modelica.Mechanics.Rotational.Sources.Torque torque annotation (
          Placement(transformation(extent = {{-16, 10}, {4, 30}})));
        Modelica.Blocks.Math.Gain gain(k = 1) annotation (
          Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 0, origin = {-64, 0})));
        Modelica.Mechanics.Rotational.Sensors.PowerSensor powSensor annotation (
          Placement(transformation(extent = {{50, 10}, {70, 30}})));
        SupportModels.MapBasedRelated.ConstPg constPg annotation (
          Placement(transformation(extent = {{-98, -10}, {-78, 10}})));
      equation
        connect(abs1.u, wSensor.w) annotation (
          Line(points = {{62, -20}, {80, -20}, {80, -10.8}}, color = {0, 0, 127}));
        connect(toElePow.w, wSensor.w) annotation (
          Line(points = {{-28, -58}, {80, -58}, {80, -10.8}}, color = {0, 0, 127}));
        connect(wSensor.flange, powSensor.flange_b) annotation (
          Line(points = {{80, 6}, {80, 6}, {80, 20}, {70, 20}, {70, 20}}));
        connect(toElePow.elePow, gain.u) annotation (
          Line(points = {{-50.6, -54}, {-58.6, -54}, {-58.6, -26}, {-44, -26}, {-44, 0}, {-52, 0}}, color = {0, 0, 127}));
        assert(wMax >= powMax / tauMax, "\n****\n" + "PARAMETER VERIFICATION ERROR:\nwMax must be not lower than powMax/tauMax" + "\n***\n");
        connect(limTau.w, abs1.y) annotation (
          Line(points = {{8, -14.4}, {28, -14.4}, {28, -20}, {39, -20}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(powSensor.flange_b, flange_a) annotation (
          Line(points = {{70, 20}, {94, 20}, {94, 0}, {100, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
        connect(inertia.flange_a, torque.flange) annotation (
          Line(points = {{22, 20}, {4, 20}}, color = {0, 0, 0}, smooth = Smooth.None));
        connect(inertia.flange_b, powSensor.flange_a) annotation (
          Line(points = {{42, 20}, {50, 20}}, color = {0, 0, 0}, smooth = Smooth.None));
        connect(limTau.y, torque.tau) annotation (
          Line(points = {{-15, -21}, {-34, -21}, {-34, 20}, {-18, 20}}, color = {0, 0, 127}));
        connect(toElePow.tau, torque.tau) annotation (
          Line(points = {{-28, -50}, {-20, -50}, {-20, -32}, {-34, -32}, {-34, 20}, {-18, 20}}, color = {0, 0, 127}));
        connect(gain.y, constPg.Pref) annotation (
          Line(points = {{-75, 0}, {-79.8, 0}}, color = {0, 0, 127}));
        connect(constPg.pin_p, pin_p) annotation (
          Line(points = {{-88, 10}, {-88, 60}, {-100, 60}}, color = {0, 0, 255}));
        connect(constPg.pin_n, pin_n) annotation (
          Line(points = {{-88, -9.8}, {-88, -9.8}, {-88, -58}, {-100, -58}}, color = {0, 0, 255}));
        annotation (
          Diagram(coordinateSystem(preserveAspectRatio = false, initialScale = 0.1)),
          Icon(coordinateSystem(preserveAspectRatio = false, initialScale = 0.1), graphics={  Rectangle(fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                    FillPattern.Solid, extent = {{-100, 80}, {100, -80}}), Rectangle(fillColor = {192, 192, 192},
                  fillPattern =                                                                                                                                                                                                        FillPattern.HorizontalCylinder, extent = {{-72, 50}, {58, -46}}), Line(points = {{62, -5}, {82, -5}}), Rectangle(fillColor = {192, 192, 192},
                  fillPattern =                                                                                                                                                                                                        FillPattern.HorizontalCylinder, extent = {{56, 10}, {100, -10}}), Line(origin = {2, 122}, rotation = 90, points = {{-72, 62}, {-60, 62}, {-60, 96}}, color = {0, 0, 255}), Text(origin = {-14.9415, 30}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-79.0585, 98}, {112.936, 60}}, textString = "%name"), Line(origin = {-138, -16}, rotation = 90, points = {{-42, -48}, {-42, -76}, {-30, -76}}, color = {0, 0, 255}), Rectangle(fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-66, 16}, {50, -12}}), Text(origin = {-9.8571, 51.1}, extent = {{-54.1429, -35.1}, {61.8571, -61.1}}, textString = "J=%J")}),
          Documentation(info = "<html>
<p>One-flange electric drive.</p>
<p>The input signal is the requested normalised torque (1 means nominal torque)</p>
</html>"));
      end PartialOneFlange2;

      model PartialIceP "Simple map-based ice model with connector and Power request"
        import Modelica.Constants.*;
        parameter Real contrGain = 0.1 "Proportional controller gain (Nm/W)";
        parameter Real wIceStart = 167;
        parameter Real iceJ = 0.5 "ICE moment of Inertia (kg.m^2)";
        // rad/s
        Modelica.Mechanics.Rotational.Components.Inertia inertia(w(fixed = true, start = wIceStart, displayUnit = "rpm"), J = iceJ) annotation (
          Placement(visible = true, transformation(extent = {{30, 42}, {50, 62}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Sources.Torque iceTau annotation (
          Placement(visible = true, transformation(extent = {{4, 42}, {24, 62}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Sensors.PowerSensor Pice annotation (
          Placement(transformation(extent = {{66, 62}, {86, 42}})));
        Modelica.Mechanics.Rotational.Sensors.SpeedSensor w annotation (
          Placement(visible = true, transformation(origin = {58, 36}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
        Modelica.Blocks.Math.Product toPowW annotation (
          Placement(visible = true, transformation(origin = {-18, 10}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
        Modelica.Blocks.Math.Feedback feedback annotation (
          Placement(transformation(extent = {{-90, 62}, {-70, 42}})));
        Modelica.Blocks.Math.Gain gain(k = contrGain) annotation (
          Placement(visible = true, transformation(extent = {{-62, 42}, {-42, 62}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_a annotation (
          Placement(transformation(extent = {{90, -10}, {110, 10}}), iconTransformation(extent = {{90, -10}, {110, 10}})));
        Modelica.Blocks.Tables.CombiTable2D toGramsPerkWh(fileName = "PSDmaps.txt", tableName = "iceSpecificCons", tableOnFile = true) annotation (
          Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = -90, origin = {42, -2})));
        Modelica.Blocks.Math.Gain tokW(k = 0.001) annotation (
          Placement(visible = true, transformation(origin = {-18, -18}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
        Modelica.Blocks.Nonlinear.Limiter limiter1(uMax = 1e99, uMin = 0) annotation (
          Placement(visible = true, transformation(origin = {-22, 52}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      equation
        connect(toPowW.y, tokW.u) annotation (
          Line(points = {{-18, -1}, {-18, -6}}, color = {0, 0, 127}));
        connect(toPowW.u2, iceTau.tau) annotation (
          Line(points = {{-24, 22}, {-24, 32}, {-6, 32}, {-6, 52}, {2, 52}}, color = {0, 0, 127}));
        connect(iceTau.tau, limiter1.y) annotation (
          Line(points = {{2, 52}, {-10, 52}, {-10, 52}, {-11, 52}}, color = {0, 0, 127}));
        connect(limiter1.u, gain.y) annotation (
          Line(points = {{-34, 52}, {-42, 52}, {-42, 52}, {-41, 52}}, color = {0, 0, 127}));
        connect(toGramsPerkWh.u1, iceTau.tau) annotation (
          Line(points = {{36, 10}, {36, 32}, {-6, 32}, {-6, 52}, {2, 52}}, color = {0, 0, 127}));
        connect(iceTau.flange, inertia.flange_a) annotation (
          Line(points = {{24, 52}, {30, 52}}));
        connect(w.flange, inertia.flange_b) annotation (
          Line(points = {{58, 46}, {58, 52}, {50, 52}}));
        connect(Pice.flange_a, inertia.flange_b) annotation (
          Line(points = {{66, 52}, {50, 52}}));
        connect(toGramsPerkWh.u2, w.w) annotation (
          Line(points = {{48, 10}, {48, 20}, {58, 20}, {58, 25}}, color = {0, 0, 127}));
        connect(toPowW.u1, w.w) annotation (
          Line(points = {{-12, 22}, {-12, 25}, {58, 25}}, color = {0, 0, 127}));
        connect(gain.u, feedback.y) annotation (
          Line(points = {{-64, 52}, {-71, 52}}, color = {0, 0, 127}));
        connect(Pice.flange_b, flange_a) annotation (
          Line(points = {{86, 52}, {94, 52}, {94, 0}, {100, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
        connect(feedback.u2, Pice.power) annotation (
          Line(points = {{-80, 60}, {-80, 72}, {68, 72}, {68, 63}}, color = {0, 0, 127}, smooth = Smooth.None));
        annotation (
          Documentation(info = "<html>
<p><b><span style=\"font-family: MS Shell Dlg 2;\">Simple map-based ICE model for power-split power trains - with connector</b> </span></p>
<p><span style=\"font-family: MS Shell Dlg 2;\">This is a &QUOT;connector&QUOT; version of MBice.</span></p>
<p><span style=\"font-family: MS Shell Dlg 2;\">For a general descritiption see the info of MBice.</span></p>
<p><span style=\"font-family: MS Shell Dlg 2;\">Signals connected to the connector:</span></p>
<p><span style=\"font-family: MS Shell Dlg 2;\">- icePowRef (input) is the power request (W). Negative values are internally converted to zero</span></p>
<p><span style=\"font-family: MS Shell Dlg 2;\">- iceW (output) is the measured ICE speed (rad/s)</span></p>
<p><span style=\"font-family: MS Shell Dlg 2;\">- icePowDel (output) delivered power (W)</span></p>
</html>"),Icon(coordinateSystem(preserveAspectRatio = false, initialScale = 0.1), graphics={  Rectangle(fillColor = {192, 192, 192},
                  fillPattern =                                                                                                                    FillPattern.HorizontalCylinder, extent = {{-24, 48}, {76, -44}}), Rectangle(fillColor = {192, 192, 192},
                  fillPattern =                                                                                                                                                                                                        FillPattern.HorizontalCylinder, extent = {{76, 10}, {100, -10}}), Text(origin = {-2, 0}, extent = {{-140, -52}, {140, -86}}, textString = "J=%J"), Rectangle(extent = {{-100, 62}, {100, -100}}), Text(origin = {0, 10}, lineColor = {0, 0, 255}, extent = {{-140, 100}, {140, 60}}, textString = "%name"), Rectangle(extent = {{-90, 48}, {-32, -46}}), Rectangle(fillColor = {95, 95, 95},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-90, 2}, {-32, -20}}), Line(points = {{-60, 36}, {-60, 12}}), Polygon(points = {{-60, 46}, {-66, 36}, {-54, 36}, {-60, 46}}), Polygon(points = {{-60, 4}, {-66, 14}, {-54, 14}, {-60, 4}}), Rectangle(fillColor = {135, 135, 135},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-64, -20}, {-54, -40}})}),
          Diagram(coordinateSystem(extent = {{-100, -80}, {100, 80}}, preserveAspectRatio = false, initialScale = 0.1, grid = {2, 2}), graphics={  Text(extent = {{-90, 20}, {-46, -16}}, textString = "follows the power
reference \nand computes consumption")}));
      end PartialIceP;
    end Partial;

    package TestingModels
      extends Modelica.Icons.ExamplesPackage;

      model TestIceT
        IceT iceT(wIceStart = 90) annotation (
          Placement(transformation(extent = {{-20, -2}, {0, 18}})));
        Modelica.Mechanics.Rotational.Components.Inertia inertia(J = 0.5, phi(start = 0, fixed = true)) annotation (
          Placement(transformation(extent = {{10, 0}, {30, 20}})));
        Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque loadTorque(w_nominal = 100, tau_nominal = -80) annotation (
          Placement(transformation(extent = {{64, 0}, {44, 20}})));
        Modelica.Blocks.Sources.Trapezoid trapezoid(rising = 10, width = 10, falling = 10, period = 1e6, startTime = 10, offset = 60, amplitude = 30) annotation (
          Placement(transformation(extent = {{-50, -38}, {-30, -18}})));
      equation
        connect(iceT.flange_a, inertia.flange_a) annotation (
          Line(points = {{0, 10}, {10, 10}}, color = {0, 0, 0}, smooth = Smooth.None));
        connect(inertia.flange_b, loadTorque.flange) annotation (
          Line(points = {{30, 10}, {44, 10}}, color = {0, 0, 0}, smooth = Smooth.None));
        connect(iceT.tauRef, trapezoid.y) annotation (
          Line(points = {{-16, -2}, {-16, -28}, {-29, -28}}, color = {0, 0, 127}, smooth = Smooth.None));
        annotation (
          Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-60, -60}, {80, 40}}), graphics),
          experiment(StopTime = 50),
          __Dymola_experimentSetupOutput,
          Icon(coordinateSystem(extent = {{-60, -60}, {80, 40}})),
          Documentation(info = "<html>
<p>This is a simple test of model IceT.</p>
<p>It shows that the generated torque follows the torque request as long as the maximum allowed is not overcome; otherwise this maximum is generated.</p>
<p>It shows also the fuel consumption output.</p>
<p>The user could compare the torque request tauRef with the torque generated and at the ICE flange (with this transient the inertia torques are very small and can be neglected). The user could also have a look at the rotational speeds and fuel consumption. </p>
</html>"));
      end TestIceT;

      model TestOneFlange
        Modelica.Mechanics.Rotational.Components.Inertia inertia(J = 0.5, phi(start = 0, fixed = true), w(start = 50, fixed = true)) annotation (
          Placement(transformation(extent = {{38, 0}, {58, 20}})));
        Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque loadTorque(tau_nominal = -50, w_nominal = 400) annotation (
          Placement(transformation(extent = {{92, 0}, {72, 20}})));
        Modelica.Blocks.Sources.Trapezoid tauRef(rising = 10, width = 10, falling = 10, period = 1e6, startTime = 10, amplitude = 50, offset = 20) annotation (
          Placement(transformation(extent = {{-60, -38}, {-40, -18}})));
        OneFlange oneFlange(powMax = 10000, tauMax = 50, J = 0.5, wMax = 300) annotation (
          Placement(transformation(extent = {{-22, 0}, {-2, 20}})));
        Modelica.Electrical.Analog.Sources.ConstantVoltage gen(V = 100) annotation (
          Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = -90, origin = {-64, 10})));
        Modelica.Electrical.Analog.Basic.Ground ground annotation (
          Placement(transformation(extent = {{-90, -20}, {-70, 0}})));
        Modelica.Mechanics.Rotational.Sensors.PowerSensor powMech annotation (
          Placement(transformation(extent = {{12, 0}, {32, 20}})));
        Modelica.Electrical.Analog.Sensors.PowerSensor powElec annotation (
          Placement(transformation(extent = {{-48, 20}, {-28, 40}})));
      equation
        connect(inertia.flange_b, loadTorque.flange) annotation (
          Line(points = {{58, 10}, {72, 10}}, color = {0, 0, 0}, smooth = Smooth.None));
        connect(tauRef.y, oneFlange.tauRef) annotation (
          Line(points = {{-39, -28}, {-32, -28}, {-32, 8}, {-23.4, 8}, {-23.4, 8.88889}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(ground.p, gen.n) annotation (
          Line(points = {{-80, 0}, {-64, 0}}, color = {0, 0, 255}, smooth = Smooth.None));
        connect(oneFlange.flange_a, powMech.flange_a) annotation (
          Line(points = {{-2, 8.88889}, {6, 8.88889}, {6, 10}, {12, 10}}, color = {0, 0, 0}, smooth = Smooth.None));
        connect(inertia.flange_a, powMech.flange_b) annotation (
          Line(points = {{38, 10}, {32, 10}}, color = {0, 0, 0}, smooth = Smooth.None));
        connect(powElec.nc, oneFlange.pin_p) annotation (
          Line(points = {{-28, 30}, {-8, 30}, {-8, 20}}, color = {0, 0, 255}, smooth = Smooth.None));
        connect(powElec.pc, gen.p) annotation (
          Line(points = {{-48, 30}, {-64, 30}, {-64, 20}}, color = {0, 0, 255}, smooth = Smooth.None));
        connect(powElec.pv, powElec.nc) annotation (
          Line(points = {{-38, 40}, {-28, 40}, {-28, 30}}, color = {0, 0, 255}, smooth = Smooth.None));
        connect(gen.n, oneFlange.pin_n) annotation (
          Line(points = {{-64, 0}, {-38, 0}, {-38, 20}, {-17.8, 20}}, color = {0, 0, 255}, smooth = Smooth.None));
        connect(powElec.nv, oneFlange.pin_n) annotation (
          Line(points = {{-38, 20}, {-17.8, 20}}, color = {0, 0, 255}, smooth = Smooth.None));
        annotation (
          Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -60}, {100, 60}}), graphics),
          experiment(StopTime = 50),
          __Dymola_experimentSetupOutput,
          Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}})),
          Documentation(info = "<html>
<p>This is a simple test of model OneFlange.</p>
<p>It shows that the generated torque follows the normalised torque request as long as it does not overcome unity. Actual torque will be this request times the maximum value that, in turn, is the minimum between tauMax and powerMax/w (while w is the rotational speed)</p>
<p>It shows also the effects of efficiency on the DC power.</p>
<p><u>First suggested plots</u>: on the same axis oneFlange.torque.tau, and tauRef vertically aligned with the previous oneFlange.limTau.state. In these plots it can be seen that:</p>
<ul>
<li>during the first 10 seconds the generated torque oneFlange.torque.tau, is 20Nm, as requested from the input. The maximum torque that can be generated is not limited by the power limit</li>
<li>between t=10 and 12 s the generated torque continues to follow the input signal; </li>
<li>between t=12 and 37.7 s, since the drive power has been reached (10 kW), the generated torque is automatically reduced to avoid this limit to be overcome </li>
<li>above t=37.7 the torque request is reduced and the drive is again able to deliver this torque.</li>
<li>All the above behaviour is confirmed by the value of boolean variable tauLimited.y.</li>
</ul>
<p><br><u>Second suggested plot</u>: Once the first plot is anaysed, the user might want to have an idea of the mechanical and electrical powers: these are seen putting in the same plot powMech.power and powElec.power.</p>
</html>"));
      end TestOneFlange;

      model TestIceConn
        Modelica.Mechanics.Rotational.Components.Inertia inertia(phi(start = 0, fixed = true), J = 10) annotation (
          Placement(transformation(extent = {{-14, 0}, {6, 20}})));
        Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque loadTorque(w_nominal = 100, tau_nominal = -80) annotation (
          Placement(transformation(extent = {{64, 0}, {44, 20}})));
        IceConnP ice(wIceStart = 90, mapsFileName="PSDmaps.txt")
                                     annotation (
          Placement(transformation(extent = {{-42, 0}, {-22, 20}})));
        SupportModels.ConnectorRelated.ToConnIcePowRef toConnIceTauRef annotation (
          Placement(transformation(extent = {{-6, -6}, {6, 6}}, rotation = 90, origin = {-32, -18})));
        Modelica.Blocks.Sources.Trapezoid powReq(rising = 10, width = 10, falling = 10, period = 1e6, startTime = 10, offset = 60, amplitude = 10e3) annotation (
          Placement(transformation(extent = {{-74, -30}, {-54, -10}})));
        Modelica.Mechanics.Rotational.Sensors.PowerSensor outPow annotation (
          Placement(transformation(extent = {{18, 0}, {38, 20}})));
      equation
        connect(inertia.flange_a, ice.flange_a) annotation (
          Line(points = {{-14, 10}, {-22, 10}}, color = {0, 0, 0}, smooth = Smooth.None));
        connect(toConnIceTauRef.conn, ice.conn) annotation (
          Line(points = {{-32, -12}, {-32, 0.2}}, color = {255, 204, 51}, thickness = 0.5, smooth = Smooth.None));
        connect(toConnIceTauRef.u, powReq.y) annotation (
          Line(points = {{-32, -25.4}, {-32, -32}, {-44, -32}, {-44, -20}, {-53, -20}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(inertia.flange_b, outPow.flange_a) annotation (
          Line(points = {{6, 10}, {18, 10}}, color = {0, 0, 0}));
        connect(loadTorque.flange, outPow.flange_b) annotation (
          Line(points = {{44, 10}, {38, 10}}, color = {0, 0, 0}));
        annotation (
          Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-80, -60}, {80, 60}})),
          experiment(StopTime = 50),
          __Dymola_experimentSetupOutput,
          Icon(coordinateSystem(extent = {{-80, -60}, {80, 60}})),
          Documentation(info = "<html>
<p>This is a simple test of model IceConn, loaded with a huge inertia and a quadratic dependent load torque.</p>
<p>It shows that the generated power (variable icePowDel inside connectors and bus) follows the power request. The load power outPow.Power differs from the generated power due to the large inertia in-between. If closer matching between icePowDel and powReq.y is wanted the ice inner control gain contrGain can be raised.</p>
<p>It shows also the fuel consumption output. The user could also have a look at the rotational speed. </p>
</html>"));
      end TestIceConn;

      model TestOneFlangeConn
        Modelica.Mechanics.Rotational.Components.Inertia inertia(J = 0.5, phi(start = 0, fixed = true), w(start = 50, fixed = true)) annotation (
          Placement(transformation(extent = {{38, 0}, {58, 20}})));
        Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque loadTorque(tau_nominal = -50, w_nominal = 400) annotation (
          Placement(transformation(extent = {{92, 0}, {72, 20}})));
        Modelica.Electrical.Analog.Sources.ConstantVoltage gen(V = 100) annotation (
          Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = -90, origin = {-64, 10})));
        Modelica.Electrical.Analog.Basic.Ground ground annotation (
          Placement(transformation(extent = {{-90, -20}, {-70, 0}})));
        Modelica.Mechanics.Rotational.Sensors.PowerSensor powMech annotation (
          Placement(transformation(extent = {{12, 0}, {32, 20}})));
        Modelica.Electrical.Analog.Sensors.PowerSensor powElec annotation (
          Placement(transformation(extent = {{-52, 14}, {-32, 34}})));
        PsdGenConn oneFlangeConn(powMax = 10000, tauMax = 50, J = 0.5, wMax = 300,
          mapsFileName="EVmaps.txt",
          effMapName="effTable")                                                   annotation (
          Placement(transformation(extent = {{-16, 0}, {4, 20}})));
        SupportModels.ConnectorRelated.ToConnGenTauRef toConnGenTauNorm annotation (
          Placement(transformation(extent = {{-16, -34}, {-4, -22}})));
        Modelica.Blocks.Sources.Trapezoid tauRef(rising = 10, width = 10, falling = 10, period = 1e6, startTime = 10, amplitude = 50, offset = 20) annotation (
          Placement(transformation(extent = {{-48, -38}, {-28, -18}})));
      equation
        connect(inertia.flange_b, loadTorque.flange) annotation (
          Line(points = {{58, 10}, {72, 10}}, color = {0, 0, 0}, smooth = Smooth.None));
        connect(ground.p, gen.n) annotation (
          Line(points = {{-80, 0}, {-64, 0}}, color = {0, 0, 255}, smooth = Smooth.None));
        connect(inertia.flange_a, powMech.flange_b) annotation (
          Line(points = {{38, 10}, {32, 10}}, color = {0, 0, 0}, smooth = Smooth.None));
        connect(powElec.pc, gen.p) annotation (
          Line(points = {{-52, 24}, {-64, 24}, {-64, 20}}, color = {0, 0, 255}, smooth = Smooth.None));
        connect(powElec.pv, powElec.nc) annotation (
          Line(points = {{-42, 34}, {-36, 34}, {-36, 34}, {-32, 34}, {-32, 24}}, color = {0, 0, 255}, smooth = Smooth.None));
        connect(powMech.flange_a, oneFlangeConn.flange_a) annotation (
          Line(points = {{12, 10}, {4, 10}}, color = {0, 0, 0}, smooth = Smooth.None));
        connect(oneFlangeConn.pin_p, powElec.nc) annotation (
          Line(points = {{-2, 20}, {-24, 20}, {-24, 24}, {-32, 24}}, color = {0, 0, 255}, smooth = Smooth.None));
        connect(oneFlangeConn.pin_n, gen.n) annotation (
          Line(points = {{-11.8, 20}, {-24, 20}, {-24, 0}, {-64, 0}}, color = {0, 0, 255}, smooth = Smooth.None));
        connect(toConnGenTauNorm.conn, oneFlangeConn.conn) annotation (
          Line(points = {{-4.2, -28}, {4, -28}, {4, 2.2}}, color = {255, 204, 51}, thickness = 0.5, smooth = Smooth.None));
        connect(powElec.nv, gen.n) annotation (
          Line(points = {{-42, 14}, {-42, 0}, {-64, 0}}, color = {0, 0, 255}, smooth = Smooth.None));
        connect(toConnGenTauNorm.u, tauRef.y) annotation (
          Line(points = {{-17, -28}, {-27, -28}}, color = {0, 0, 127}, smooth = Smooth.None));
        annotation (
          Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -60}, {100, 60}}), graphics),
          experiment(StopTime = 50),
          __Dymola_experimentSetupOutput,
          Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}})),
          Documentation(info = "<html>
<p>This is a simple test of model OneFlange with bus connector.</p>
<p>For the description see the description of TestOneFlange (substitute the word &QUOT;oneFlange&QUOT; with &QUOT;oneFlangeConn&QUOT;).</p>
</html>"));
      end TestOneFlangeConn;

      model TestTwoFlange "Test of TwoFlange drive train model"

        Modelica.Mechanics.Rotational.Components.Inertia inertia(J = 0.5, phi(start = 0, fixed = true), w(start = 50, fixed = true)) annotation (
          Placement(transformation(extent = {{38, -10}, {58, 10}})));
        Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque loadTorque(w_nominal = 400, tau_nominal = -50.0) annotation (
          Placement(transformation(extent = {{92, -10}, {72, 10}})));
        Modelica.Electrical.Analog.Sources.ConstantVoltage gen(V = 100) annotation (
          Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = -90, origin = {-60, 28})));
        Modelica.Electrical.Analog.Basic.Ground ground annotation (
          Placement(transformation(extent = {{-100, -2}, {-80, 18}})));
        Modelica.Mechanics.Rotational.Sensors.PowerSensor powMech2 annotation (
          Placement(transformation(extent = {{12, -10}, {32, 10}})));
        Modelica.Electrical.Analog.Sensors.PowerSensor powElec annotation (
          Placement(transformation(extent = {{-48, 32}, {-28, 52}})));
        TwoFlange twoFlanges(J = 0.5, wMax = 300, tauMax = 60, powMax = 22000,
          mapsFileName="EVmaps.txt",
          effMapName="effTable")                                               annotation (
          Placement(transformation(extent = {{-18, -10}, {2, 10}})));
        Modelica.Mechanics.Rotational.Sources.ConstantTorque tau1(tau_constant = -5.0) annotation (
          Placement(transformation(extent = {{-76, -10}, {-56, 10}})));
        Modelica.Mechanics.Rotational.Sensors.PowerSensor powMech1 annotation (
          Placement(transformation(extent = {{-28, -10}, {-48, 10}})));
        Modelica.Blocks.Sources.Trapezoid tauRef(rising = 10, width = 10, falling = 10, period = 1e6, startTime = 10, amplitude = 50, offset = 20) annotation (
          Placement(transformation(extent = {{-40, -48}, {-20, -28}})));
      equation
        connect(inertia.flange_b, loadTorque.flange) annotation (
          Line(points = {{58, 0}, {72, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
        connect(ground.p, gen.n) annotation (
          Line(points = {{-90, 18}, {-60, 18}}, color = {0, 0, 255}, smooth = Smooth.None));
        connect(inertia.flange_a, powMech2.flange_b) annotation (
          Line(points = {{38, 0}, {32, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
        connect(powElec.pc, gen.p) annotation (
          Line(points = {{-48, 42}, {-60, 42}, {-60, 38}}, color = {0, 0, 255}, smooth = Smooth.None));
        connect(powElec.pv, powElec.nc) annotation (
          Line(points = {{-38, 52}, {-28, 52}, {-28, 42}}, color = {0, 0, 255}, smooth = Smooth.None));
        connect(powMech2.flange_a, twoFlanges.flange_b) annotation (
          Line(points = {{12, 0}, {8, 0}, {8, -0.2}, {2, -0.2}}, color = {0, 0, 0}, smooth = Smooth.None));
        connect(powElec.nc, twoFlanges.pin_n) annotation (
          Line(points = {{-28, 42}, {-4, 42}, {-4, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
        connect(twoFlanges.pin_p, gen.n) annotation (
          Line(points = {{-12, 9.8}, {-12, 18}, {-60, 18}}, color = {0, 0, 255}, smooth = Smooth.None));
        connect(powElec.nv, gen.n) annotation (
          Line(points = {{-38, 32}, {-38, 18}, {-60, 18}}, color = {0, 0, 255}, smooth = Smooth.None));
        connect(twoFlanges.flange_a, powMech1.flange_a) annotation (
          Line(points = {{-18, 0}, {-28, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
        connect(tau1.flange, powMech1.flange_b) annotation (
          Line(points = {{-56, 0}, {-48, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
        connect(tauRef.y, twoFlanges.tauRef) annotation (
          Line(points = {{-19, -38}, {-8, -38}, {-8, -9.2}}, color = {0, 0, 127}));
        annotation (
          Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -60}, {100, 60}})),
          experiment(StopTime = 50),
          __Dymola_experimentSetupOutput,
          Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}})),
          Documentation(info="<html>
<p>This is a simple test of model TwoFlange. </p>
<p>It shows that the generated torque follows the normalised torque request as long as it does not overcome torque and power limits. The generated torque will act on the machine inertia in conjunction with the torques applied from the exterior to the two flanges. </p>
<p>It shows also the effects of efficiency on the DC power. </p>
<p>First suggested plots: a plot with tauRef.y and twoFlanges.inertia.flange_a.tau; with the same axes another plot with twoFlanges.limTau.powLimActive and twoFlanges.limTau.powLimActive. In these plots it can be seen that: </p>
<p>&middot;<span style=\"font-size: 7pt;\">&nbsp; </span>during the first 18 seconds the generated torque equals the torque request tauRef.y </p>
<p>&middot;<span style=\"font-size: 7pt;\">&nbsp; </span>between 18 and 21 s the maximum torque limit is reached, but not the maximum power limit </p>
<p>&middot;<span style=\"font-size: 7pt;\">&nbsp; </span>between 21 and 33 s the maximum power occurs </p>
<p>&middot;<span style=\"font-size: 7pt;\">&nbsp; </span>after 33s, the requested torque is delivered. </p>
<p>Second suggested plot: once the first plots are analysed, the user might want to have an idea of the mechanical and electrical powers: these are seen putting in the same plot (powMech1.power+powMech2.power) and powElec.power. </p>
</html>"));
      end TestTwoFlange;
    end TestingModels;

    package ECUs
      model Ecu1 "Power Split hybrid power train controller, not using ON/OFF strategy"
        parameter Real genTorqueMax = 80 "maximum absolute value of gen torque (Nm)";
        parameter Real maxTorqueReq = 80 "Torque request (Nm) that corresponds to 1 from driver";
        parameter Real powFiltT = 60 "Power filter time constant (s)";
        parameter Real genLoopGain = 0.1 "Control gain between ice speed error and gen torque: Nm/(rad/s)";
        Modelica.Blocks.Interfaces.RealInput tauRef annotation (
          Placement(visible = true, transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0), iconTransformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Modelica.Blocks.Continuous.FirstOrder powFilt(T = powFiltT) annotation (
          Placement(visible = true, transformation(origin = {-40, 40}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
        SupportModels.ConnectorRelated.Conn conn1 annotation (
          Placement(visible = true, transformation(extent = {{-20, 60}, {20, 100}}, rotation = 0), iconTransformation(extent = {{-20, 78}, {20, 118}}, rotation = 0)));
        Modelica.Blocks.Math.Gain gain(k = genLoopGain) annotation (
          Placement(visible = true, transformation(extent = {{32, -30}, {52, -10}}, rotation = 0)));
        Modelica.Blocks.Math.Feedback feedback annotation (
          Placement(visible = true, transformation(extent = {{6, -10}, {26, -30}}, rotation = 0)));
        Modelica.Blocks.Nonlinear.Limiter limiter(uMax = genTorqueMax, uMin = -genTorqueMax) annotation (
          Placement(visible = true, transformation(origin = {60, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
        Modelica.Blocks.Tables.CombiTable1Ds powToW(fileName = "wToTau.txt", tableOnFile = false, table = [0, 0; 1884, 126; 9800, 126; 36600, 366; 52300, 523]) "optimal ice speed as a function of power" annotation (
          Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {-40, -20})));
        Modelica.Blocks.Math.Gain toNm(k = maxTorqueReq) "converts p.u. torque request into Nm" annotation (
          Placement(visible = true, transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-80, 32})));
        Modelica.Blocks.Nonlinear.Limiter limiter1(uMax = 1e6, uMin = 125) annotation (
          Placement(visible = true, transformation(origin = {-10, -20}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        Modelica.Blocks.Continuous.FirstOrder genTauFilt(T = 1) annotation (
          Placement(visible = true, transformation(origin = {60, 12}, extent = {{10, -10}, {-10, 10}}, rotation = -90)));
      equation
        connect(powFilt.u, conn1.motPowDelB) annotation (
          Line(points = {{-40, 52}, {-40, 60}, {0, 60}, {0, 80}}, color = {0, 0, 127}));
        connect(powFilt.y, conn1.icePowRef) annotation (
          Line(points = {{-40, 29}, {-40, 20}, {0, 20}, {0, 80}}, color = {0, 0, 127}, smooth = Smooth.None),
          Text(string = "%second", index = 1, extent = {{6, 3}, {6, 3}}));
        connect(gain.u, feedback.y) annotation (
          Line(points = {{30, -20}, {25, -20}}, color = {0, 0, 127}));
        connect(powToW.u, powFilt.y) annotation (
          Line(points = {{-52, -20}, {-60, -20}, {-60, 20}, {-40, 20}, {-40, 29}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(feedback.u2, conn1.iceW) annotation (
          Line(points = {{16, -12}, {16, 40}, {16, 40}, {16, 66}, {0, 66}, {0, 80}}, color = {0, 0, 127}, smooth = Smooth.None),
          Text(string = "%second", index = 1, extent = {{6, 3}, {6, 3}}));
        connect(limiter.y, conn1.genTauRef) annotation (
          Line(points = {{60, 51}, {60, 74}, {0, 74}, {0, 80}}, color = {0, 0, 127}, smooth = Smooth.None),
          Text(string = "%second", index = 1, extent = {{6, 3}, {6, 3}}));
        connect(toNm.u, tauRef) annotation (
          Line(points = {{-80, 20}, {-80, 0}, {-120, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(toNm.y, conn1.motTauRef) annotation (
          Line(points = {{-80, 43}, {-80, 80}, {0, 80}}, color = {0, 0, 127}, smooth = Smooth.None),
          Text(string = "%second", index = 1, extent = {{6, 3}, {6, 3}}));
        connect(feedback.u1, limiter1.y) annotation (
          Line(points = {{8, -20}, {1, -20}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(limiter1.u, powToW.y[1]) annotation (
          Line(points = {{-22, -20}, {-29, -20}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(limiter.u, genTauFilt.y) annotation (
          Line(points = {{60, 28}, {60, 23}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(genTauFilt.u, gain.y) annotation (
          Line(points = {{60, 0}, {60, -20}, {53, -20}}, color = {0, 0, 127}, smooth = Smooth.None));
        annotation (
          Diagram(coordinateSystem(extent = {{-100, -60}, {100, 80}}, preserveAspectRatio = false, initialScale = 0.1), graphics={  Text(extent = {{-82, 74}, {-24, 70}}, textString = "Send requested torque to mot"), Text(extent = {{-22, 16}, {6, 12}}, textString = "send filtered 
power to ice"), Text(extent = {{62, 70}, {94, 60}}, textString = "send 
reference tau
to gen", horizontalAlignment = TextAlignment.Left)}),
          Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = false, initialScale = 0.1, grid = {2, 2}), graphics={  Text(lineColor = {0, 0, 255}, extent = {{-100, -102}, {100, -140}}, textString = "%name"), Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-4, -40}, {74, 16}, {74, -6}, {-4, -62}, {-4, -40}}, lineColor = {95, 95, 95}, fillColor = {175, 175, 175},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{8, -38}, {28, -48}, {20, -54}, {0, -44}, {8, -38}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{20, -54}, {28, -48}, {32, -56}, {24, -62}, {20, -54}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{24, -62}, {32, -56}, {32, -78}, {24, -84}, {24, -62}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 127},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{0, -44}, {20, -54}, {24, -62}, {24, -84}, {22, -84}, {22, -62}, {20, -58}, {0, -48}, {0, -44}}, lineColor = {0, 0, 255}, fillColor = {191, 191, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-14, 40}, {-18, 32}, {-10, 38}, {-8, 44}, {-14, 40}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-18, 32}, {-10, 38}, {-10, 14}, {-18, 8}, {-18, 32}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 127},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-20, 10}, {-20, 32}, {-16, 40}, {4, 30}, {4, 26}, {-16, 36}, {-18, 32}, {-18, 8}, {-20, 10}}, lineColor = {0, 0, 255}, fillColor = {191, 191, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-8, 46}, {12, 36}, {4, 30}, {-16, 40}, {-8, 46}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{28, -22}, {48, -32}, {40, -38}, {20, -28}, {28, -22}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{40, -38}, {48, -32}, {52, -40}, {44, -46}, {40, -38}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{44, -46}, {52, -40}, {52, -62}, {44, -68}, {44, -46}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 127},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{20, -28}, {40, -38}, {44, -46}, {44, -68}, {42, -68}, {42, -46}, {40, -42}, {20, -32}, {20, -28}}, lineColor = {0, 0, 255}, fillColor = {191, 191, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{48, -8}, {68, -18}, {60, -24}, {40, -14}, {48, -8}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{60, -24}, {68, -18}, {72, -26}, {64, -32}, {60, -24}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{64, -32}, {72, -26}, {72, -48}, {64, -54}, {64, -32}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 127},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{40, -14}, {60, -24}, {64, -32}, {64, -54}, {62, -54}, {62, -32}, {60, -28}, {40, -18}, {40, -14}}, lineColor = {0, 0, 255}, fillColor = {191, 191, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{68, 6}, {88, -4}, {80, -10}, {60, 0}, {68, 6}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{80, -10}, {88, -4}, {92, -12}, {84, -18}, {80, -10}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{84, -18}, {92, -12}, {92, -34}, {84, -40}, {84, -18}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 127},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{60, 0}, {80, -10}, {84, -18}, {84, -40}, {82, -40}, {82, -18}, {80, -14}, {60, -4}, {60, 0}}, lineColor = {0, 0, 255}, fillColor = {191, 191, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-34, 26}, {-38, 18}, {-30, 24}, {-28, 30}, {-34, 26}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-38, 18}, {-30, 24}, {-30, 0}, {-38, -6}, {-38, 18}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 127},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-40, -4}, {-40, 18}, {-36, 26}, {-16, 16}, {-16, 12}, {-36, 22}, {-38, 18}, {-38, -6}, {-40, -4}}, lineColor = {0, 0, 255}, fillColor = {191, 191, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-28, 32}, {-8, 22}, {-16, 16}, {-36, 26}, {-28, 32}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-54, 12}, {-58, 4}, {-50, 10}, {-48, 16}, {-54, 12}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-58, 4}, {-50, 10}, {-50, -14}, {-58, -20}, {-58, 4}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 127},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-60, -18}, {-60, 4}, {-56, 12}, {-36, 2}, {-36, -2}, {-56, 8}, {-58, 4}, {-58, -20}, {-60, -18}}, lineColor = {0, 0, 255}, fillColor = {191, 191, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-48, 18}, {-28, 8}, {-36, 2}, {-56, 12}, {-48, 18}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-74, -4}, {-78, -12}, {-70, -6}, {-68, 0}, {-74, -4}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-78, -12}, {-70, -6}, {-70, -30}, {-78, -36}, {-78, -12}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 127},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-80, -34}, {-80, -12}, {-76, -4}, {-56, -14}, {-56, -18}, {-76, -8}, {-78, -12}, {-78, -36}, {-80, -34}}, lineColor = {0, 0, 255}, fillColor = {191, 191, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-68, 2}, {-48, -8}, {-56, -14}, {-76, -4}, {-68, 2}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-64, -8}, {-4, -40}, {-4, -62}, {-64, -30}, {-64, -8}}, lineColor = {95, 95, 95}, fillColor = {75, 75, 75},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-64, -8}, {-4, -40}, {74, 16}, {14, 48}, {-64, -8}}, lineColor = {95, 95, 95}, fillColor = {160, 160, 164},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Rectangle(extent = {{-98, 92}, {98, 62}}, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, pattern = LinePattern.None), Text(extent = {{-100, 84}, {100, 54}}, lineColor = {0, 0, 0}, textString = "PSD-ecu1")}),
          Documentation(info = "<html>
<p><b><span style=\"font-family: MS Shell Dlg 2;\">Power Split Power Train Controller without ON/OFF</span></b></p>
<p><span style=\"font-family: MS Shell Dlg 2;\">This controller operates as follows:</span></p>
<ul>
<li><span style=\"font-family: MS Shell Dlg 2;\">it makes the ice deliver the average power needed by the power train</span></li>
<li><span style=\"font-family: MS Shell Dlg 2;\">it determines the optimal ice speed at which the requested power is delivered with minimum fuel consumption and asks the &QUOT;gen&QUOT; to control so that the ice opertes at that speed</span></li>
<li><span style=\"font-family: MS Shell Dlg 2;\">the vehicle motion is controlled acting on the &QUOT;mot&QUOT;.</span></li>
</ul>
<p><span style=\"font-family: MS Shell Dlg 2;\"></p><p>Since this technique allows only approximatively the correct energy balance of the vehicle, the battery tends to sdischarge.This is solved with MBEcu2, in which a closed loop StateOfCharge (SOC) control is added.</span></p>
<p><span style=\"font-family: MS Shell Dlg 2;\">So:</span></p>
<ul>
<li><span style=\"font-family: MS Shell Dlg 2;\">powFilt Block filters the delivered power to obtained the power to ask the ICE to deliver</span></li>
<li><span style=\"font-family: MS Shell Dlg 2;\">toIceWref converts the power to be requested from the ICE by its maximum torque at the actual speed</span></li>
<li><span style=\"font-family: MS Shell Dlg 2;\">after a limiting block, this torque is the reference signal of a feedback; the corresponnding error controls the Gen torque.</span></li>
</ul>
</html>"));
      end Ecu1;

      model Ecu2 "Power Split hybrid power train controller, with SOC control, without ON/OFF"
        parameter Modelica.SIunits.Torque genTorqueMax = 80 "maximum absolute valoe of gen torque";
        parameter Real socRef = 0.6 "Target value of SOC";
        parameter Modelica.SIunits.Power socLoopGain = 10000 "soc loop gain";
        parameter Modelica.SIunits.Torque maxTorqueReq = 80 "Torque request (Nm) that corresponds to 1 from driver";
        parameter Real genLoopGain(unit = "N.m/(rad/s)") = 0.1 "Control gain between ICE speed error and gen torque";
        parameter Modelica.SIunits.Time powFiltT = 60 "Power filter time constant (s)";
        Modelica.Blocks.Interfaces.RealInput tauReference annotation (
          Placement(visible = true, transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0), iconTransformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Modelica.Blocks.Continuous.FirstOrder powFilt(T = powFiltT) annotation (
          Placement(visible = true, transformation(origin = {20, 46}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
        SupportModels.ConnectorRelated.Conn conn1 annotation (
          Placement(visible = true, transformation(extent = {{-20, 60}, {20, 100}}, rotation = 0), iconTransformation(extent = {{-20, 78}, {20, 118}}, rotation = 0)));
        Modelica.Blocks.Math.Gain gain(k = genLoopGain) annotation (
          Placement(visible = true, transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {84, -4})));
        Modelica.Blocks.Math.Feedback feedback annotation (
          Placement(visible = true, transformation(extent = {{36, -30}, {56, -50}}, rotation = 0)));
        Modelica.Blocks.Tables.CombiTable1Ds toIceWref(fileName = "wToTau.txt", tableOnFile = false, table = [0, 0; 1884, 126; 9800, 126; 36600, 366; 52300, 523]) "optimal ice speed as a function of power" annotation (
          Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {-16, -40})));
        Modelica.Blocks.Math.Gain toNm(k = maxTorqueReq) "converts p.u. torque request into Nm" annotation (
          Placement(visible = true, transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-86, 30})));
        Modelica.Blocks.Nonlinear.Limiter limiter1(uMax = 1e6, uMin = 125) annotation (
          Placement(visible = true, transformation(origin = {14, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        Modelica.Blocks.Continuous.FirstOrder genTauFilt(T = 1) annotation (
          Placement(visible = true, transformation(origin = {84, 38}, extent = {{10, -10}, {-10, 10}}, rotation = -90)));
        Modelica.Blocks.Math.Add add annotation (
          Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = -90, origin = {18, 8})));
        Modelica.Blocks.Math.Feedback fbSOC annotation (
          Placement(transformation(extent = {{-50, 38}, {-30, 18}})));
        Modelica.Blocks.Math.Gain socErrToPow(k = socLoopGain) annotation (
          Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {-10, 28})));
        Modelica.Blocks.Sources.Constant socRef_(k = socRef) annotation (
          Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-54, -2})));
      equation
        connect(powFilt.u, conn1.motPowDelB) annotation (
          Line(points = {{20, 58}, {20, 58}, {20, 64}, {20, 64}, {0, 64}, {0, 80}}, color = {0, 0, 127}));
        connect(gain.u, feedback.y) annotation (
          Line(points = {{84, -16}, {84, -16}, {84, -40}, {82, -40}, {56, -40}, {56, -40}, {55, -40}}, color = {0, 0, 127}));
        connect(feedback.u2, conn1.iceW) annotation (
          Line(points = {{46, -32}, {46, 4}, {40, 4}, {40, 64}, {0, 64}, {0, 80}}, color = {0, 0, 127}, smooth = Smooth.None),
          Text(string = "%second", index = 1, extent = {{6, 3}, {6, 3}}));
        connect(toNm.u, tauReference) annotation (
          Line(points = {{-86, 18}, {-86, 0}, {-120, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(toNm.y, conn1.motTauRef) annotation (
          Line(points = {{-86, 41}, {-86, 80}, {0, 80}}, color = {0, 0, 127}, smooth = Smooth.None),
          Text(string = "%second", index = 1, extent = {{6, 3}, {6, 3}}));
        connect(feedback.u1, limiter1.y) annotation (
          Line(points = {{38, -40}, {25, -40}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(limiter1.u, toIceWref.y[1]) annotation (
          Line(points = {{2, -40}, {-5, -40}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(genTauFilt.u, gain.y) annotation (
          Line(points = {{84, 26}, {84, 7}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(powFilt.y, add.u1) annotation (
          Line(points = {{20, 35}, {20, 26}, {24, 26}, {24, 20}}, color = {0, 0, 127}));
        connect(socErrToPow.y, add.u2) annotation (
          Line(points = {{1, 28}, {12, 28}, {12, 20}}, color = {0, 0, 127}));
        connect(fbSOC.y, socErrToPow.u) annotation (
          Line(points = {{-31, 28}, {-22, 28}}, color = {0, 0, 127}));
        connect(add.y, toIceWref.u) annotation (
          Line(points = {{18, -3}, {18, -3}, {18, -10}, {18, -12}, {18, -18}, {-40, -18}, {-40, -40}, {-28, -40}}, color = {0, 0, 127}));
        connect(fbSOC.u2, conn1.batSOC) annotation (
          Line(points = {{-40, 36}, {-40, 36}, {-40, 80}, {0, 80}}, color = {0, 0, 127}),
          Text(string = "%second", index = 1, extent = {{6, 3}, {6, 3}}));
        connect(socRef_.y, fbSOC.u1) annotation (
          Line(points = {{-54, 9}, {-54, 28}, {-48, 28}}, color = {0, 0, 127}));
        connect(add.y, conn1.icePowRef) annotation (
          Line(points = {{18, -3}, {18, -3}, {18, -18}, {18, -12}, {56, -12}, {56, 74}, {0, 74}, {0, 80}}, color = {0, 0, 127}),
          Text(string = "%second", index = 1, extent = {{6, 3}, {6, 3}}));
        connect(genTauFilt.y, conn1.genTauRef) annotation (
          Line(points = {{84, 49}, {84, 49}, {84, 80}, {0, 80}}, color = {0, 0, 127}),
          Text(string = "%second", index = 1, extent = {{6, 3}, {6, 3}}));
        annotation (
          Diagram(coordinateSystem(extent = {{-100, -60}, {100, 80}}, preserveAspectRatio = false, initialScale = 0.1), graphics={  Text(extent = {{-84, 68}, {-70, 56}}, textString = "Send 
requested
torque 
to mot", horizontalAlignment = TextAlignment.Left), Text(extent = {{54, 68}, {82, 58}}, textString = "send 
reference tau
to gen", horizontalAlignment = TextAlignment.Right), Text(extent = {{28, 66}, {54, 54}}, textString = "send 
ref pow
to ice", horizontalAlignment = TextAlignment.Right)}),
          Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = false, initialScale = 0.1, grid = {2, 2}), graphics={  Text(lineColor = {0, 0, 255}, extent = {{-100, -102}, {100, -140}}, textString = "%name"), Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-4, -40}, {74, 16}, {74, -6}, {-4, -62}, {-4, -40}}, lineColor = {95, 95, 95}, fillColor = {175, 175, 175},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{8, -38}, {28, -48}, {20, -54}, {0, -44}, {8, -38}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{20, -54}, {28, -48}, {32, -56}, {24, -62}, {20, -54}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{24, -62}, {32, -56}, {32, -78}, {24, -84}, {24, -62}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 127},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{0, -44}, {20, -54}, {24, -62}, {24, -84}, {22, -84}, {22, -62}, {20, -58}, {0, -48}, {0, -44}}, lineColor = {0, 0, 255}, fillColor = {191, 191, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-14, 40}, {-18, 32}, {-10, 38}, {-8, 44}, {-14, 40}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-18, 32}, {-10, 38}, {-10, 14}, {-18, 8}, {-18, 32}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 127},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-20, 10}, {-20, 32}, {-16, 40}, {4, 30}, {4, 26}, {-16, 36}, {-18, 32}, {-18, 8}, {-20, 10}}, lineColor = {0, 0, 255}, fillColor = {191, 191, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-8, 46}, {12, 36}, {4, 30}, {-16, 40}, {-8, 46}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{28, -22}, {48, -32}, {40, -38}, {20, -28}, {28, -22}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{40, -38}, {48, -32}, {52, -40}, {44, -46}, {40, -38}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{44, -46}, {52, -40}, {52, -62}, {44, -68}, {44, -46}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 127},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{20, -28}, {40, -38}, {44, -46}, {44, -68}, {42, -68}, {42, -46}, {40, -42}, {20, -32}, {20, -28}}, lineColor = {0, 0, 255}, fillColor = {191, 191, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{48, -8}, {68, -18}, {60, -24}, {40, -14}, {48, -8}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{60, -24}, {68, -18}, {72, -26}, {64, -32}, {60, -24}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{64, -32}, {72, -26}, {72, -48}, {64, -54}, {64, -32}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 127},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{40, -14}, {60, -24}, {64, -32}, {64, -54}, {62, -54}, {62, -32}, {60, -28}, {40, -18}, {40, -14}}, lineColor = {0, 0, 255}, fillColor = {191, 191, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{68, 6}, {88, -4}, {80, -10}, {60, 0}, {68, 6}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{80, -10}, {88, -4}, {92, -12}, {84, -18}, {80, -10}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{84, -18}, {92, -12}, {92, -34}, {84, -40}, {84, -18}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 127},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{60, 0}, {80, -10}, {84, -18}, {84, -40}, {82, -40}, {82, -18}, {80, -14}, {60, -4}, {60, 0}}, lineColor = {0, 0, 255}, fillColor = {191, 191, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-34, 26}, {-38, 18}, {-30, 24}, {-28, 30}, {-34, 26}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-38, 18}, {-30, 24}, {-30, 0}, {-38, -6}, {-38, 18}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 127},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-40, -4}, {-40, 18}, {-36, 26}, {-16, 16}, {-16, 12}, {-36, 22}, {-38, 18}, {-38, -6}, {-40, -4}}, lineColor = {0, 0, 255}, fillColor = {191, 191, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-28, 32}, {-8, 22}, {-16, 16}, {-36, 26}, {-28, 32}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-54, 12}, {-58, 4}, {-50, 10}, {-48, 16}, {-54, 12}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-58, 4}, {-50, 10}, {-50, -14}, {-58, -20}, {-58, 4}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 127},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-60, -18}, {-60, 4}, {-56, 12}, {-36, 2}, {-36, -2}, {-56, 8}, {-58, 4}, {-58, -20}, {-60, -18}}, lineColor = {0, 0, 255}, fillColor = {191, 191, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-48, 18}, {-28, 8}, {-36, 2}, {-56, 12}, {-48, 18}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-74, -4}, {-78, -12}, {-70, -6}, {-68, 0}, {-74, -4}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-78, -12}, {-70, -6}, {-70, -30}, {-78, -36}, {-78, -12}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 127},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-80, -34}, {-80, -12}, {-76, -4}, {-56, -14}, {-56, -18}, {-76, -8}, {-78, -12}, {-78, -36}, {-80, -34}}, lineColor = {0, 0, 255}, fillColor = {191, 191, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-68, 2}, {-48, -8}, {-56, -14}, {-76, -4}, {-68, 2}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-64, -8}, {-4, -40}, {-4, -62}, {-64, -30}, {-64, -8}}, lineColor = {95, 95, 95}, fillColor = {75, 75, 75},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-64, -8}, {-4, -40}, {74, 16}, {14, 48}, {-64, -8}}, lineColor = {95, 95, 95}, fillColor = {160, 160, 164},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Rectangle(extent = {{-98, 92}, {98, 62}}, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, pattern = LinePattern.None), Text(extent = {{-100, 82}, {100, 54}}, lineColor=
                    {0,0,0},
                textString="PSD-ecu2")}),
          Documentation(info = "<html>
<p><b><span style=\"font-family: MS Shell Dlg 2;\">Power Split Power Train Controller without ON/OFF</span></b></p>
<p><span style=\"font-family: MS Shell Dlg 2;\">This controller is derived from MBecu1, in which the basic description can be found.</span></p>
<p><span style=\"font-family: MS Shell Dlg 2;\">It adds a soc control loop to avoid soc Drifts.</span></p>
</html>"));
      end Ecu2;

      model Ecu3 "Power Split hybrid power train controller, using ON/OFF strategy"
        parameter Real socRef = 0.6 "Reference soc";
        parameter Real maxTorqueReq = 80 "Maximum torque that can be requested from mot";
        parameter Real powFiltT = 60 "Power filter time constant (s)";
        parameter Real socLoopGain = 50e3 "gain of the soc loop (w/pu)";
        parameter Real genLoopGain = 0.02 "gain of the soc loop (Nm/(rad/s))";
        parameter Real onThreshold = 7000 "average power over which engine is switched on (W)";
        parameter Real offThreshold = 5000 "average power below which engine is switched off (W)";
        Modelica.Blocks.Interfaces.RealInput tauReference annotation (
          Placement(visible = true, transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0), iconTransformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Modelica.Blocks.Continuous.FirstOrder powFilt(T = powFiltT) annotation (
          Placement(visible = true, transformation(origin = {-50, 58}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
        SupportModels.ConnectorRelated.Conn conn annotation (
          Placement(visible = true, transformation(extent = {{-20, 78}, {20, 118}}, rotation = 0), iconTransformation(extent = {{-20, 78}, {20, 118}}, rotation = 0)));
        Modelica.Blocks.Math.Gain gain(k = genLoopGain) annotation (
          Placement(visible = true, transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {104, 30})));
        Modelica.Blocks.Math.Gain toNm(k = maxTorqueReq) "converts p.u. torque request into Nm" annotation (
          Placement(visible = true, transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-88, 50})));
        Modelica.Blocks.Continuous.FirstOrder genTauFilt(T = 1) annotation (
          Placement(visible = true, transformation(origin = {104, 76}, extent = {{10, -10}, {-10, 10}}, rotation = -90)));
        Modelica.Blocks.Math.Add add annotation (
          Placement(visible = true, transformation(origin = {-36, 12}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        Modelica.Blocks.Tables.CombiTable1Ds powToW(fileName = "wToTau.txt", tableOnFile = false, table = [0, 0; 1884, 126; 9800, 126; 36600, 366; 52300, 523]) "optimal ice speed as a function of power" annotation (
          Placement(visible = true, transformation(origin = {64, -16}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
        Modelica.Blocks.Nonlinear.Limiter iceSpeedLimiter(uMax = 1e6, uMin = 125) annotation (
          Placement(visible = true, transformation(origin = {64, 16}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
        Modelica.Blocks.Math.Feedback feedback annotation (
          Placement(visible = true, transformation(extent = {{-10, 10}, {10, -10}}, rotation = 90, origin = {64, 46})));
        Modelica.Blocks.Sources.Constant socRef_(k = socRef) annotation (
          Placement(visible = true, transformation(extent = {{-98, -34}, {-78, -14}}, rotation = 0)));
        Modelica.Blocks.Math.Feedback fbSOC annotation (
          Placement(visible = true, transformation(extent = {{-74, -14}, {-54, -34}}, rotation = 0)));
        Modelica.Blocks.Math.Gain socErrrToPow(k = socLoopGain) annotation (
          Placement(visible = true, transformation(origin = {-38, -24}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        Modelica.Blocks.Logical.Hysteresis hysteresis(uLow = offThreshold, uHigh = onThreshold) annotation (
          Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-18, 42})));
        Modelica.Blocks.Logical.Switch switch1 annotation (
          Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {12, 12})));
        Modelica.Blocks.Sources.Constant constZero(k = 0) annotation (
          Placement(transformation(extent = {{20, -30}, {0, -10}})));
      equation
        connect(socErrrToPow.y, add.u2) annotation (
          Line(points = {{-27, -24}, {-22, -24}, {-22, -4}, {-58, -4}, {-58, 6}, {-48, 6}}, color = {0, 0, 127}));
        connect(socErrrToPow.u, fbSOC.y) annotation (
          Line(points = {{-50, -24}, {-55, -24}}, color = {0, 0, 127}));
        connect(fbSOC.u2, conn.batSOC) annotation (
          Line(points = {{-64, -16}, {-64, -12}, {-68, -12}, {-68, 98}, {0, 98}}, color = {0, 0, 127}));
        connect(fbSOC.u1, socRef_.y) annotation (
          Line(points = {{-72, -24}, {-77, -24}}, color = {0, 0, 127}));
        connect(feedback.u1, iceSpeedLimiter.y) annotation (
          Line(points = {{64, 38}, {64, 38}, {64, 28}, {64, 27}}, color = {0, 0, 127}));
        connect(feedback.u2, conn.iceW) annotation (
          Line(points = {{56, 46}, {56, 72}, {56, 98}, {0, 98}}, color = {0, 0, 127}));
        connect(gain.u, feedback.y) annotation (
          Line(points = {{104, 18}, {104, 18}, {104, 0}, {88, 0}, {88, 55}, {64, 55}}, color = {0, 0, 127}));
        connect(iceSpeedLimiter.u, powToW.y[1]) annotation (
          Line(points = {{64, 4}, {64, -5}}, color = {0, 0, 127}));
        connect(add.u1, powFilt.y) annotation (
          Line(points = {{-48, 18}, {-54, 18}, {-58, 18}, {-58, 42}, {-50, 42}, {-50, 47}}, color = {0, 0, 127}));
        connect(powFilt.u, conn.motPowDelB) annotation (
          Line(points = {{-50, 70}, {-50, 78}, {0, 78}, {0, 98}}, color = {0, 0, 127}));
        connect(toNm.u, tauReference) annotation (
          Line(points = {{-88, 38}, {-88, 0}, {-120, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(toNm.y, conn.motTauRef) annotation (
          Line(points = {{-88, 61}, {-88, 98}, {0, 98}}, color = {0, 0, 127}, smooth = Smooth.None),
          Text(string = "%second", index = 1, extent = {{6, 3}, {6, 3}}));
        connect(gain.y, genTauFilt.u) annotation (
          Line(points = {{104, 41}, {104, 64}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(genTauFilt.y, conn.genTauRef) annotation (
          Line(points = {{104, 87}, {104, 98}, {0, 98}}, color = {0, 0, 127}, smooth = Smooth.None),
          Text(string = "%second", index = 1, extent = {{6, 3}, {6, 3}}));
        connect(hysteresis.u, add.y) annotation (
          Line(points = {{-18, 30}, {-18, 12}, {-25, 12}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(hysteresis.y, conn.iceON) annotation (
          Line(points = {{-18, 53}, {-18, 92}, {0, 92}, {0, 98}}, color = {255, 0, 255}, smooth = Smooth.None),
          Text(string = "%second", index = 1, extent = {{6, 3}, {6, 3}}));
        connect(powToW.u, switch1.y) annotation (
          Line(points = {{64, -28}, {30, -28}, {30, 12}, {23, 12}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(switch1.u1, add.y) annotation (
          Line(points = {{0, 20}, {-18, 20}, {-18, 12}, {-25, 12}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(switch1.u3, constZero.y) annotation (
          Line(points = {{0, 4}, {-8, 4}, {-8, -20}, {-1, -20}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(switch1.u2, hysteresis.y) annotation (
          Line(points = {{0, 12}, {-8, 12}, {-8, 30}, {0, 30}, {0, 62}, {-18, 62}, {-18, 53}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(powToW.u, conn.icePowRef) annotation (
          Line(points = {{64, -28}, {30, -28}, {30, 72}, {0, 72}, {0, 98}}, color = {0, 0, 127}, smooth = Smooth.None),
          Text(string = "%second", index = 1, extent = {{6, 3}, {6, 3}}));
        annotation (
          Documentation(info = "<html>
<p><b><span style=\"font-family: MS Shell Dlg 2;\">Power Split Power Train Controller with ON/OFF</span></b></p>
<p><span style=\"font-family: MS Shell Dlg 2;\">This controller operates as follows:</span></p>
<ul>
<li><span style=\"font-family: MS Shell Dlg 2;\">it makes the ice deliver the average power needed by the power train</span></li>
<li><span style=\"font-family: MS Shell Dlg 2;\">it determines the optimal ice speed at which the requested power is delivered with minimum fuel consumption and asks the &QUOT;gen&QUOT; to control so that the ice opertes at that speed</span></li>
<li><span style=\"font-family: MS Shell Dlg 2;\">the vehicle motion is controlled acting on the &QUOT;mot&QUOT;.</span></li>
<li><span style=\"font-family: MS Shell Dlg 2;\">a closed-loop SOC control avoids the battery do become too charged or discharged</span></li>
<li><span style=\"font-family: MS Shell Dlg 2;\">an ON/OFF control determines ICe switching OFF when the looad is to loow and switching it ON again when the requested power is sifnigicntly high. this normally reduces fuel consumpton.</span></li>
</ul>
<p><span style=\"font-family: MS Shell Dlg 2;\">So:</span></p>
<ul>
<li><span style=\"font-family: MS Shell Dlg 2;\">powFilt Block filters the delivered power to obtained the power to ask the ICE to deliver</span></li>
<li><span style=\"font-family: MS Shell Dlg 2;\">toIceWref converts the power to be requested from the ICE by its maximum torque at the actual speed</span></li>
<li><span style=\"font-family: MS Shell Dlg 2;\">after a limiting block, this torque is the reference signal of a feedback; the corresponnding error controls the Gen torque.</span></li>
<li><span style=\"font-family: MS Shell Dlg 2;\">fbSOC sis the feedback for SOC control and socLoopGain is its gain</span></li>
<li><span style=\"font-family: MS Shell Dlg 2;\">hysteresis manages switching ON/OFF the ice. </span></li>
</ul>
<p><span style=\"font-family: MS Shell Dlg 2;\"></p><p>Details of ice going to off (e.g. bringing its speed to zero) and to on (i.e. first making ice speed to rise, then start sending fuel) are not implemented.</span></p>
</html>"),Diagram(coordinateSystem(extent = {{-100, -40}, {120, 100}}, preserveAspectRatio = false, initialScale = 0.1, grid = {2, 2}), graphics={  Text(extent = {{-86, 92}, {-28, 88}}, textString = "Send requested torque to mot"), Text(extent = {{12, 50}, {40, 46}}, textString = "send reterence tau
 to gen"), Ellipse(extent = {{-44, 100}, {48, -42}}, lineColor = {255, 0, 0},
                  lineThickness =                                                             0.5)}),
          Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = false, initialScale = 0.1, grid = {2, 2}), graphics={  Text(lineColor = {0, 0, 255}, extent = {{-102, -102}, {98, -140}}, textString = "%name"), Rectangle(fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-100, 100}, {100, -100}}), Polygon(lineColor = {95, 95, 95}, fillColor = {175, 175, 175},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-4, -40}, {74, 16}, {74, -6}, {-4, -62}, {-4, -40}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{8, -38}, {28, -48}, {20, -54}, {0, -44}, {8, -38}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{20, -54}, {28, -48}, {32, -56}, {24, -62}, {20, -54}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 127},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{24, -62}, {32, -56}, {32, -78}, {24, -84}, {24, -62}}), Polygon(lineColor = {0, 0, 255}, fillColor = {191, 191, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{0, -44}, {20, -54}, {24, -62}, {24, -84}, {22, -84}, {22, -62}, {20, -58}, {0, -48}, {0, -44}}), Polygon(lineColor = {128, 128, 128}, fillColor = {128, 128, 128},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-14, 40}, {-18, 32}, {-10, 38}, {-8, 44}, {-14, 40}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 127},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-18, 32}, {-10, 38}, {-10, 14}, {-18, 8}, {-18, 32}}), Polygon(lineColor = {0, 0, 255}, fillColor = {191, 191, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-20, 10}, {-20, 32}, {-16, 40}, {4, 30}, {4, 26}, {-16, 36}, {-18, 32}, {-18, 8}, {-20, 10}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-8, 46}, {12, 36}, {4, 30}, {-16, 40}, {-8, 46}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{28, -22}, {48, -32}, {40, -38}, {20, -28}, {28, -22}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{40, -38}, {48, -32}, {52, -40}, {44, -46}, {40, -38}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 127},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{44, -46}, {52, -40}, {52, -62}, {44, -68}, {44, -46}}), Polygon(lineColor = {0, 0, 255}, fillColor = {191, 191, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{20, -28}, {40, -38}, {44, -46}, {44, -68}, {42, -68}, {42, -46}, {40, -42}, {20, -32}, {20, -28}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{48, -8}, {68, -18}, {60, -24}, {40, -14}, {48, -8}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{60, -24}, {68, -18}, {72, -26}, {64, -32}, {60, -24}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 127},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{64, -32}, {72, -26}, {72, -48}, {64, -54}, {64, -32}}), Polygon(lineColor = {0, 0, 255}, fillColor = {191, 191, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{40, -14}, {60, -24}, {64, -32}, {64, -54}, {62, -54}, {62, -32}, {60, -28}, {40, -18}, {40, -14}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{68, 6}, {88, -4}, {80, -10}, {60, 0}, {68, 6}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{80, -10}, {88, -4}, {92, -12}, {84, -18}, {80, -10}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 127},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{84, -18}, {92, -12}, {92, -34}, {84, -40}, {84, -18}}), Polygon(lineColor = {0, 0, 255}, fillColor = {191, 191, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{60, 0}, {80, -10}, {84, -18}, {84, -40}, {82, -40}, {82, -18}, {80, -14}, {60, -4}, {60, 0}}), Polygon(lineColor = {128, 128, 128}, fillColor = {128, 128, 128},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-34, 26}, {-38, 18}, {-30, 24}, {-28, 30}, {-34, 26}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 127},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-38, 18}, {-30, 24}, {-30, 0}, {-38, -6}, {-38, 18}}), Polygon(lineColor = {0, 0, 255}, fillColor = {191, 191, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-40, -4}, {-40, 18}, {-36, 26}, {-16, 16}, {-16, 12}, {-36, 22}, {-38, 18}, {-38, -6}, {-40, -4}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-28, 32}, {-8, 22}, {-16, 16}, {-36, 26}, {-28, 32}}), Polygon(lineColor = {128, 128, 128}, fillColor = {128, 128, 128},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-54, 12}, {-58, 4}, {-50, 10}, {-48, 16}, {-54, 12}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 127},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-58, 4}, {-50, 10}, {-50, -14}, {-58, -20}, {-58, 4}}), Polygon(lineColor = {0, 0, 255}, fillColor = {191, 191, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-60, -18}, {-60, 4}, {-56, 12}, {-36, 2}, {-36, -2}, {-56, 8}, {-58, 4}, {-58, -20}, {-60, -18}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-48, 18}, {-28, 8}, {-36, 2}, {-56, 12}, {-48, 18}}), Polygon(lineColor = {128, 128, 128}, fillColor = {128, 128, 128},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-74, -4}, {-78, -12}, {-70, -6}, {-68, 0}, {-74, -4}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 127},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-78, -12}, {-70, -6}, {-70, -30}, {-78, -36}, {-78, -12}}), Polygon(lineColor = {0, 0, 255}, fillColor = {191, 191, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-80, -34}, {-80, -12}, {-76, -4}, {-56, -14}, {-56, -18}, {-76, -8}, {-78, -12}, {-78, -36}, {-80, -34}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-68, 2}, {-48, -8}, {-56, -14}, {-76, -4}, {-68, 2}}), Polygon(lineColor = {95, 95, 95}, fillColor = {75, 75, 75},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-64, -8}, {-4, -40}, {-4, -62}, {-64, -30}, {-64, -8}}), Polygon(lineColor = {95, 95, 95}, fillColor = {160, 160, 164},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-64, -8}, {-4, -40}, {74, 16}, {14, 48}, {-64, -8}}), Rectangle(fillColor = {255, 255, 255}, pattern = LinePattern.None,
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-98, 92}, {98, 62}}), Text(extent = {{-100, 84}, {100, 54}}, lineColor=
                    {0,0,0},
                textString="PSD-ecu3
")}));
      end Ecu3;

      model GMS "Genset Management System (simplified)"
        parameter Real contrGain = 0.01 "speed controller gain (throttle per rad/s)";
        parameter String mapsFileName = "maps.txt" "File name where optimal speed is stored";
        import Modelica.Constants.pi;
        Modelica.Blocks.Tables.CombiTable1D optiSpeed(tableOnFile = true, columns = {2}, tableName = "optiSpeed", fileName = mapsFileName) "gives the optimal speed as a function of requested power" annotation (
          Placement(transformation(extent = {{-42, -50}, {-22, -30}})));
        Modelica.Blocks.Math.Division division annotation (
          Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-30, 48})));
        Modelica.Blocks.Interfaces.RealInput Wmecc annotation (
          Placement(transformation(extent = {{-15, -15}, {15, 15}}, rotation = 90, origin = {1, -115}), iconTransformation(extent = {{-15, -15}, {15, 15}}, rotation = 90, origin = {1, -115})));
        Modelica.Blocks.Interfaces.RealInput pRef annotation (
          Placement(transformation(extent = {{-134, -20}, {-94, 20}}), iconTransformation(extent = {{-140, -20}, {-100, 20}})));
        Modelica.Blocks.Interfaces.RealOutput tRef "Torque request (positive when ICE delivers power)" annotation (
          Placement(transformation(extent = {{100, 50}, {120, 70}}), iconTransformation(extent = {{100, 50}, {120, 70}})));
        Modelica.Blocks.Interfaces.RealOutput throttle annotation (
          Placement(transformation(extent = {{100, -70}, {120, -50}}), iconTransformation(extent = {{100, -70}, {120, -50}})));
        Modelica.Blocks.Math.Feedback feedback annotation (
          Placement(transformation(extent = {{24, -50}, {44, -30}})));
        Modelica.Blocks.Math.Gain gain(k = 0.01) annotation (
          Placement(transformation(extent = {{66, -50}, {86, -30}})));
        Modelica.Blocks.Math.UnitConversions.To_rpm to_rpm annotation (
          Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {34, -70})));
        Modelica.Blocks.Tables.CombiTable1D maxTau(tableOnFile = true, columns = {2}, fileName = mapsFileName, tableName = "maxIceTau") "gives the optimal spees ad a function of requested power" annotation (
          Placement(transformation(extent = {{6, 68}, {26, 88}})));
        Modelica.Blocks.Nonlinear.VariableLimiter tauLimiter annotation (
          Placement(transformation(extent = {{48, 50}, {68, 70}})));
        Modelica.Blocks.Math.Gain gain1(k = -1) annotation (
          Placement(transformation(extent = {{20, 28}, {36, 44}})));
      equation
        connect(division.u1, optiSpeed.u[1]) annotation (
          Line(points = {{-36, 36}, {-36, 0}, {-60, 0}, {-60, -40}, {-44, -40}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(optiSpeed.u[1], pRef) annotation (
          Line(points = {{-44, -40}, {-60, -40}, {-60, 0}, {-114, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(throttle, gain.y) annotation (
          Line(points = {{110, -60}, {98, -60}, {98, -40}, {87, -40}}, color = {0, 0, 127}));
        connect(feedback.y, gain.u) annotation (
          Line(points = {{43, -40}, {64, -40}}, color = {0, 0, 127}));
        connect(feedback.u1, optiSpeed.y[1]) annotation (
          Line(points = {{26, -40}, {4, -40}, {-21, -40}}, color = {0, 0, 127}));
        connect(to_rpm.y, feedback.u2) annotation (
          Line(points = {{34, -59}, {34, -59}, {34, -48}}, color = {0, 0, 127}));
        connect(to_rpm.u, Wmecc) annotation (
          Line(points = {{34, -82}, {34, -94}, {1, -94}, {1, -115}}, color = {0, 0, 127}));
        connect(division.u2, Wmecc) annotation (
          Line(points = {{-24, 36}, {-24, 0}, {-6, 0}, {-6, -96}, {1, -96}, {1, -115}}, color = {0, 0, 127}));
        connect(tauLimiter.y, tRef) annotation (
          Line(points = {{69, 60}, {84, 60}, {110, 60}}, color = {0, 0, 127}));
        connect(tauLimiter.limit1, maxTau.y[1]) annotation (
          Line(points = {{46, 68}, {36, 68}, {36, 78}, {27, 78}}, color = {0, 0, 127}));
        connect(maxTau.u[1], Wmecc) annotation (
          Line(points = {{4, 78}, {0, 78}, {0, -115}, {1, -115}}, color = {0, 0, 127}));
        connect(division.y, tauLimiter.u) annotation (
          Line(points = {{-30, 59}, {-32, 59}, {-32, 60}, {46, 60}}, color = {0, 0, 127}));
        connect(gain1.y, tauLimiter.limit2) annotation (
          Line(points = {{36.8, 36}, {40, 36}, {40, 34}, {40, 52}, {46, 52}}, color = {0, 0, 127}));
        connect(gain1.u, maxTau.y[1]) annotation (
          Line(points = {{18.4, 36}, {12, 36}, {12, 54}, {36, 54}, {36, 78}, {27, 78}}, color = {0, 0, 127}));
        annotation (
          Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}})),
          experiment(StopTime = 3, Interval = 0.01),
          experimentSetupOutput,
          Icon(coordinateSystem(initialScale = 0.1), graphics={  Rectangle(lineColor = {0, 0, 127}, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                FillPattern.Solid, extent = {{-100, 100}, {100, -100}}), Text(origin = {-2, 0}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-98, 22}, {98, -16}}, textString = "%name")}),
          Documentation(info = "<html>
<p>Genset Management System.</p>
<p>The control logic commands the genset to deliver at the DC port the input power, using the optimal generator speed.</p>
</html>"));
      end GMS;

      model GMSoo "Genset Management System (simplified)"
        parameter Real tauMax = 100 "maximum torque internally exchanged by the two machines";
        parameter Real throttlePerWerr = 0.01 "speed controller gain (throttle per rad/s)";
        parameter Boolean tablesOnFile = false;
        parameter String mapsFileName = "maps.txt" "Name of the file containing data maps (names: maxIceTau, specificCons, optiSpeed)" annotation (
          Dialog(enable = tablesOnFile));
        parameter Real optiTable[:, :] = [0, 800; 20000, 850; 40000, 1100; 60000, 1250; 80000, 1280; 100000, 1340; 120000, 1400; 140000, 1650; 160000, 2130] "first row: speed, 1st column: torque, body: sp. consumption" annotation (
          Dialog(enable = not tablesOnFile));
        import Modelica.Constants.pi;
        Modelica.Blocks.Tables.CombiTable1D optiSpeed(columns = {2}, smoothness = Modelica.Blocks.Types.Smoothness.ContinuousDerivative, table = optiTable, fileName = mapsFileName, tableOnFile = tablesOnFile, tableName = "optiSpeed") "gives the optimal spees ad a function of requested power" annotation (
          Placement(visible = true, transformation(extent = {{-66, -22}, {-46, -2}}, rotation = 0)));
        Modelica.Blocks.Math.Division division annotation (
          Placement(visible = true, transformation(origin = {4, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
        Modelica.Blocks.Nonlinear.Limiter limiter(uMax = tauMax) annotation (
          Placement(visible = true, transformation(origin = {52, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput Wmecc annotation (
          Placement(visible = true, transformation(origin = {0, -96}, extent = {{-20, -20}, {20, 20}}, rotation = 90), iconTransformation(origin = {1, -115}, extent = {{-15, -15}, {15, 15}}, rotation = 90)));
        Modelica.Blocks.Interfaces.RealInput pRef annotation (
          Placement(visible = true, transformation(extent = {{-140, -60}, {-100, -20}}, rotation = 0), iconTransformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput tRef "Torque request (positive when ICE delivers power)" annotation (
          Placement(visible = true, transformation(extent = {{100, 30}, {120, 50}}, rotation = 0), iconTransformation(extent = {{100, 50}, {120, 70}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput throttle annotation (
          Placement(visible = true, transformation(extent = {{100, -50}, {120, -30}}, rotation = 0), iconTransformation(extent = {{100, -70}, {120, -50}}, rotation = 0)));
        Modelica.Blocks.Math.Feedback feedback annotation (
          Placement(visible = true, transformation(extent = {{24, -30}, {44, -10}}, rotation = 0)));
        Modelica.Blocks.Math.Gain gain(k = throttlePerWerr) annotation (
          Placement(visible = true, transformation(extent = {{66, -30}, {86, -10}}, rotation = 0)));
        Modelica.Blocks.Math.UnitConversions.To_rpm to_rpm annotation (
          Placement(visible = true, transformation(origin = {34, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
        Modelica.Blocks.Interfaces.BooleanInput on annotation (
          Placement(visible = true, transformation(extent = {{-138, 28}, {-98, 68}}, rotation = 0), iconTransformation(extent = {{-138, 40}, {-98, 80}}, rotation = 0)));
        Modelica.Blocks.Logical.Switch switch1 annotation (
          Placement(visible = true, transformation(extent = {{-18, -30}, {2, -10}}, rotation = 0)));
        Modelica.Blocks.Sources.Constant zero(k = 0) annotation (
          Placement(visible = true, transformation(extent = {{-60, -60}, {-40, -40}}, rotation = 0)));
        Modelica.Blocks.Nonlinear.Limiter limMinW(uMax = 1e9, uMin = 10) annotation (
          Placement(visible = true, transformation(origin = {4, -48}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
      equation
        connect(to_rpm.u, Wmecc) annotation (
          Line(points = {{34, -62}, {34, -70}, {0, -70}, {0, -96}}, color = {0, 0, 127}));
        connect(zero.y, switch1.u3) annotation (
          Line(points = {{-39, -50}, {-30, -50}, {-30, -28}, {-20, -28}}, color = {0, 0, 127}));
        connect(on, switch1.u2) annotation (
          Line(points = {{-118, 48}, {-34, 48}, {-34, -20}, {-20, -20}}, color = {255, 0, 255}));
        connect(optiSpeed.y[1], switch1.u1) annotation (
          Line(points = {{-45, -12}, {-45, -12}, {-20, -12}}, color = {0, 0, 127}));
        connect(switch1.y, feedback.u1) annotation (
          Line(points = {{3, -20}, {26, -20}}, color = {0, 0, 127}));
        connect(to_rpm.y, feedback.u2) annotation (
          Line(points = {{34, -39}, {34, -39}, {34, -28}}, color = {0, 0, 127}));
        connect(feedback.y, gain.u) annotation (
          Line(points = {{43, -20}, {64, -20}}, color = {0, 0, 127}));
        connect(throttle, gain.y) annotation (
          Line(points = {{110, -40}, {98, -40}, {98, -20}, {87, -20}}, color = {0, 0, 127}));
        connect(limiter.y, tRef) annotation (
          Line(points = {{63, 40}, {110, 40}}, color = {0, 0, 127}));
        connect(limiter.u, division.y) annotation (
          Line(points = {{40, 40}, {4, 40}, {4, 31}}, color = {0, 0, 127}));
        connect(division.u1, optiSpeed.u[1]) annotation (
          Line(points = {{-2, 8}, {-80, 8}, {-80, -12}, {-68, -12}}, color = {0, 0, 127}));
        connect(optiSpeed.u[1], pRef) annotation (
          Line(points = {{-68, -12}, {-80, -12}, {-80, -40}, {-120, -40}}, color = {0, 0, 127}));
        connect(limMinW.u, Wmecc) annotation (
          Line(points = {{4, -60}, {4, -70}, {0, -70}, {0, -96}}, color = {0, 0, 127}));
        connect(limMinW.y, division.u2) annotation (
          Line(points = {{4, -37}, {6, -37}, {6, -34}, {6, 8}, {10, 8}}, color = {0, 0, 127}));
        annotation (
          Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -80}, {100, 60}})),
          experiment(StopTime = 3, Interval = 0.01),
          experimentSetupOutput,
          Icon(coordinateSystem(initialScale = 0.1), graphics={  Rectangle(lineColor = {0, 0, 127}, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                FillPattern.Solid, extent = {{-100, 100}, {100, -100}}), Text(origin = {-2, 0}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-98, 22}, {98, -16}}, textString = "%name")}),
          Documentation(info = "<html>
<p>Genset Management System witn ON/OFF.</p>
<p>The control logic commands the genset to deliver at the DC port the input power, using the optimal generator speed.</p>
<p>In addition, it commands ON or OFF depending on the input boolean control signal.</p>
</html>"),__OpenModelica_commandLineOptions = "");
      end GMSoo;

      block EleBalanceTau
        parameter Real sigma;
        Modelica.Blocks.Interfaces.RealInput motW annotation (
          Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = -90, origin = {-60, 116})));
        Modelica.Blocks.Interfaces.RealInput iceW annotation (
          Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = -90, origin = {60, 114})));
        Modelica.Blocks.Interfaces.RealOutput tauMot annotation (
          Placement(transformation(extent = {{98, -10}, {118, 10}})));
        Modelica.Blocks.Interfaces.RealInput tauP annotation (
          Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = -90, origin = {0, 116})));
      equation
        tauMot = tauP * ((1 + sigma) * iceW - motW) / ((1 + sigma) * iceW);
        annotation (
          Icon(coordinateSystem(preserveAspectRatio = false)),
          Diagram(coordinateSystem(preserveAspectRatio = false)));
      end EleBalanceTau;

      model EMS "Ice, Generator, DriveTrain, all map-based"
        //€
        parameter Real tauPowFilt = 300 "power filter time constant";
        parameter Real powLow = 3000 "hysteresis control lower limit";
        parameter Real powHigh = 5000 "hysteresis control higher limit";
        parameter Real powPerSoc = 100e3 "SOC loop gain";
        parameter Real powMax = 100e3;
        parameter Real socRef = 0.7;
        Modelica.Blocks.Nonlinear.Limiter limiter(uMin = 0, uMax = powMax) annotation (
          Placement(visible = true, transformation(origin = {12, 46}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        Modelica.Blocks.Math.Feedback fbSOC annotation (
          Placement(visible = true, transformation(origin = {-60, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        Modelica.Blocks.Sources.Constant socRef_(k = socRef) annotation (
          Placement(visible = true, transformation(origin = {-90, 0}, extent = {{10, 10}, {-10, -10}}, rotation = 180)));
        Modelica.Blocks.Math.Gain socErrToPow(k = powPerSoc) annotation (
          Placement(visible = true, transformation(origin = {-40, 22}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
        Modelica.Blocks.Math.Add add annotation (
          Placement(visible = true, transformation(origin = {-18, 46}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput edPow annotation (
          Placement(visible = true, transformation(origin = {-120, 40}, extent = {{-20, -20}, {20, 20}}, rotation = 0), iconTransformation(origin = {-120, 60}, extent = {{-20, -20}, {20, 20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput soc annotation (
          Placement(visible = true, transformation(origin = {-120, -40}, extent = {{-20, -20}, {20, 20}}, rotation = 0), iconTransformation(origin = {-120, -60}, extent = {{-20, -20}, {20, 20}}, rotation = 0)));
        Modelica.Blocks.Logical.Switch powSwitch annotation (
          Placement(visible = true, transformation(origin = {64, -20}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        Modelica.Blocks.Logical.Hysteresis powHyst(uHigh = powHigh, uLow = powLow) annotation (
          Placement(visible = true, transformation(origin = {50, 46}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        Modelica.Blocks.Sources.Constant zero(k = 0.0) annotation (
          Placement(visible = true, transformation(origin = {26, -48}, extent = {{10, 10}, {-10, -10}}, rotation = 180)));
        Modelica.Blocks.Interfaces.BooleanOutput on annotation (
          Placement(visible = true, transformation(extent = {{100, 30}, {120, 50}}, rotation = 0), iconTransformation(extent = {{98, 50}, {118, 70}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput pcPowReq(displayUnit = "kW") annotation (
          Placement(visible = true, transformation(extent = {{100, -50}, {120, -30}}, rotation = 0), iconTransformation(extent = {{98, -70}, {118, -50}}, rotation = 0)));
        Modelica.Blocks.Continuous.FirstOrder powFilt(y_start = 20e3, T = tauPowFilt) annotation (
          Placement(transformation(extent = {{-72, 44}, {-56, 60}})));
      equation
        connect(powSwitch.u1, limiter.y) annotation (
          Line(points = {{52, -12}, {30, -12}, {30, 46}, {23, 46}}, color = {0, 0, 127}));
        connect(powSwitch.u3, zero.y) annotation (
          Line(points = {{52, -28}, {44, -28}, {44, -48}, {37, -48}}, color = {0, 0, 127}));
        connect(powHyst.u, limiter.y) annotation (
          Line(points = {{38, 46}, {23, 46}}, color = {0, 0, 127}));
        connect(socErrToPow.y, add.u2) annotation (
          Line(points = {{-40, 33}, {-40, 40}, {-30, 40}}, color = {0, 0, 127}));
        connect(limiter.u, add.y) annotation (
          Line(points = {{0, 46}, {0, 46}, {-7, 46}}, color = {0, 0, 127}));
        connect(socErrToPow.u, fbSOC.y) annotation (
          Line(points = {{-40, 10}, {-40, 0}, {-51, 0}}, color = {0, 0, 127}));
        connect(fbSOC.u2, soc) annotation (
          Line(points = {{-60, -8}, {-60, -40}, {-120, -40}}, color = {0, 0, 127}));
        connect(socRef_.y, fbSOC.u1) annotation (
          Line(points = {{-79, -1.33227e-015}, {-76, -1.33227e-015}, {-76, 0}, {-74, 0}, {-68, 0}}, color = {0, 0, 127}));
        connect(powSwitch.u2, on) annotation (
          Line(points = {{52, -20}, {40, -20}, {26, -20}, {26, 22}, {88, 22}, {88, 40}, {110, 40}}, color = {255, 0, 255}));
        connect(powSwitch.y, pcPowReq) annotation (
          Line(points = {{75, -20}, {84, -20}, {84, -40}, {110, -40}}, color = {0, 0, 127}));
        connect(powFilt.y, add.u1) annotation (
          Line(points = {{-55.2, 52}, {-44, 52}, {-30, 52}}, color = {0, 0, 127}));
        connect(powFilt.u, edPow) annotation (
          Line(points = {{-73.6, 52}, {-78, 52}, {-78, 40}, {-120, 40}}, color = {0, 0, 127}));
        connect(powHyst.y, on) annotation (
          Line(points = {{61, 46}, {88, 46}, {88, 40}, {110, 40}}, color = {255, 0, 255}));
        annotation (
          Placement(visible = true, transformation(origin = {-58, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 0)),
          Diagram(coordinateSystem(extent = {{-100, -80}, {100, 80}})),
          Icon(coordinateSystem(initialScale = 0.1), graphics={  Rectangle(fillColor = {255, 255, 255},
                  fillPattern =                                                                                       FillPattern.Solid, extent = {{-100, 100}, {100, -100}}), Polygon(lineColor = {95, 95, 95}, fillColor = {175, 175, 175},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-4, -40}, {74, 16}, {74, -6}, {-4, -62}, {-4, -40}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{8, -38}, {28, -48}, {20, -54}, {0, -44}, {8, -38}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{20, -54}, {28, -48}, {32, -56}, {24, -62}, {20, -54}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 127},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{24, -62}, {32, -56}, {32, -78}, {24, -84}, {24, -62}}), Polygon(lineColor = {0, 0, 255}, fillColor = {191, 191, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{0, -44}, {20, -54}, {24, -62}, {24, -84}, {22, -84}, {22, -62}, {20, -58}, {0, -48}, {0, -44}}), Polygon(lineColor = {128, 128, 128}, fillColor = {128, 128, 128},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-14, 40}, {-18, 32}, {-10, 38}, {-8, 44}, {-14, 40}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 127},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-18, 32}, {-10, 38}, {-10, 14}, {-18, 8}, {-18, 32}}), Polygon(lineColor = {0, 0, 255}, fillColor = {191, 191, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-20, 10}, {-20, 32}, {-16, 40}, {4, 30}, {4, 26}, {-16, 36}, {-18, 32}, {-18, 8}, {-20, 10}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-8, 46}, {12, 36}, {4, 30}, {-16, 40}, {-8, 46}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{28, -22}, {48, -32}, {40, -38}, {20, -28}, {28, -22}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{40, -38}, {48, -32}, {52, -40}, {44, -46}, {40, -38}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 127},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{44, -46}, {52, -40}, {52, -62}, {44, -68}, {44, -46}}), Polygon(lineColor = {0, 0, 255}, fillColor = {191, 191, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{20, -28}, {40, -38}, {44, -46}, {44, -68}, {42, -68}, {42, -46}, {40, -42}, {20, -32}, {20, -28}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{48, -8}, {68, -18}, {60, -24}, {40, -14}, {48, -8}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{60, -24}, {68, -18}, {72, -26}, {64, -32}, {60, -24}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 127},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{64, -32}, {72, -26}, {72, -48}, {64, -54}, {64, -32}}), Polygon(lineColor = {0, 0, 255}, fillColor = {191, 191, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{40, -14}, {60, -24}, {64, -32}, {64, -54}, {62, -54}, {62, -32}, {60, -28}, {40, -18}, {40, -14}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{68, 6}, {88, -4}, {80, -10}, {60, 0}, {68, 6}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{80, -10}, {88, -4}, {92, -12}, {84, -18}, {80, -10}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 127},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{84, -18}, {92, -12}, {92, -34}, {84, -40}, {84, -18}}), Polygon(lineColor = {0, 0, 255}, fillColor = {191, 191, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{60, 0}, {80, -10}, {84, -18}, {84, -40}, {82, -40}, {82, -18}, {80, -14}, {60, -4}, {60, 0}}), Polygon(lineColor = {128, 128, 128}, fillColor = {128, 128, 128},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-34, 26}, {-38, 18}, {-30, 24}, {-28, 30}, {-34, 26}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 127},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-38, 18}, {-30, 24}, {-30, 0}, {-38, -6}, {-38, 18}}), Polygon(lineColor = {0, 0, 255}, fillColor = {191, 191, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-40, -4}, {-40, 18}, {-36, 26}, {-16, 16}, {-16, 12}, {-36, 22}, {-38, 18}, {-38, -6}, {-40, -4}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-28, 32}, {-8, 22}, {-16, 16}, {-36, 26}, {-28, 32}}), Polygon(lineColor = {128, 128, 128}, fillColor = {128, 128, 128},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-54, 12}, {-58, 4}, {-50, 10}, {-48, 16}, {-54, 12}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 127},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-58, 4}, {-50, 10}, {-50, -14}, {-58, -20}, {-58, 4}}), Polygon(lineColor = {0, 0, 255}, fillColor = {191, 191, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-60, -18}, {-60, 4}, {-56, 12}, {-36, 2}, {-36, -2}, {-56, 8}, {-58, 4}, {-58, -20}, {-60, -18}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-48, 18}, {-28, 8}, {-36, 2}, {-56, 12}, {-48, 18}}), Polygon(lineColor = {128, 128, 128}, fillColor = {128, 128, 128},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-74, -4}, {-78, -12}, {-70, -6}, {-68, 0}, {-74, -4}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 127},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-78, -12}, {-70, -6}, {-70, -30}, {-78, -36}, {-78, -12}}), Polygon(lineColor = {0, 0, 255}, fillColor = {191, 191, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-80, -34}, {-80, -12}, {-76, -4}, {-56, -14}, {-56, -18}, {-76, -8}, {-78, -12}, {-78, -36}, {-80, -34}}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-68, 2}, {-48, -8}, {-56, -14}, {-76, -4}, {-68, 2}}), Polygon(lineColor = {95, 95, 95}, fillColor = {75, 75, 75},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-64, -8}, {-4, -40}, {-4, -62}, {-64, -30}, {-64, -8}}), Polygon(lineColor = {95, 95, 95}, fillColor = {160, 160, 164},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, points = {{-64, -8}, {-4, -40}, {74, 16}, {14, 48}, {-64, -8}}), Rectangle(fillColor = {255, 255, 255}, pattern = LinePattern.None,
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-98, 92}, {98, 62}}), Text(origin = {-6.08518, -4.3529}, lineColor = {0, 0, 255}, extent = {{-91.9148, 98.3529}, {100.085, 60.353}}, textString = "%name", fontName = "Helvetica")}),
          experiment(StopTime = 1800, __Dymola_NumberOfIntervals = 2000),
          experimentSetupOutput(derivatives = false),
          Documentation(info = "<html>
<p>SHEV logic. Contains:</p>
<p>- basic logic, which requests the average load power from the ICE</p>
<p>- additional SOC loop to avoid SOC drift</p>
<p>- further ON/OFF control to switch OFF the engine when the average power is too low to permit efficient operation</p>
</html>"),__OpenModelica_commandLineOptions = "");
      end EMS;
    end ECUs;
    annotation (
      Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics={  Line(points = {{-80, -84}, {-80, 68}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{-88, -80}, {78, -80}}, color = {0, 0, 0}, smooth = Smooth.None), Polygon(points = {{94, -80}, {78, -74}, {78, -86}, {94, -80}}, lineColor = {0, 0, 0}, smooth = Smooth.None), Polygon(points = {{8, 0}, {-8, 6}, {-8, -6}, {8, 0}}, lineColor = {0, 0, 0}, smooth = Smooth.None, origin = {-80, 76}, rotation = 90), Line(points = {{-84, 40}, {-14, 40}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{-14, 40}, {-4, 2}, {22, -32}, {62, -44}, {62, -80}}, color = {0, 0, 0}, smooth = Smooth.None)}));
  end MapBased;

  package SupportModels "Useful addtional models"
    extends Modelica.Icons.Package;
    // extends EHPowerTrain.Icons.SupportIcon;

    package MapBasedRelated
      block TauLim "Torque limiter"
        Modelica.Blocks.Interfaces.RealInput w annotation (
          Placement(transformation(extent = {{-140, 40}, {-100, 80}})));
        Modelica.Blocks.Interfaces.RealInput tau annotation (
          Placement(transformation(extent = {{-140, -80}, {-100, -40}})));
        Modelica.Blocks.Interfaces.RealOutput y annotation (
          Placement(transformation(extent = {{100, -10}, {120, 10}})));
        parameter Modelica.SIunits.Power powMax(start = 50000) "Maximum absolute mechanical power";
        parameter Modelica.SIunits.Torque tauMax(start = 400) "Maximum absolute torque";
        Boolean powLimActive "true when power limitation prevails over torque limitation";
        Boolean tauLimActive "true when power limitation prevails over torque limitation";
        Modelica.SIunits.Torque maxAbsTau;
        Modelica.SIunits.Torque negMaxTau = -maxAbsTau;
      protected
        final Boolean powLimPrevails;
        final Boolean tauLimited;
      algorithm
        tauLimited := false;
        if w < powMax / tauMax then
          powLimPrevails := false;
          maxAbsTau := tauMax;
        else
          powLimPrevails := true;
          maxAbsTau := powMax / w;
        end if;
        if tau > 0 then
          if tau > maxAbsTau then
            y := maxAbsTau;
            tauLimited := true;
          else
            y := tau;
          end if;
        else
          if tau < (-maxAbsTau) then
            y := -maxAbsTau;
            tauLimited := true;
          else
            y := tau;
          end if;
        end if;
        if tauLimited then
          if powLimPrevails then
            powLimActive := true;
            tauLimActive := false;
          else
            powLimActive := false;
            tauLimActive := true;
          end if;
        else
          powLimActive := false;
          tauLimActive := false;
        end if;
        annotation (
          Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}})),
          Icon(coordinateSystem(preserveAspectRatio = false, initialScale = 0.1), graphics={  Rectangle(fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                    FillPattern.Solid, extent = {{-100, 90}, {100, -88}}), Line(points = {{-72, 80}, {-72, -80}}, arrow = {Arrow.Filled, Arrow.None}, arrowSize = 7), Text(origin = {0, 12}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-100, 122}, {100, 84}}, textString = "%name
    "), Text(lineColor = {0, 0, 255}, extent = {{-98, 54}, {-84, 48}}, textString = "T"), Line(points = {{92, -2}, {-74, -2}}, arrow = {Arrow.Filled, Arrow.None}, arrowSize = 7), Text(lineColor = {0, 0, 255}, extent = {{72, -22}, {86, -28}}, textString = "W"), Line(points = {{-72, 54}, {-12, 54}, {-2, 40}, {16, 26}, {30, 18}, {44, 14}}), Line(points = {{-72, -58}, {-12, -58}, {-2, -44}, {16, -30}, {30, -22}, {42, -18}})}),
          Documentation(info = "<html><head></head><body><p><font size=\"5\">Gives the maximum output torque as a function of the input speed.</font></p>
<p><font size=\"5\">The output is Tmax if Tmax*w&lt;PowMax, otherwise it is PowMax/w</font></p>
<p><font size=\"5\">Torques and powers are in SI units</font></p>
</body></html>"));
      end TauLim;

      model InertiaTq "Inertia with added torque"
        import SI = Modelica.SIunits;
        Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_a "Left flange of shaft" annotation (
          Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_b "Right flange of shaft" annotation (
          Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
        parameter SI.Inertia J(min = 0, start = 1) "Moment of inertia";
        parameter StateSelect stateSelect = StateSelect.default "Priority to use phi and w as states" annotation (
          HideResult = true,
          Dialog(tab = "Advanced"));
        SI.Angle phi(stateSelect = stateSelect) "Absolute rotation angle of component" annotation (
          Dialog(group = "Initialization", showStartAttribute = true));
        SI.AngularVelocity w(stateSelect = stateSelect) "Absolute angular velocity of component (= der(phi))" annotation (
          Dialog(group = "Initialization", showStartAttribute = true));
        SI.AngularAcceleration a "Absolute angular acceleration of component (= der(w))" annotation (
          Dialog(group = "Initialization", showStartAttribute = true));
        Modelica.Blocks.Interfaces.RealInput tau annotation (
          Placement(transformation(extent = {{-20.5, -20}, {20.5, 20}}, rotation = 90, origin = {-54.5, -100})));
      equation
        phi = flange_a.phi;
        phi = flange_b.phi;
        w = der(phi);
        a = der(w);
        J * a = flange_a.tau + flange_b.tau + tau;
        annotation (
          Documentation(info = "<html>
    <p>
    Rotational component with <b>inertia</b> and two rigidly connected flanges.
    </p>

    </HTML>
    "),   Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, grid = {2, 2}), graphics={  Rectangle(extent = {{-100, 10}, {-50, -10}}, lineColor = {0, 0, 0},
                  fillPattern =                                                                                                                                                                                FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Rectangle(extent = {{50, 10}, {100, -10}}, lineColor = {0, 0, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Line(points = {{-80, -25}, {-60, -25}}, color = {0, 0, 0}), Line(points = {{60, -25}, {80, -25}}, color = {0, 0, 0}), Line(points = {{-70, -25}, {-70, -70}}, color = {0, 0, 0}), Line(points = {{70, -25}, {70, -70}}, color = {0, 0, 0}), Line(points = {{-80, 25}, {-60, 25}}, color = {0, 0, 0}), Line(points = {{60, 25}, {80, 25}}, color = {0, 0, 0}), Line(points = {{-70, 45}, {-70, 25}}, color = {0, 0, 0}), Line(points = {{70, 45}, {70, 25}}, color = {0, 0, 0}), Line(points = {{-70, -70}, {70, -70}}, color = {0, 0, 0}), Rectangle(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Text(extent = {{-150, 100}, {150, 60}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-150, -80}, {150, -120}}, lineColor = {0, 0, 0}, textString = "J=%J")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, grid = {2, 2}), graphics));
      end InertiaTq;

      block EfficiencyT "Determines the electric power from the mechanical considering efficiency map"
        import wbEHPTlib;
        parameter Modelica.SIunits.Torque tauMax(start = 400) "Maximum machine torque(Nm)";
        parameter Modelica.SIunits.Power powMax(start = 22000) "Maximum drive power";
        parameter Modelica.SIunits.AngularVelocity wMax(start = 650) "Maximum machine speed(rad/s)";
        parameter String mapsFileName = "mapsFileName.txt" "Name of the file containing data maps";
        parameter String effMapName = "effMapName.txt" "Name of the efficiency map in mapsFileName";
        //the name is passed because a file can contain efficiency tables for
        //different submodels, e.g. genEfficiency for generator and motEfficiency for motor.
        Modelica.Blocks.Tables.CombiTable2D effTable_(smoothness = Modelica.Blocks.Types.Smoothness.LinearSegments, tableOnFile = true, fileName = mapsFileName, tableName = effMapName) "normalised efficiency" annotation (
          Placement(transformation(extent = {{-14, -14}, {14, 14}}, rotation = 0, origin = {18, -18})));
        Modelica.Blocks.Interfaces.RealInput w annotation (
          Placement(transformation(extent = {{-140, -60}, {-100, -20}}), iconTransformation(extent = {{-140, -60}, {-100, -20}})));
        Modelica.Blocks.Interfaces.RealInput tau annotation (
          Placement(transformation(extent = {{-140, 20}, {-100, 60}}), iconTransformation(extent = {{-140, 20}, {-100, 60}})));
        Modelica.Blocks.Interfaces.RealOutput elePow annotation (
          Placement(transformation(extent = {{96, -10}, {116, 10}})));
        Modelica.Blocks.Math.Abs abs1 annotation (
          Placement(transformation(extent = {{-76, -50}, {-56, -30}})));
        Modelica.Blocks.Math.Abs abs2 annotation (
          Placement(transformation(extent = {{-80, 40}, {-60, 60}})));
        Modelica.Blocks.Math.Gain normalizeTau(k = 1 / tauMax) annotation (
          Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {-36, 50})));
        wbEHPTlib.SupportModels.MapBasedRelated.Pel applyEta annotation (
          Placement(transformation(extent = {{60, -10}, {84, 12}})));
        Modelica.Blocks.Math.Product PMOT annotation (
          Placement(transformation(extent = {{-72, 0}, {-52, 20}})));
        Modelica.Blocks.Math.Gain normalizeSpeed(k = 1 / wMax) annotation (
          Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {-34, -40})));
      equation
        connect(tau, abs2.u) annotation (
          Line(points = {{-120, 40}, {-94, 40}, {-94, 50}, {-82, 50}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(w, abs1.u) annotation (
          Line(points = {{-120, -40}, {-78, -40}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(abs2.y, normalizeTau.u) annotation (
          Line(points = {{-59, 50}, {-48, 50}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(normalizeTau.y, effTable_.u1) annotation (
          Line(points = {{-25, 50}, {-7.7, 50}, {-7.7, -9.6}, {1.2, -9.6}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(applyEta.Pel, elePow) annotation (
          Line(points = {{85.2, 1}, {92.48, 1}, {92.48, 0}, {106, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(effTable_.y, applyEta.eta) annotation (
          Line(points = {{33.4, -18}, {46, -18}, {46, -5.6}, {57.6, -5.6}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(PMOT.u1, tau) annotation (
          Line(points = {{-74, 16}, {-84, 16}, {-84, 40}, {-120, 40}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(PMOT.u2, w) annotation (
          Line(points = {{-74, 4}, {-84, 4}, {-84, -40}, {-120, -40}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(PMOT.y, applyEta.P) annotation (
          Line(points = {{-51, 10}, {42, 10}, {42, 7.6}, {57.6, 7.6}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(abs1.y, normalizeSpeed.u) annotation (
          Line(points = {{-55, -40}, {-46, -40}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(normalizeSpeed.y, effTable_.u2) annotation (
          Line(points = {{-23, -40}, {-10, -40}, {-10, -26.4}, {1.2, -26.4}}, color = {0, 0, 127}, smooth = Smooth.None));
        annotation (
          Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -80}, {100, 80}})),
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics={  Rectangle(extent = {{-100, 72}, {100, -72}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                               FillPattern.Solid), Line(points = {{-74, -54}, {-74, 58}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{-82, -48}, {78, -48}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{-74, 38}, {-24, 38}, {-4, 12}, {28, -8}, {60, -22}, {62, -48}}, color = {0, 0, 0}, smooth = Smooth.None), Polygon(points = {{-20, 14}, {-40, 24}, {-56, -4}, {-38, -36}, {12, -38}, {26, -28}, {22, -20}, {8, -6}, {-8, 4}, {-20, 14}}, lineColor = {0, 0, 0}, smooth = Smooth.None, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-28, 4}, {-38, 2}, {-32, -20}, {0, -32}, {10, -28}, {12, -20}, {-28, 4}}, lineColor = {0, 0, 0}, smooth = Smooth.None, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Text(extent = {{-102, 118}, {100, 78}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, textString = "%name"), Text(extent = {{26, 46}, {76, 4}}, lineColor = {0, 0, 0}, textString = "T")}),
          Documentation(info = "<html>
<p>This block computes the machine and inverter losses from the mechanical input quantities and determines the power to be drawn from the electric circuit. The &QUOT;drawn&QUOT; power can be also a negative numer, meaning that themachine is actually delivering electric power.</p>
<p>The given efficiency map is intended as being built with torques being ratios of actual torques to tauMax and speeds being ratios of w to wMax. In case the user uses, in the given efficiency map, torques in Nm and speeds in rad/s, the block can be used selecting tauTmax=1, wMax=1.</p>
<p>The choice of having normalised efficiency computation allows simulations of machines different in sizes and similar in characteristics to be repeated without having to rebuild the efficiency maps. </p>
<p>Torques are written in the first matrix column, speeds on the first row.</p>
</html>"));
      end EfficiencyT;

      block EfficiencyLF "Determines the electric from the mechanical power considering loss with a formula"
        import wbEHPTlib;
        parameter Modelica.SIunits.Torque tauMax(start = 400) "Maximum machine torque(Nm)";
        parameter Modelica.SIunits.Power powMax(start = 22000) "Maximum drive power";
        parameter Modelica.SIunits.AngularVelocity wMax(start = 650) "Maximum machine speed(rad/s)";
        parameter Real effTable[:, :] = [0.00, 0.00, 0.25, 0.50, 0.75, 1.00; 0.00, 0.75, 0.80, 0.81, 0.82, 0.83; 0.25, 0.76, 0.81, 0.82, 0.83, 0.84; 0.50, 0.77, 0.82, 0.83, 0.84, 0.85; 0.75, 0.78, 0.83, 0.84, 0.85, 0.87; 1.00, 0.80, 0.84, 0.85, 0.86, 0.88];
        Modelica.Blocks.Interfaces.RealInput w annotation (
          Placement(transformation(extent = {{-140, -60}, {-100, -20}}), iconTransformation(extent = {{-140, -60}, {-100, -20}})));
        Modelica.Blocks.Interfaces.RealInput tau annotation (
          Placement(transformation(extent = {{-140, 20}, {-100, 60}}), iconTransformation(extent = {{-140, 20}, {-100, 60}})));
        Modelica.Blocks.Interfaces.RealOutput elePow annotation (
          Placement(transformation(extent = {{96, -10}, {116, 10}})));
        Modelica.Blocks.Math.Abs abs1 annotation (
          Placement(transformation(extent = {{-76, -50}, {-56, -30}})));
        Modelica.Blocks.Math.Abs abs2 annotation (
          Placement(transformation(extent = {{-80, 40}, {-60, 60}})));
        Modelica.Blocks.Math.Gain normalizeTau(k = 1 / tauMax) annotation (
          Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {-36, 50})));
        Modelica.Blocks.Math.Gain normalizeSpeed(k = 1 / wMax) annotation (
          Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {-34, -40})));
        wbEHPTlib.SupportModels.MapBasedRelated.AddLossesWT addLosses annotation (
          Placement(transformation(extent = {{10, -10}, {30, 10}})));
      equation
        connect(tau, abs2.u) annotation (
          Line(points = {{-120, 40}, {-94, 40}, {-94, 50}, {-82, 50}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(w, abs1.u) annotation (
          Line(points = {{-120, -40}, {-78, -40}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(abs2.y, normalizeTau.u) annotation (
          Line(points = {{-59, 50}, {-48, 50}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(abs1.y, normalizeSpeed.u) annotation (
          Line(points = {{-55, -40}, {-46, -40}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(addLosses.T, normalizeTau.y) annotation (
          Line(points = {{8.2, 6}, {-18, 6}, {-18, 50}, {-25, 50}}, color = {0, 0, 127}));
        connect(addLosses.W, normalizeSpeed.y) annotation (
          Line(points = {{8.2, -6}, {-18, -6}, {-18, -40}, {-23, -40}}, color = {0, 0, 127}));
        connect(addLosses.y, elePow) annotation (
          Line(points = {{30.6, 0}, {30.6, 0}, {106, 0}}, color = {0, 0, 127}));
        annotation (
          Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -80}, {100, 80}})),
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics={  Rectangle(extent = {{-100, 72}, {100, -72}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                               FillPattern.Solid), Line(points = {{-74, -54}, {-74, 58}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{-82, -48}, {78, -48}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{-74, 38}, {-24, 38}, {-4, 12}, {28, -8}, {60, -22}, {62, -48}}, color = {0, 0, 0}, smooth = Smooth.None), Polygon(points = {{-20, 14}, {-40, 24}, {-56, -4}, {-38, -36}, {12, -38}, {26, -28}, {22, -20}, {8, -6}, {-8, 4}, {-20, 14}}, lineColor = {0, 0, 0}, smooth = Smooth.None, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-28, 4}, {-38, 2}, {-32, -20}, {0, -32}, {10, -28}, {12, -20}, {-28, 4}}, lineColor = {0, 0, 0}, smooth = Smooth.None, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Text(extent = {{-102, 118}, {100, 78}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, textString = "%name"), Text(extent = {{26, 46}, {76, 4}}, lineColor = {0, 0, 0}, textString = "L")}),
          Documentation(info = "<html>
<p>This block computes the machine and inverter losses from the mechanical input quantities and determines the power to be drawn from the electric circuit. The &QUOT;drawn&QUOT; power can be also a negative numer, meaning that the machine is actually delivering electric power.</p>
<p>The signs ot T and W must be such that their product is positive when mechanical power exits from the machine (electric motor operation)</p>
</html>"));
      end EfficiencyLF;

      block EffiPlot "Utility to plot efficiencies from an effTable"
        import wbEHPTlib;
        parameter Real tauMax(start = 400) "Maximum machine torque(Nm)";
        parameter Real powMax(start = 22000) "Maximum drive power";
        parameter Real wMax(start = 650) "Maximum machine speed(rad/s)";
        parameter Real effTable[:, :] = [0.00, 0.00, 0.25, 0.50, 0.75, 1.00; 0.00, 0.75, 0.80, 0.81, 0.82, 0.83; 0.25, 0.76, 0.81, 0.82, 0.83, 0.84; 0.50, 0.77, 0.82, 0.83, 0.84, 0.85; 0.75, 0.78, 0.83, 0.84, 0.85, 0.87; 1.00, 0.80, 0.84, 0.85, 0.86, 0.88];
        Real tau[size(effs, 1)];
        Real effs[:] = {0.75, 0.775, 0.8, 0.825, 0.85, 0.875};
        Modelica.Blocks.Tables.CombiTable2D effTable_[size(effs, 1)](each tableOnFile = false, each smoothness = Modelica.Blocks.Types.Smoothness.LinearSegments, each table = effTable) "normalised efficiency" annotation (
          Placement(transformation(extent = {{-14, -14}, {14, 14}}, rotation = 0, origin = {-2, 0})));
      equation
        for i in 1:size(effs, 1) loop
          effTable_[i].y = effs[i];
          effTable_[i].u1 = time;
          effTable_[i].u2 = tau[i];
        end for;
        annotation (
          Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -80}, {100, 80}})),
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics={  Rectangle(extent = {{-100, 72}, {100, -72}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                               FillPattern.Solid), Line(points = {{-74, -54}, {-74, 58}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{-82, -48}, {78, -48}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{-74, 38}, {-24, 38}, {-4, 12}, {28, -8}, {60, -22}, {62, -48}}, color = {0, 0, 0}, smooth = Smooth.None), Polygon(points = {{-20, 14}, {-40, 24}, {-56, -4}, {-38, -36}, {12, -38}, {26, -28}, {22, -20}, {8, -6}, {-8, 4}, {-20, 14}}, lineColor = {0, 0, 0}, smooth = Smooth.None, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-28, 4}, {-38, 2}, {-32, -20}, {0, -32}, {10, -28}, {12, -20}, {-28, 4}}, lineColor = {0, 0, 0}, smooth = Smooth.None, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Text(extent = {{-102, 118}, {100, 78}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, textString = "%name"), Text(extent = {{26, 46}, {76, 4}}, lineColor = {0, 0, 0}, textString = "M")}),
          Documentation(info = "<html>
<p>This block computes the machine and inverter losses from the mechanical input quantities and determines the power to be drawn from the electric circuit. The &QUOT;drawn&QUOT; power can be also a negative numer, meaning that themachine is actually delivering electric power.</p>
<p>The given efficiency map is intended as being built with torques being ratios of actual torques to tauMax and speeds being ratios of w to wMax. In case the user uses, in the given efficiency map, torques in Nm and speeds in rad/s, the block can be used selecting tauTmax=1, wMax=1.</p>
<p>The choice of having normalised efficiency computation allows simulations of machines different in sizes and similar in characteristics to be repeated without having to rebuild the efficiency maps. </p>
<p>Torques are written in the first matrix column, speeds on the first row.</p>
</html>"));
      end EffiPlot;

      block EffiPlot2 "Utility to plot efficiencies from an effTable"
        import wbEHPTlib;
        parameter Modelica.SIunits.Torque tauMax(start = 400) "Maximum machine torque(Nm)";
        parameter Modelica.SIunits.Power powMax(start = 22000) "Maximum drive power";
        parameter Modelica.SIunits.AngularVelocity wMax(start = 650) "Maximum machine speed(rad/s)";

        function eff
          input Real A, bT, bS, bP;
          input Real tq "input torque";
          input Real sp "input speed";
          output Real eff;
        protected
          Real pLoss;
        algorithm
          pLoss := A + bT * tq ^ 2 + bS * sp ^ 2 + bP * (tq * sp) ^ 2;
          eff := tq * sp / (tq * sp + pLoss);
        end eff;

        function lossFun
          input Real A, bT, bS, bP;
          input Real tq "input torque";
          input Real sp "input speed";
          output Real pLoss;
        algorithm
          pLoss := A + bT * tq ^ 2 + bS * sp ^ 2 + bP * (tq * sp) ^ 2;
        end lossFun;

        Real tauE[size(effs, 1)];
        Real tauL[size(loss, 1)];
        Real loss[:] = {0.02, 0.04, 0.06, 0.08};
        Real effs[:] = {0.75, 0.8, 0.85, 0.9};
      equation
        for i in 1:size(effs, 1) loop
          effs[i] = eff(0.0005, 0.02, 0.01, 0.025, tauE[i], time);
        end for;
        for i in 1:size(loss, 1) loop
          loss[i] = lossFun(0.0005, 0.02, 0.01, 0.025, tauL[i], time);
        end for;
        annotation (
          Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -80}, {100, 80}})),
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics={  Rectangle(extent = {{-100, 72}, {100, -72}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                               FillPattern.Solid), Line(points = {{-74, -54}, {-74, 58}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{-82, -48}, {78, -48}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{-74, 38}, {-24, 38}, {-4, 12}, {28, -8}, {60, -22}, {62, -48}}, color = {0, 0, 0}, smooth = Smooth.None), Polygon(points = {{-20, 14}, {-40, 24}, {-56, -4}, {-38, -36}, {12, -38}, {26, -28}, {22, -20}, {8, -6}, {-8, 4}, {-20, 14}}, lineColor = {0, 0, 0}, smooth = Smooth.None, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-28, 4}, {-38, 2}, {-32, -20}, {0, -32}, {10, -28}, {12, -20}, {-28, 4}}, lineColor = {0, 0, 0}, smooth = Smooth.None, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Text(extent = {{-102, 118}, {100, 78}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, textString = "%name"), Text(extent = {{26, 46}, {76, 4}}, lineColor = {0, 0, 0}, textString = "M")}),
          Documentation(info = "<html>
<p>Questo modello di ausilio cerca di riprodurre i contour descritti nel file Efficiency.docx.</p>
<p>Nella versione presente ha due difetti fondamentali:</p>
<p>1) non riesce a descrivere curve polidrome a differenza di contour. Questo &egrave; critico in quanto le curve di efficienza sono proprio di questo tipo.</p>
<p>2) non riesce a gestire i casi in cui per un certo vaore della velocit&agrave; non si trova la coppia che da una data perdita. Se ad es. le perdite si cercano a partire da 0.02 dappiako da ecciciency.docx che ha curva delle perdite ha tangente verticale per velocit&agrave; pari a 1.4, ed infatti il modello proposto deve rimanere con 0.02 al di sotto di 1.4, altrimenti non converge.</p>
</html>"),experiment(StartTime = 0.1, StopTime = 1.35));
      end EffiPlot2;

      block Pel "Outputs a power signal computed from the given efficiency and input power"
        Modelica.Blocks.Interfaces.RealInput eta "efficiency" annotation (
          Placement(transformation(extent = {{-140, -80}, {-100, -40}}), iconTransformation(extent = {{-140, -80}, {-100, -40}})));
        Modelica.Blocks.Interfaces.RealInput P "Delivered Mechanical Power" annotation (
          Placement(transformation(extent = {{-140, 40}, {-100, 80}}), iconTransformation(extent = {{-140, 40}, {-100, 80}})));
        Modelica.Blocks.Interfaces.RealOutput Pel "Absorbed Electrical power" annotation (
          Placement(transformation(extent = {{100, -10}, {120, 10}})));
      protected
        Real etaProc;
      algorithm
        etaProc := if eta > 1e-3 then eta else 1e-3;
        if noEvent(P <= 0) then
          Pel := P * etaProc;
        else
          Pel := P / etaProc;
        end if;
        annotation (
          Icon(graphics={  Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 127}, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                               FillPattern.Solid), Text(extent = {{-100, 142}, {96, 106}}, lineColor = {0, 0, 255}, textString = "%name"), Line(points = {{-100, 62}, {98, 0}}, color = {28, 108, 200}), Line(points = {{-100, -58}, {-18, 20}}, color = {28, 108, 200}), Ellipse(extent = {{-22, 46}, {10, 16}}, lineColor = {28, 108, 200}, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid)}),
          Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics),
          Documentation(info = "<html>
<pre><span style=\"font-family: Courier New,courier;\">Outputs&nbsp;a power signal computed from the&nbsp;given&nbsp;efficiency&nbsp;and&nbsp;input&nbsp;power</span></pre>
</html>"));
      end Pel;

      block AddLossesWT "adds drive losses function of W and T"
        parameter Real A = 0.006 "fixed p.u. losses";
        parameter Real bT(unit = "J/(N2.m2)") = 0.1 "torque losses coefficient";
        parameter Real bW(unit = "J/(rad2.s2)") = 0.1 "speed losses coefficient";
        parameter Real bP(unit = "J/W2") = 0.07 "power losses coefficient";
        Modelica.SIunits.Energy losses;
        Modelica.Blocks.Interfaces.RealInput W annotation (
          Placement(transformation(extent = {{-138, -80}, {-98, -40}})));
        Modelica.Blocks.Interfaces.RealOutput y annotation (
          Placement(transformation(extent = {{96, -10}, {116, 10}})));
        Modelica.Blocks.Interfaces.RealInput T annotation (
          Placement(transformation(extent = {{-138, 40}, {-98, 80}})));
      equation
        losses = A + bT * T ^ 2 + bW * W ^ 2 + bP * (T * W) ^ 2;
        y = T * W + losses;
        annotation (
          Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}})),
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics={  Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 127}, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                                   FillPattern.Solid), Polygon(points = {{-20, -22}, {20, -22}, {20, -60}, {40, -60}, {0, -100}, {-40, -60}, {-20, -60}, {-20, -22}}, lineColor = {0, 0, 0}, smooth = Smooth.None, fillColor = {255, 0, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Text(extent = {{-100, 75}, {100, 35}}, lineColor = {0, 0, 127}, fillColor = {255, 0, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, textString = "%name"), Text(extent = {{-70, 10}, {74, -16}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, textString = "W-T")}));
      end AddLossesWT;

      model ConstPg "Constant Power DC Load"
        parameter Real vNom = 100;
        parameter Modelica.SIunits.Time Ti = 0.01 "inner PI follower integral time constant";
        Real v "DC voltage";
        Modelica.Blocks.Math.Feedback feedback1 annotation (
          Placement(visible = true, transformation(origin = {56, -44}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
        Modelica.Electrical.Analog.Interfaces.PositivePin pin_p annotation (
          Placement(visible = true, transformation(extent = {{-108, 58}, {-88, 78}}, rotation = 0), iconTransformation(extent = {{-10, 90}, {10, 110}}, rotation = 0)));
        Modelica.Electrical.Analog.Interfaces.NegativePin pin_n annotation (
          Placement(visible = true, transformation(extent = {{-108, -74}, {-88, -54}}, rotation = 0), iconTransformation(extent = {{-10, -108}, {10, -88}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput Pref "Reference power" annotation (
          Placement(visible = true, transformation(origin = {100, -44}, extent = {{-18, -18}, {18, 18}}, rotation = 180), iconTransformation(origin = {82, 0}, extent = {{-18, -18}, {18, 18}}, rotation = 180)));
        Modelica.Electrical.Analog.Sensors.PowerSensor pSensor annotation (
          Placement(visible = true, transformation(extent = {{-82, 58}, {-62, 78}}, rotation = 0)));
        Modelica.Electrical.Analog.Basic.VariableConductor varCond annotation (
          Placement(visible = true, transformation(origin = {-50, 0}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
        Modelica.Blocks.Continuous.Integrator integrator1(k = 1 / vNom ^ 2 / Ti) annotation (
          Placement(visible = true, transformation(origin = {-2, -44}, extent = {{10, -10}, {-10, 10}}, rotation = 0)));
      equation
        connect(integrator1.u, feedback1.y) annotation (
          Line(points = {{10, -44}, {47, -44}}, color = {0, 0, 127}));
        connect(integrator1.y, varCond.G) annotation (
          Line(points = {{-13, -44}, {-28, -44}, {-28, 0}, {-39, 0}}, color = {0, 0, 127}));
        connect(feedback1.u2, pSensor.power) annotation (
          Line(points = {{56, -36}, {56, 42}, {-80, 42}, {-80, 57}}, color = {0, 0, 127}));
        connect(varCond.n, pin_n) annotation (
          Line(points = {{-50, -10}, {-50, -10}, {-50, -64}, {-98, -64}, {-98, -64}}, color = {0, 0, 255}));
        connect(varCond.p, pSensor.nc) annotation (
          Line(points = {{-50, 10}, {-50, 10}, {-50, 68}, {-62, 68}, {-62, 68}}, color = {0, 0, 255}));
        connect(pSensor.pv, pSensor.pc) annotation (
          Line(points = {{-72, 78}, {-82, 78}, {-82, 68}}, color = {0, 0, 255}));
        connect(pSensor.pc, pin_p) annotation (
          Line(points = {{-82, 68}, {-98, 68}}, color = {0, 0, 255}));
        connect(pSensor.nv, pin_n) annotation (
          Line(points = {{-72, 58}, {-72, -64}, {-98, -64}}, color = {0, 0, 255}));
        connect(feedback1.u1, Pref) annotation (
          Line(points = {{64, -44}, {100, -44}}, color = {0, 0, 127}));
        v = pin_p.v - pin_n.v;
        annotation (
          Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics={  Line(points = {{-4, 0}, {70, 0}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{0, 94}, {0, -88}, {-2, -90}}, color = {0, 0, 255}, smooth = Smooth.None), Rectangle(extent = {{-28, 68}, {28, -52}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Text(extent = {{42, 58}, {78, 22}}, lineColor = {255, 0, 0}, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid, textString = "P")}),
          Documentation(info = "<html>
    <p>Questo componente simula, mediante inseguimento di un riferimento esterno, un carico a potenza costante.</p>
    <p>I parametri k e T sono i parametri del regolatore PI che insegue l&apos;input. TIpicamente si potr&agrave; utilizzare k=1 e T di un ordine di grandezza pi&ugrave; piccolo delle costanti di tempo del segnale di ingresso di potenza</p>
    </html>"));
      end ConstPg;
    end MapBasedRelated;

    package ConnectorRelated
      model ToConnIceTauRef "Signal adaptor to send iceTauRef to a connector"

        Modelica.Blocks.Interfaces.RealInput u annotation (
          Placement(transformation(extent = {{-94, -20}, {-54, 20}}), iconTransformation(extent = {{-94, -20}, {-54, 20}})));
        Conn conn annotation (
          Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = -90, origin = {60, 0}), iconTransformation(extent = {{-20, -20}, {20, 20}}, rotation = -90, origin = {60, 0})));
      equation
        connect(u, conn.iceTauRef) annotation (
          Line(points = {{-74, 0}, {60, 0}}, color = {0, 0, 127}, smooth = Smooth.None),
          Text(string = "%second", index = 1, extent = {{6, 3}, {6, 3}}));
        annotation (
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-60, -60}, {60, 60}}), graphics={  Rectangle(extent = {{-60, 40}, {60, -40}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                         FillPattern.Solid), Line(points = {{-38, 0}, {30, 0}}, color = {0, 0, 0}, smooth = Smooth.None), Polygon(points = {{42, 0}, {22, 8}, {22, -8}, {42, 0}}, lineColor = {0, 0, 0}, smooth = Smooth.None, fillColor = {0, 0, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid)}),
          Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-60, -60}, {60, 60}}), graphics),
          Documentation(info = "<html>
<p><span style=\"font-family: MS Shell Dlg 2;\">Adapter for an input signal into &QUOT;iceTauRef&QUOT; signal in the library connector.</span></p>
</html>"));
      end ToConnIceTauRef;

      model ToConnGenTauRef "Signal adaptor to send genTauRef to a connector"

        Modelica.Blocks.Interfaces.RealInput u annotation (
          Placement(transformation(extent = {{-90, -20}, {-50, 20}}), iconTransformation(extent = {{-90, -20}, {-50, 20}})));
        Conn conn annotation (
          Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = -90, origin = {58, 0}), iconTransformation(extent = {{-20, -20}, {20, 20}}, rotation = -90, origin = {58, 0})));
      equation
        connect(u, conn.genTauRef) annotation (
          Line(points = {{-70, 0}, {58, 0}}, color = {0, 0, 127}, smooth = Smooth.None),
          Text(string = "%second", index = 1, extent = {{6, 3}, {6, 3}}));
        annotation (
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-60, -60}, {60, 60}}), graphics={  Rectangle(extent = {{-60, 40}, {60, -40}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                         FillPattern.Solid), Line(points = {{-40, 0}, {32, 0}}, color = {0, 0, 0}, smooth = Smooth.None), Polygon(points = {{42, 0}, {22, 8}, {22, -8}, {42, 0}}, lineColor = {0, 0, 0}, smooth = Smooth.None, fillColor = {0, 0, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid)}),
          Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-60, -60}, {60, 60}}), graphics),
          Documentation(info = "<html>
<p><span style=\"font-family: MS Shell Dlg 2;\">Adapter for an input signal into &QUOT;genTauRef&QUOT; signal in the library connector.</span></p>
</html>"));
      end ToConnGenTauRef;

      model ToConnIcePowRef "Signal adaptor to send icePowRef to a connector"

        Modelica.Blocks.Interfaces.RealInput u annotation (
          Placement(transformation(extent = {{-94, -20}, {-54, 20}}), iconTransformation(extent = {{-94, -20}, {-54, 20}})));
        Conn conn annotation (
          Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = -90, origin = {60, 0}), iconTransformation(extent = {{-20, -20}, {20, 20}}, rotation = -90, origin = {60, 0})));
      equation
        connect(u, conn.icePowRef) annotation (
          Line(points = {{-74, 0}, {60, 0}, {60, 0}}, color = {0, 0, 127}),
          Text(string = "%second", index = 1, extent = {{6, 3}, {6, 3}}));
        annotation (
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-60, -60}, {60, 60}}), graphics={  Rectangle(extent = {{-60, 40}, {60, -40}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255},
                  fillPattern =                                                                                                                                                                                         FillPattern.Solid), Line(points = {{-38, 0}, {30, 0}}, color = {0, 0, 0}, smooth = Smooth.None), Polygon(points = {{42, 0}, {22, 8}, {22, -8}, {42, 0}}, lineColor = {0, 0, 0}, smooth = Smooth.None, fillColor = {0, 0, 0},
                  fillPattern =                                                                                                                                                                                                        FillPattern.Solid)}),
          Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-60, -60}, {60, 60}})),
          Documentation(info = "<html>
<p><span style=\"font-family: MS Shell Dlg 2;\">Adapter for an input signal into &QUOT;icePowRef&QUOT; signal in the library connector.</span></p>
</html>"));
      end ToConnIcePowRef;

      expandable connector Conn "Control bus that is adapted to the signals connected to it"
        extends Modelica.Icons.SignalBus;
        annotation (
          Diagram(graphics));
      end Conn;
    end ConnectorRelated;

    block Greater "Output y is true, if input u1 is greater than input u2"
      extends PartialBooleanComparison;
    equation
      y = u1 > u2;
      annotation (
        Icon(coordinateSystem(preserveAspectRatio = false, initialScale = 0.1), graphics={  Ellipse(lineColor = {0, 0, 127}, extent = {{32, 10}, {52, -10}}, endAngle = 360), Line(points = {{-100, -80}, {42, -80}, {42, 0}}, color = {0, 0, 127}), Line(points = {{-54, 22}, {-8, 2}, {-54, -18}}, thickness = 0.5), Text(origin = {-20, 133}, lineColor = {0, 0, 255}, extent = {{-96, 25}, {122, -19}}, textString = "%name")}),
        Documentation(info = "<html>
<p>
The output is <b>true</b> if Real input u1 is greater than
Real input u2, otherwise the output is <b>false</b>.
</p>
</html>"));
    end Greater;

    partial block PartialBooleanComparison "Partial block with 2 Real input and 1 Boolean output signal (the result of a comparison of the two Real inputs)"
      Modelica.Blocks.Interfaces.RealInput u1 "Connector of first Real input signal" annotation (
        Placement(transformation(extent = {{-140, -20}, {-100, 20}})));
      Modelica.Blocks.Interfaces.RealInput u2 "Connector of second Real input signal" annotation (
        Placement(transformation(extent = {{-140, -100}, {-100, -60}})));
      Modelica.Blocks.Interfaces.BooleanOutput y "Connector of Boolean output signal" annotation (
        Placement(transformation(extent = {{100, -10}, {120, 10}})));
      annotation (
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics={  Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0},
                lineThickness =                                                                                                                                                                   0.5, fillColor = {210, 210, 210},
                fillPattern =                                                                                                                                                                                                        FillPattern.Solid,
                borderPattern =                                                                                                                                                                                                        BorderPattern.Raised), Ellipse(extent = {{73, 7}, {87, -7}}, lineColor = DynamicSelect({235, 235, 235}, if y > 0.5 then {0, 255, 0} else {235, 235, 235}), fillColor = DynamicSelect({235, 235, 235}, if y > 0.5 then {0, 255, 0} else {235, 235, 235}),
                fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Ellipse(extent = {{32, 10}, {52, -10}}, lineColor = {0, 0, 127}), Line(points = {{-100, -80}, {42, -80}, {42, 0}}, color = {0, 0, 127})}),
        Documentation(info = "<html>
<p>
Block has two continuous Real input and one continuous Boolean output signal
as a result of the comparison of the two input signals. The block
has a 3D icon (e.g., used in Blocks.Logical library).
</p>
</html>"));
    end PartialBooleanComparison;
    annotation (
      Icon(graphics={  Ellipse(extent = {{-36, 40}, {40, -36}}, lineColor = {0, 0, 0}), Line(points = {{4, 82}, {-6, 82}, {-10, 72}, {-24, 68}, {-34, 78}, {-46, 70}, {-42, 58}, {-54, 46}, {-66, 50}, {-74, 36}, {-66, 30}, {-68, 16}, {-78, 12}, {-78, 2}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{4, -78}, {-6, -78}, {-10, -68}, {-24, -64}, {-34, -74}, {-46, -66}, {-42, -54}, {-54, -42}, {-66, -46}, {-74, -32}, {-66, -26}, {-68, -12}, {-78, -8}, {-78, 2}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{2, -78}, {12, -78}, {16, -68}, {30, -64}, {40, -74}, {52, -66}, {48, -54}, {60, -42}, {72, -46}, {80, -32}, {72, -26}, {74, -12}, {84, -8}, {84, 2}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{2, 82}, {12, 82}, {16, 72}, {30, 68}, {40, 78}, {52, 70}, {48, 58}, {60, 46}, {72, 50}, {80, 36}, {72, 30}, {74, 16}, {84, 12}, {84, 2}}, color = {0, 0, 0}, smooth = Smooth.None)}));
  end SupportModels;

  package Icons
    model EcuIcon
      SupportModels.ConnectorRelated.Conn conn1 annotation (
        Placement(visible = true, transformation(extent = {{-20, 78}, {20, 118}}, rotation = 0), iconTransformation(extent = {{-20, 80}, {20, 120}}, rotation = 0)));
      Modelica.Blocks.Interfaces.RealInput motTauInt annotation (
        Placement(visible = true, transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0), iconTransformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
      annotation (
        Icon(graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-4, -40}, {74, 16}, {74, -6}, {-4, -62}, {-4, -40}}, lineColor = {95, 95, 95}, fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid), Polygon(points = {{8, -38}, {28, -48}, {20, -54}, {0, -44}, {8, -38}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{20, -54}, {28, -48}, {32, -56}, {24, -62}, {20, -54}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{24, -62}, {32, -56}, {32, -78}, {24, -84}, {24, -62}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 127}, fillPattern = FillPattern.Solid), Polygon(points = {{0, -44}, {20, -54}, {24, -62}, {24, -84}, {22, -84}, {22, -62}, {20, -58}, {0, -48}, {0, -44}}, lineColor = {0, 0, 255}, fillColor = {191, 191, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-14, 40}, {-18, 32}, {-10, 38}, {-8, 44}, {-14, 40}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Polygon(points = {{-18, 32}, {-10, 38}, {-10, 14}, {-18, 8}, {-18, 32}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 127}, fillPattern = FillPattern.Solid), Polygon(points = {{-20, 10}, {-20, 32}, {-16, 40}, {4, 30}, {4, 26}, {-16, 36}, {-18, 32}, {-18, 8}, {-20, 10}}, lineColor = {0, 0, 255}, fillColor = {191, 191, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-8, 46}, {12, 36}, {4, 30}, {-16, 40}, {-8, 46}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{28, -22}, {48, -32}, {40, -38}, {20, -28}, {28, -22}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{40, -38}, {48, -32}, {52, -40}, {44, -46}, {40, -38}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{44, -46}, {52, -40}, {52, -62}, {44, -68}, {44, -46}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 127}, fillPattern = FillPattern.Solid), Polygon(points = {{20, -28}, {40, -38}, {44, -46}, {44, -68}, {42, -68}, {42, -46}, {40, -42}, {20, -32}, {20, -28}}, lineColor = {0, 0, 255}, fillColor = {191, 191, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{48, -8}, {68, -18}, {60, -24}, {40, -14}, {48, -8}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{60, -24}, {68, -18}, {72, -26}, {64, -32}, {60, -24}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{64, -32}, {72, -26}, {72, -48}, {64, -54}, {64, -32}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 127}, fillPattern = FillPattern.Solid), Polygon(points = {{40, -14}, {60, -24}, {64, -32}, {64, -54}, {62, -54}, {62, -32}, {60, -28}, {40, -18}, {40, -14}}, lineColor = {0, 0, 255}, fillColor = {191, 191, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{68, 6}, {88, -4}, {80, -10}, {60, 0}, {68, 6}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{80, -10}, {88, -4}, {92, -12}, {84, -18}, {80, -10}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{84, -18}, {92, -12}, {92, -34}, {84, -40}, {84, -18}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 127}, fillPattern = FillPattern.Solid), Polygon(points = {{60, 0}, {80, -10}, {84, -18}, {84, -40}, {82, -40}, {82, -18}, {80, -14}, {60, -4}, {60, 0}}, lineColor = {0, 0, 255}, fillColor = {191, 191, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-34, 26}, {-38, 18}, {-30, 24}, {-28, 30}, {-34, 26}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Polygon(points = {{-38, 18}, {-30, 24}, {-30, 0}, {-38, -6}, {-38, 18}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 127}, fillPattern = FillPattern.Solid), Polygon(points = {{-40, -4}, {-40, 18}, {-36, 26}, {-16, 16}, {-16, 12}, {-36, 22}, {-38, 18}, {-38, -6}, {-40, -4}}, lineColor = {0, 0, 255}, fillColor = {191, 191, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-28, 32}, {-8, 22}, {-16, 16}, {-36, 26}, {-28, 32}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-54, 12}, {-58, 4}, {-50, 10}, {-48, 16}, {-54, 12}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Polygon(points = {{-58, 4}, {-50, 10}, {-50, -14}, {-58, -20}, {-58, 4}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 127}, fillPattern = FillPattern.Solid), Polygon(points = {{-60, -18}, {-60, 4}, {-56, 12}, {-36, 2}, {-36, -2}, {-56, 8}, {-58, 4}, {-58, -20}, {-60, -18}}, lineColor = {0, 0, 255}, fillColor = {191, 191, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-48, 18}, {-28, 8}, {-36, 2}, {-56, 12}, {-48, 18}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-74, -4}, {-78, -12}, {-70, -6}, {-68, 0}, {-74, -4}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Polygon(points = {{-78, -12}, {-70, -6}, {-70, -30}, {-78, -36}, {-78, -12}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 127}, fillPattern = FillPattern.Solid), Polygon(points = {{-80, -34}, {-80, -12}, {-76, -4}, {-56, -14}, {-56, -18}, {-76, -8}, {-78, -12}, {-78, -36}, {-80, -34}}, lineColor = {0, 0, 255}, fillColor = {191, 191, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-68, 2}, {-48, -8}, {-56, -14}, {-76, -4}, {-68, 2}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-64, -8}, {-4, -40}, {-4, -62}, {-64, -30}, {-64, -8}}, lineColor = {95, 95, 95}, fillColor = {75, 75, 75}, fillPattern = FillPattern.Solid), Polygon(points = {{-64, -8}, {-4, -40}, {74, 16}, {14, 48}, {-64, -8}}, lineColor = {95, 95, 95}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid), Text(origin = {-1, -42}, lineColor = {0, 0, 255}, extent = {{-119, -64}, {119, -104}}, textString = "%name"), Rectangle(extent = {{-98, 92}, {98, 62}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None)}),
        Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics));
    end EcuIcon;

    model SupportIcon
      annotation (
        Icon(graphics = {Ellipse(extent = {{-38, 38}, {38, -38}}, lineColor = {0, 0, 0}), Line(points = {{2, 80}, {-8, 80}, {-12, 70}, {-26, 66}, {-36, 76}, {-48, 68}, {-44, 56}, {-56, 44}, {-68, 48}, {-76, 34}, {-68, 28}, {-70, 14}, {-80, 10}, {-80, 0}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{2, -80}, {-8, -80}, {-12, -70}, {-26, -66}, {-36, -76}, {-48, -68}, {-44, -56}, {-56, -44}, {-68, -48}, {-76, -34}, {-68, -28}, {-70, -14}, {-80, -10}, {-80, 0}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{0, -80}, {10, -80}, {14, -70}, {28, -66}, {38, -76}, {50, -68}, {46, -56}, {58, -44}, {70, -48}, {78, -34}, {70, -28}, {72, -14}, {82, -10}, {82, 0}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{0, 80}, {10, 80}, {14, 70}, {28, 66}, {38, 76}, {50, 68}, {46, 56}, {58, 44}, {70, 48}, {78, 34}, {70, 28}, {72, 14}, {82, 10}, {82, 0}}, color = {0, 0, 0}, smooth = Smooth.None)}));
    end SupportIcon;
  end Icons;
  annotation (
    uses(Modelica(version = "3.2.2"), Complex(version = "3.2.2")),
    Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics={  Polygon(points = {{-60, 16}, {78, 16}, {94, 0}, {96, -16}, {-98, -16}, {-90, 0}, {-76, 12}, {-60, 16}}, lineColor = {0, 0, 0}, smooth = Smooth.None, fillColor = {0, 0, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Ellipse(extent = {{-70, -4}, {-30, -40}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Ellipse(extent = {{34, -6}, {74, -42}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-54, 16}, {-18, 46}, {46, 46}, {74, 16}, {-54, 16}}, lineColor = {0, 0, 0}, smooth = Smooth.None, fillColor = {0, 0, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Ellipse(extent = {{-86, -6}, {-92, 4}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Ellipse(extent = {{98, -10}, {92, -4}}, lineColor = {0, 0, 0}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{-46, 20}, {-20, 42}, {16, 42}, {14, 20}, {-46, 20}}, lineColor = {0, 0, 0}, smooth = Smooth.None, fillColor = {255, 255, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Polygon(points = {{22, 42}, {42, 42}, {60, 20}, {20, 20}, {22, 42}}, lineColor = {0, 0, 0}, smooth = Smooth.None, fillColor = {255, 255, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Ellipse(extent = {{-60, -12}, {-40, -30}}, lineColor = {95, 95, 95}, fillColor = {215, 215, 215},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid), Ellipse(extent = {{44, -14}, {64, -32}}, lineColor = {95, 95, 95}, fillColor = {215, 215, 215},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid)}),
    Documentation(info = "<html>
<p>Library containing models of components, subsystems and full vehicle examples for simulation of electric and Hybrid vehicular power trains.</p>
<p>A general description of the library composition and on how to use it effectively is in the compaion paper:</p>
<p>M. Ceraolo &QUOT;Modelica Electric and hybrid power trains library&QUOT; submitted for publication at the 11th International Modelica Conference, 2015, September 21-23, Palais des congr&egrave;s de Versailles, 23-23 September, France</p>
</html>"));
end wbEHPTlib;

